@@ -21,11 +21,13 @@
 
 #include "converter.h"
 #include "genmeshify.h"
+#include "thing/persist/thingldr.h"
+#include "thing/object/thing.h"
 
 namespace genmeshify
 {
-  Converter::Converter (App* app, iLoaderContext* context, iRegion* region) : 
-    app (app), context (context), region (region)
+  Converter::Converter (App* app, iLoaderContext* context, iCollection* collection) : 
+    app (app), context (context), collection (collection)
   {
     gmfactSaver = csLoadPluginCheck<iSaverPlugin> (app->objectRegistry,
       "crystalspace.mesh.saver.factory.genmesh", true);
@@ -33,13 +35,18 @@ namespace genmeshify
       "crystalspace.mesh.saver.genmesh", true);
     if (gmfactSaver.IsValid() && gmSaver.IsValid())
     {
-      thingFactLoader = csLoadPluginCheck<iLoaderPlugin> (app->objectRegistry,
-        "crystalspace.mesh.loader.factory.thing", true);
-      thingObjLoader = csLoadPluginCheck<iLoaderPlugin> (app->objectRegistry,
-        "crystalspace.mesh.loader.thing", true);
+      csRef<iComponent> comp;
+
+      thingFactLoader.AttachNew (new csThingFactoryLoader (0));
+      comp = scfQueryInterface<iComponent> (thingFactLoader);
+      comp->Initialize (app->objectRegistry);
+
+      thingObjLoader.AttachNew (new csThingLoader (0));
+      comp = scfQueryInterface<iComponent> (thingObjLoader);
+      comp->Initialize (app->objectRegistry);
     }
 
-    idTexLightmap = app->strings->Request ("tex lightmap");
+    idTexLightmap = app->svStrings->Request ("tex lightmap");
   }
 
   bool Converter::ConvertMeshFact (const char* factoryName, 
@@ -72,17 +79,17 @@ namespace genmeshify
       csRef<iMeshFactoryWrapper> thingmfw = app->engine->CreateMeshFactory (
         factoryName);
       thingmfw->SetMeshObjectFactory (mfact);
-      region->QueryObject ()->ObjAdd (thingmfw->QueryObject());
+      collection->Add (thingmfw->QueryObject());
     }
 
     csRef<iMeshFactoryWrapper> mfw = app->engine->CreateMeshFactory (
-      "crystalspace.mesh.object.genmesh", 0);
+      "crystalspace.mesh.object.genmesh", 0, true);
     if (!mfw)
     {
       app->Report ("Could not create genmesh factory");
       return false;
     }
-    region->QueryObject ()->ObjAdd (mfw->QueryObject());
+    collection->Add (mfw->QueryObject());
     csRef<iMeshObjectFactory> mof = mfw->GetMeshObjectFactory();
     csRef<iGeneralFactoryState> gmfact = 
       scfQueryInterface<iGeneralFactoryState> (mof);
@@ -128,7 +135,7 @@ namespace genmeshify
     if (!paramsNode) return false;
 
     csRef<iMeshWrapper> meshWrap = app->engine->CreateMeshWrapper (meshName);
-    region->QueryObject ()->ObjAdd (meshWrap->QueryObject());
+    collection->Add (meshWrap->QueryObject());
 
     csRef<iBase> obj = thingObjLoader->Parse (paramsNode, 0, context, meshWrap);
     if (!obj) return false;
@@ -175,8 +182,12 @@ namespace genmeshify
       }
     }
     meshWrap->GetMovable()->SetSector (sector);
-    iLightingInfo* linfo = meshWrap->GetLightingInfo ();
-    if (linfo) linfo->ReadFromCache (app->engine->GetCacheManager());
+    iMeshObject* mo = meshWrap->GetMeshObject ();
+    csThing* t = static_cast<csThing*> (mo);
+    if (t)
+    {
+      t->ReadFromCache (app->engine->GetCacheManager());
+    }
 
     // @@@ FIXME: properly share factories, if possible
     csRef<iThingFactoryState> thingfact = 
@@ -213,7 +224,7 @@ namespace genmeshify
         app->Report ("Could not create genmesh factory");
         return false;
       }
-      region->QueryObject ()->ObjAdd (mfw->QueryObject());
+      collection->Add (mfw->QueryObject());
       mof = mfw->GetMeshObjectFactory();
       gmfact = scfQueryInterface<iGeneralFactoryState> (mof);
       if (!gmfact)
@@ -260,7 +271,6 @@ namespace genmeshify
     newObj->SetMixMode (mobj->GetMixMode ());
     csRef<iGeneralMeshState> gmObj = 
       scfQueryInterface<iGeneralMeshState> (newObj);
-    gmObj->SetShadowReceiving (false);
     /* Bit of a trick to make unlit polys work: if no lightmap is attached,
      * vertex lighting will be used. For this case set a manual color of
      * 1,1,1 to get fullbright. */
@@ -272,12 +282,12 @@ namespace genmeshify
     {
       const LMLayout::SubMesh& layoutSM = lmLayout.subMeshes[i];
       csRef<iTextureWrapper> dummyTex = 
-        app->engine->FindTexture (slmNames[layoutSM.slm], region);
+        app->engine->FindTexture (slmNames[layoutSM.slm], collection);
       if (!dummyTex.IsValid())
       {
         dummyTex = app->engine->CreateBlackTexture (
           slmNames[layoutSM.slm], 1, 1, 0, 0);
-        region->Add (dummyTex->QueryObject());
+        collection->Add (dummyTex->QueryObject());
       }
 
       iGeneralMeshSubMesh* submesh = 
@@ -868,7 +878,7 @@ namespace genmeshify
     csRef<iDocumentNode> meshNode = to->CreateNodeBefore (CS_NODE_ELEMENT, 0);
     meshNode->SetValue ("mesh");
 
-    int vc = triMesh->GetVertexCount();
+    int vc = (int)triMesh->GetVertexCount();
     const csVector3* vertices = triMesh->GetVertices ();
     for (int v = 0; v < vc; v++)
     {
@@ -878,7 +888,7 @@ namespace genmeshify
       if (!app->synsrv->WriteVector (vertNode, vertices[v])) return false;
     }
 
-    int tc = triMesh->GetTriangleCount ();
+    int tc = (int)triMesh->GetTriangleCount ();
     const csTriangle* tris = triMesh->GetTriangles ();
     for (int t = 0; t < tc; t++)
     {
