@@ -21,9 +21,7 @@
 #include "primitive.h"
 #include "object.h"
 
-#include "csutil/custom_new_disable.h"
 #include <algorithm>
-#include "csutil/custom_new_enable.h"
 
 namespace lighter
 {
@@ -85,12 +83,14 @@ namespace lighter
   void PrimitiveBase::ComputePlane ()
   {
     //Setup a temporary array of our vertices
-    Vector3DArray vertices;
-    vertices.Push (vertexData->positions[triangle.a]);
-    vertices.Push (vertexData->positions[triangle.b]);
-    vertices.Push (vertexData->positions[triangle.c]);
+    const csVector3 vertices[] =
+    {
+      vertexData->positions[triangle.a],
+      vertexData->positions[triangle.b],
+      vertexData->positions[triangle.c]
+    };
 
-    plane = csPoly3D::ComputePlane (vertices);
+    plane = csPoly3D::ComputePlane (vertices, 3);
   }
 
   template<typename Array>
