@@ -84,8 +84,8 @@ public:
 };
 
 /**
- * A structure used by TestRectangle() and initialized by
- * PrepareTestRectangle().
+ * A structure used by csTiledCoverageBuffer::TestRectangle() and initialized by
+ * csTiledCoverageBuffer::PrepareTestRectangle().
  */
 struct csTestRectData
 {
@@ -512,8 +512,9 @@ private:
    * polygon will be extended horizontally.
    * Returns false if polygon is outside screen.
    */
-  bool DrawPolygon (csVector2* verts, size_t num_verts, csBox2Int& bbox);
+  bool DrawPolygon (const csVector2* verts, size_t num_verts, csBox2Int& bbox);
 
+  //@{
   /**
    * Draw an outline on the coverage buffer.
    * Returns false if outline is outside screen.
@@ -525,6 +526,12 @@ private:
   	bool* used_verts,
   	int* edges, size_t num_edges, csBox2Int& bbox,
 	float& max_depth, bool splat_outline);
+  bool DrawOutline (const csReversibleTransform& trans,
+  	const CS::Math::Matrix4& projection, 
+  	csVector3* verts, size_t num_verts, bool* used_verts,
+  	int* edges, size_t num_edges, csBox2Int& bbox,
+	float& max_depth, bool splat_outline);
+  //@}
 
   /**
    * Find a given tile.
@@ -554,6 +561,13 @@ public:
 
   /// Setup coverage buffer for given size.
   void Setup (int w, int h);
+  
+  void SetSize (int w, int h)
+  {
+    if ((w > width_po2) || (h > height_64)) Setup (w, h);
+    width = w;
+    height = h;
+  }
 
   /// Initialize the coverage buffer to empty.
   void Initialize ();
@@ -572,7 +586,7 @@ public:
   /**
    * Insert an inverted polygon in the coverage buffer.
    */
-  void InsertPolygonInverted (csVector2* verts, size_t num_verts,
+  void InsertPolygonInverted (const csVector2* verts, size_t num_verts,
   	float max_depth);
 
   /**
@@ -581,7 +595,7 @@ public:
    * This function ignores depth in the depth buffer and should only
    * be used if you don't plan to use depth information nor depend on it.
    */
-  void InsertPolygonInvertedNoDepth (csVector2* verts, size_t num_verts);
+  void InsertPolygonInvertedNoDepth (const csVector2* verts, size_t num_verts);
 
   /**
    * Insert a polygon in the coverage buffer.
@@ -593,7 +607,7 @@ public:
    * <p>
    * If this function returns the number of tiles that were modified.
    */
-  int InsertPolygon (csVector2* verts, size_t num_verts, float max_depth,
+  int InsertPolygon (const csVector2* verts, size_t num_verts, float max_depth,
   	csBox2Int& modified_bbox);
 
   /**
@@ -607,8 +621,9 @@ public:
    * This function ignores depth in the depth buffer and should only
    * be used if you don't plan to use depth information nor depend on it.
    */
-  int InsertPolygonNoDepth (csVector2* verts, size_t num_verts);
+  int InsertPolygonNoDepth (const csVector2* verts, size_t num_verts);
 
+  //@{
   /**
    * Insert an outline in the coverage buffer.
    * It will update the screen buffer. 'bbox' will contain the screen
@@ -627,6 +642,12 @@ public:
   	bool* used_verts,
   	int* edges, size_t num_edges, bool splat_outline,
 	csBox2Int& modified_bbox);
+  int InsertOutline (const csReversibleTransform& trans,
+  	const CS::Math::Matrix4& projection, 
+  	csVector3* verts, size_t num_verts, bool* used_verts,
+  	int* edges, size_t num_edges, bool splat_outline,
+	csBox2Int& modified_bbox);
+  //@}
 
   /**
    * Prepare data for TestRectangle. If this returns false you have
