@@ -64,33 +64,33 @@
     return currentContext->enabled_##name; \
   }
 
-#define DECLARE_CACHED_BOOL_CURRENTLAYER(name) \
+#define DECLARE_CACHED_BOOL_IMAGEUNIT(name) \
   AutoArray<bool> enabled_##name;
 
-#define IMPLEMENT_CACHED_BOOL_CURRENTLAYER(name)                              \
+#define IMPLEMENT_CACHED_BOOL_IMAGEUNIT(name)                       \
   void Enable_##name ()                                                       \
   {                                                                           \
-    const int currentUnit = currentContext->currentUnit;                      \
+    const int currentUnit = currentContext->currentImageUnit;                 \
     if (!currentContext->enabled_##name[currentUnit] || FORCE_STATE_CHANGE)   \
     {                                                                         \
-      ActivateTU (activateTexEnable);                                         \
+      ActivateImageUnit ();                                                   \
       currentContext->enabled_##name[currentUnit] = true;                     \
       glEnable (name);                                                        \
     }                                                                         \
   }                                                                           \
   void Disable_##name ()                                                      \
   {                                                                           \
-    const int currentUnit = currentContext->currentUnit;                      \
+    const int currentUnit = currentContext->currentImageUnit;                 \
     if (currentContext->enabled_##name[currentUnit] || FORCE_STATE_CHANGE)    \
     {                                                                         \
-      ActivateTU (activateTexEnable);                                         \
+      ActivateImageUnit ();                                                   \
       currentContext->enabled_##name[currentUnit] = false;                    \
       glDisable (name);                                                       \
     }                                                                         \
   }                                                                           \
   bool IsEnabled_##name () const                                              \
   {                                                                           \
-    const int currentUnit = currentContext->currentUnit;                      \
+    const int currentUnit = currentContext->currentImageUnit;                 \
     return currentContext->enabled_##name[currentUnit];                       \
   }
 
@@ -290,54 +290,56 @@
     return currentContext->enabled_##name; \
   }
 
-#define DECLARE_CACHED_CLIENT_STATE_LAYER(name)	                              \
+#define DECLARE_CACHED_CLIENT_STATE_TCS(name)	                              \
   AutoArray<bool> enabled_##name;
 
-#define IMPLEMENT_CACHED_CLIENT_STATE_LAYER(name)	                      \
+#define IMPLEMENT_CACHED_CLIENT_STATE_TCS(name)                      \
   void Enable_##name ()                                                       \
   {                                                                           \
-    const int currentUnit = currentContext->currentUnit;                      \
+    const int currentUnit = currentContext->currentTCUnit;                    \
     if (!currentContext->enabled_##name[currentUnit] || FORCE_STATE_CHANGE)   \
     {                                                                         \
-      ActivateTU (activateTexCoord);                                          \
+      ActivateTCUnit (activateTexCoord);                                      \
       currentContext->enabled_##name[currentUnit] = true;                     \
       glEnableClientState (name);                                             \
     }                                                                         \
   }                                                                           \
   void Disable_##name ()                                                      \
   {                                                                           \
-    const int currentUnit = currentContext->currentUnit;                      \
+    const int currentUnit = currentContext->currentTCUnit;                    \
     if (currentContext->enabled_##name[currentUnit] || FORCE_STATE_CHANGE)    \
     {                                                                         \
-      ActivateTU (activateTexCoord);                                          \
+      ActivateTCUnit (activateTexCoord);                                      \
       currentContext->enabled_##name[currentUnit] = false;                    \
       glDisableClientState (name);                                            \
     }                                                                         \
   }                                                                           \
   bool IsEnabled_##name () const                                              \
   {                                                                           \
-    const int currentUnit = currentContext->currentUnit;                      \
+    const int currentUnit = currentContext->currentTCUnit;                    \
     return currentContext->enabled_##name[currentUnit];                       \
   }
 
 #define DECLARE_CACHED_PARAMETER_1_LAYER(func, name, type1, param1) \
   AutoArray<type1> parameter_##param1;
 
-#define IMPLEMENT_CACHED_PARAMETER_1_LAYER(func, name, type1, param1) \
+#define IMPLEMENT_CACHED_PARAMETER_1_LAYER(func, name, type1, param1, limit) \
   void Set##name (type1 param1, bool forced = false) \
   { \
+    const int currentUnit = currentContext->currentUnit;                      \
     if (forced || \
-        (param1 != currentContext->parameter_##param1[currentContext->currentUnit] \
+        (param1 != currentContext->parameter_##param1[currentUnit] \
         || FORCE_STATE_CHANGE)) \
     { \
       ActivateTU (); \
-      currentContext->parameter_##param1[currentContext->currentUnit] = param1;  \
+      currentContext->parameter_##param1[currentUnit] = param1;  \
       func (param1); \
     } \
   } \
   void Get##name (type1 &param1) const\
   { \
-    param1 = currentContext->parameter_##param1[currentContext->currentUnit];  \
+    const int currentUnit = currentContext->currentUnit;                      \
+    param1 = currentContext->parameter_##param1[currentUnit];  \
   }
 
 #define DECLARE_CACHED_PARAMETER_2_LAYER(func, name, type1, param1, \
@@ -346,25 +348,27 @@
   AutoArray<type2> parameter_##param2;
 
 #define IMPLEMENT_CACHED_PARAMETER_2_LAYER(func, name, type1, param1, \
-  type2, param2) \
+  type2, param2, limit) \
   void Set##name (type1 param1, type2 param2, bool forced = false) \
   { \
+    const int currentUnit = currentContext->currentUnit;                      \
     if (forced || (param1 != currentContext->parameter_##param1[ \
-                    currentContext->currentUnit]) || \
+                    currentUnit]) || \
                   (param2 != currentContext->parameter_##param2[ \
-                    currentContext->currentUnit]) \
+                    currentUnit]) \
                || FORCE_STATE_CHANGE) \
     { \
       ActivateTU (); \
-      currentContext->parameter_##param1[currentContext->currentUnit] = param1;  \
-      currentContext->parameter_##param2[currentContext->currentUnit] = param2;  \
+      currentContext->parameter_##param1[currentUnit] = param1;  \
+      currentContext->parameter_##param2[currentUnit] = param2;  \
       func (param1, param2); \
     } \
   } \
   void Get##name (type1 &param1, type2 & param2) const\
   { \
-    param1 = currentContext->parameter_##param1[currentContext->currentUnit];  \
-    param2 = currentContext->parameter_##param2[currentContext->currentUnit];  \
+    const int currentUnit = currentContext->currentUnit;                      \
+    param1 = currentContext->parameter_##param1[currentUnit];  \
+    param2 = currentContext->parameter_##param2[currentUnit];  \
   }
 
 #define DECLARE_CACHED_PARAMETER_3_LAYER(func, name, type1, param1, \
@@ -375,34 +379,36 @@
 
 
 #define IMPLEMENT_CACHED_PARAMETER_3_LAYER(func, name, type1, param1, \
-  type2, param2, type3, param3) \
+  type2, param2, type3, param3, limit) \
   void Set##name (type1 param1, type2 param2, type3 param3,\
     bool forced = false) \
   { \
+    const int currentUnit = currentContext->currentUnit;                      \
     if (forced || (param1 != currentContext->parameter_##param1[ \
-                    currentContext->currentUnit]) || \
+                    currentUnit]) || \
                   (param2 != currentContext->parameter_##param2[ \
-                    currentContext->currentUnit]) || \
+                    currentUnit]) || \
                   (param3 != currentContext->parameter_##param3[ \
-                    currentContext->currentUnit]) \
+                    currentUnit]) \
                || FORCE_STATE_CHANGE) \
     { \
       ActivateTU (); \
-      currentContext->parameter_##param1[currentContext->currentUnit] = param1;  \
-      currentContext->parameter_##param2[currentContext->currentUnit] = param2;  \
-      currentContext->parameter_##param3[currentContext->currentUnit] = param3;  \
+      currentContext->parameter_##param1[currentUnit] = param1;  \
+      currentContext->parameter_##param2[currentUnit] = param2;  \
+      currentContext->parameter_##param3[currentUnit] = param3;  \
       func (param1, param2, param3); \
     } \
   } \
   void Get##name (type1 &param1, type2 & param2, type3 & param3) const\
   { \
-    param1 = currentContext->parameter_##param1[currentContext->currentUnit];  \
-    param2 = currentContext->parameter_##param2[currentContext->currentUnit];  \
-    param3 = currentContext->parameter_##param3[currentContext->currentUnit];  \
+    const int currentUnit = currentContext->currentUnit;                      \
+    param1 = currentContext->parameter_##param1[currentUnit];  \
+    param2 = currentContext->parameter_##param2[currentUnit];  \
+    param3 = currentContext->parameter_##param3[currentUnit];  \
   }
 
 
-#define DECLARE_CACHED_PARAMETER_4_BUF_LAYER(func, name, type1, param1,       \
+#define DECLARE_CACHED_PARAMETER_4_BUF_TCS(func, name, type1, param1,       \
   type2, param2, type3, param3, type4, param4, vbo)                           \
   AutoArray<type1> parameter_##param1;                                        \
   AutoArray<type2> parameter_##param2;                                        \
@@ -410,12 +416,12 @@
   AutoArray<type4> parameter_##param4;                                        \
   AutoArray<GLuint> parameter_##vbo;
 
-#define IMPLEMENT_CACHED_PARAMETER_4_BUF_LAYER(func, name, type1, param1,     \
-    type2, param2, type3, param3, type4, param4, vbo)                         \
+#define IMPLEMENT_CACHED_PARAMETER_4_BUF_TCS(func, name, type1, param1,     \
+    type2, param2, type3, param3, type4, param4, vbo)                  \
   void Set##name (type1 param1, type2 param2, type3 param3, type4 param4,     \
     bool forced = false)                                                      \
   {                                                                           \
-    const int currentUnit = currentContext->currentUnit;                      \
+    const int currentUnit = currentContext->currentTCUnit;                    \
     if (forced                                                                \
       || (param1 != currentContext->parameter_##param1[currentUnit])          \
       || (param2 != currentContext->parameter_##param2[currentUnit])          \
@@ -425,7 +431,7 @@
         != currentContext->parameter_##vbo[currentUnit])                      \
       || FORCE_STATE_CHANGE)                                                  \
     {                                                                         \
-      ActivateTU (activateTexCoord);                                          \
+      ActivateTCUnit (activateTexCoord);                                      \
       currentContext->parameter_##param1[currentUnit] = param1;               \
       currentContext->parameter_##param2[currentUnit] = param2;               \
       currentContext->parameter_##param3[currentUnit] = param3;               \
@@ -442,7 +448,7 @@
   void Get##name (type1 &param1, type2 & param2, type3 & param3,              \
     type4& param4) const                                                      \
   {                                                                           \
-    const int currentUnit = currentContext->currentUnit;                      \
+    const int currentUnit = currentContext->currentTCUnit;                    \
     param1 = currentContext->parameter_##param1[currentUnit];                 \
     param2 = currentContext->parameter_##param2[currentUnit];                 \
     param3 = currentContext->parameter_##param3[currentUnit];                 \
@@ -450,7 +456,7 @@
   }
 
 
-class csGLStateCacheContext
+class CS_CSPLUGINCOMMON_GL_EXPORT csGLStateCacheContext
 {
   template<typename T>
   struct AutoArray
@@ -480,7 +486,7 @@ public:
   AutoArray<GLuint> boundtexture;
   GLint numImageUnits;
   GLint numTexCoords;
-  int currentUnit;
+  int currentImageUnit, currentTCUnit;
   int activeUnit[2];
   enum
   {
@@ -518,6 +524,24 @@ public:
   // Pixel storage
   GLint pixelUnpackAlignment;
   bool pixelUnpackSwapBytes;
+  
+  // Color clamp control
+  enum
+  {
+    clampVertex = 0, clampFragment = 1, clampRead = 2,
+    clampCount
+  };
+  GLenum clampState[clampCount];
+  static int GLClampTargetToCacheIndex (GLenum target)
+  {
+    switch (target)
+    {
+    case GL_CLAMP_VERTEX_COLOR_ARB:         return clampVertex;
+    case GL_CLAMP_FRAGMENT_COLOR_ARB:         return clampFragment;
+    case GL_CLAMP_READ_COLOR_ARB:         return clampRead;
+    default: return -1;      
+    }
+  }
 
   // Standardized caches
   DECLARE_CACHED_BOOL (GL_DEPTH_TEST)
@@ -537,11 +561,14 @@ public:
   DECLARE_CACHED_BOOL (GL_COLOR_SUM_EXT)
   DECLARE_CACHED_BOOL (GL_VERTEX_PROGRAM_POINT_SIZE_ARB)
   DECLARE_CACHED_BOOL (GL_POINT_SPRITE_ARB)
-  DECLARE_CACHED_BOOL_CURRENTLAYER (GL_TEXTURE_1D)
-  DECLARE_CACHED_BOOL_CURRENTLAYER (GL_TEXTURE_2D)
-  DECLARE_CACHED_BOOL_CURRENTLAYER (GL_TEXTURE_3D)
-  DECLARE_CACHED_BOOL_CURRENTLAYER (GL_TEXTURE_CUBE_MAP)
-  DECLARE_CACHED_BOOL_CURRENTLAYER (GL_TEXTURE_RECTANGLE_ARB)
+  DECLARE_CACHED_BOOL (GL_TEXTURE_CUBE_MAP_SEAMLESS)
+  DECLARE_CACHED_BOOL (GL_SAMPLE_ALPHA_TO_COVERAGE_ARB)
+  DECLARE_CACHED_BOOL (GL_SAMPLE_ALPHA_TO_ONE_ARB)
+  DECLARE_CACHED_BOOL_IMAGEUNIT (GL_TEXTURE_1D)
+  DECLARE_CACHED_BOOL_IMAGEUNIT (GL_TEXTURE_2D)
+  DECLARE_CACHED_BOOL_IMAGEUNIT (GL_TEXTURE_3D)
+  DECLARE_CACHED_BOOL_IMAGEUNIT (GL_TEXTURE_CUBE_MAP)
+  DECLARE_CACHED_BOOL_IMAGEUNIT (GL_TEXTURE_RECTANGLE_ARB)
   DECLARE_CACHED_PARAMETER_2 (glAlphaFunc, AlphaFunc, GLenum, alpha_func, GLclampf, alpha_ref)
   DECLARE_CACHED_PARAMETER_1 (glCullFace, CullFace, GLenum, cull_mode)
   DECLARE_CACHED_PARAMETER_1 (glDepthFunc, DepthFunc, GLenum, depth_func)
@@ -557,7 +584,7 @@ public:
   DECLARE_CACHED_CLIENT_STATE (GL_COLOR_ARRAY)
   DECLARE_CACHED_CLIENT_STATE (GL_SECONDARY_COLOR_ARRAY_EXT)
   DECLARE_CACHED_CLIENT_STATE (GL_NORMAL_ARRAY)
-  DECLARE_CACHED_CLIENT_STATE_LAYER (GL_TEXTURE_COORD_ARRAY)
+  DECLARE_CACHED_CLIENT_STATE_TCS (GL_TEXTURE_COORD_ARRAY)
 
   DECLARE_CACHED_PARAMETER_1 (glMatrixMode, MatrixMode, GLenum, matrixMode)
   
@@ -570,262 +597,16 @@ public:
   DECLARE_CACHED_PARAMETER_4_BUF (extmgr->glSecondaryColorPointerEXT, 
     SecondaryColorPointerEXT, GLint, scsize, GLenum, sctype, GLsizei, scstride, 
     GLvoid*, scpointer, scvbo);
-  DECLARE_CACHED_PARAMETER_4_BUF_LAYER (glTexCoordPointer, TexCoordPointer, GLint, tsize,
+  DECLARE_CACHED_PARAMETER_4_BUF_TCS (glTexCoordPointer, TexCoordPointer, GLint, tsize,
     GLenum, ttype, GLsizei, tstride, GLvoid*, tpointer, tvbo)
   
-  csGLStateCacheContext (csGLExtensionManager* extmgr)
-  {
-    csGLStateCacheContext::extmgr = extmgr;
-  
-    // Need to init exts here b/c we need the image units count
-    extmgr->InitGL_ARB_multitexture ();
-    extmgr->InitGL_ARB_fragment_program ();
-    
-    if (extmgr->CS_GL_ARB_fragment_program)
-    {
-      glGetIntegerv (GL_MAX_TEXTURE_IMAGE_UNITS_ARB, &numImageUnits);
-      glGetIntegerv (GL_MAX_TEXTURE_COORDS_ARB, &numTexCoords);
-    }
-    else if (extmgr->CS_GL_ARB_multitexture)
-    {
-      glGetIntegerv (GL_MAX_TEXTURE_UNITS_ARB, &numImageUnits);
-      numTexCoords = numImageUnits;
-    }
-    else
-    {
-      numTexCoords = numImageUnits = 1;
-    }
-    
-    boundtexture.Setup (numImageUnits);
-    enabled_GL_TEXTURE_1D.Setup (numImageUnits);
-    enabled_GL_TEXTURE_2D.Setup (numImageUnits);
-    enabled_GL_TEXTURE_3D.Setup (numImageUnits);
-    enabled_GL_TEXTURE_CUBE_MAP.Setup (numImageUnits);
-    enabled_GL_TEXTURE_RECTANGLE_ARB.Setup (numImageUnits);
-    
-    enabled_GL_TEXTURE_COORD_ARRAY.Setup (numTexCoords);
-    parameter_tsize.Setup (numTexCoords);
-    parameter_ttype.Setup (numTexCoords);
-    parameter_tstride.Setup (numTexCoords);
-    parameter_tpointer.Setup (numTexCoords);
-    parameter_tvbo.Setup (numTexCoords);
-    
-    memset (activeBufferID, 0, sizeof (activeBufferID));
-  }
+  csGLStateCacheContext (csGLExtensionManager* extmgr);
 
   /** 
    * Init cache. Does both retrieval of current GL state as well as setting
    * some states to known values.
    */
-  void InitCache()
-  {
-    int i;
-    glGetIntegerv (GL_ALPHA_TEST_FUNC, (GLint*)&parameter_alpha_func);
-    glGetFloatv (GL_ALPHA_TEST_REF, &parameter_alpha_ref);
-    if (extmgr->CS_GL_EXT_blend_func_separate)
-    {
-      glGetIntegerv (GL_BLEND_SRC_RGB_EXT, (GLint*)&blend_sourceRGB);
-      glGetIntegerv (GL_BLEND_SRC_ALPHA_EXT, (GLint*)&blend_sourceA);
-      glGetIntegerv (GL_BLEND_DST_RGB_EXT, (GLint*)&blend_destinationRGB);
-      glGetIntegerv (GL_BLEND_DST_ALPHA_EXT, (GLint*)&blend_destinationA);
-    }
-    else
-    {
-      glGetIntegerv (GL_BLEND_SRC, (GLint*)&blend_sourceRGB);
-      blend_sourceA = blend_sourceRGB;
-      glGetIntegerv (GL_BLEND_DST, (GLint*)&blend_destinationRGB);
-      blend_destinationA = blend_destinationRGB;
-    }
-    glGetIntegerv (GL_CULL_FACE_MODE, (GLint*)&parameter_cull_mode);
-    glGetIntegerv (GL_DEPTH_FUNC, (GLint*)&parameter_depth_func);
-    glGetBooleanv (GL_DEPTH_WRITEMASK, &parameter_depth_mask);
-    glGetIntegerv (GL_SHADE_MODEL, (GLint*)&parameter_shade_model);
-    glGetIntegerv (GL_STENCIL_BITS, (GLint*)&parameter_maskl);
-    glGetIntegerv (GL_STENCIL_FUNC, (GLint*)&parameter_stencil_func);
-    glGetIntegerv (GL_STENCIL_VALUE_MASK, (GLint*)&parameter_stencil_mask);
-    glGetIntegerv (GL_STENCIL_REF, &parameter_stencil_ref);
-    glGetIntegerv (GL_STENCIL_FAIL, (GLint*)&parameter_stencil_fail);
-    glGetIntegerv (GL_STENCIL_PASS_DEPTH_FAIL, (GLint*)&parameter_stencil_zfail);
-    glGetIntegerv (GL_STENCIL_PASS_DEPTH_PASS, (GLint*)&parameter_stencil_zpass);
-    glGetIntegerv (GL_MATRIX_MODE, (GLint*)&parameter_matrixMode);
-    GLboolean writemask[4];
-    glGetBooleanv (GL_COLOR_WRITEMASK, writemask);
-    parameter_wmRed = writemask[0];
-    parameter_wmGreen = writemask[1];
-    parameter_wmBlue = writemask[2];
-    parameter_wmAlpha = writemask[3];
-    enabled_GL_DEPTH_TEST = (glIsEnabled (GL_DEPTH_TEST) == GL_TRUE);
-    enabled_GL_BLEND = (glIsEnabled (GL_BLEND) == GL_TRUE);
-    enabled_GL_DITHER = (glIsEnabled (GL_DITHER) == GL_TRUE);
-    enabled_GL_STENCIL_TEST = (glIsEnabled (GL_STENCIL_TEST) == GL_TRUE);
-    enabled_GL_CULL_FACE = (glIsEnabled (GL_CULL_FACE) == GL_TRUE);
-    enabled_GL_POLYGON_OFFSET_FILL = (glIsEnabled (GL_POLYGON_OFFSET_FILL) == GL_TRUE);
-    enabled_GL_LIGHTING = (glIsEnabled (GL_LIGHTING) == GL_TRUE);
-    enabled_GL_ALPHA_TEST = (glIsEnabled (GL_ALPHA_TEST) == GL_TRUE);
-    enabled_GL_TEXTURE_GEN_S = (glIsEnabled (GL_TEXTURE_GEN_S) == GL_TRUE);
-    enabled_GL_TEXTURE_GEN_T = (glIsEnabled (GL_TEXTURE_GEN_T) == GL_TRUE);
-    enabled_GL_TEXTURE_GEN_R = (glIsEnabled (GL_TEXTURE_GEN_R) == GL_TRUE);
-    enabled_GL_TEXTURE_GEN_Q = (glIsEnabled (GL_TEXTURE_GEN_Q) == GL_TRUE);
-    enabled_GL_FOG = (glIsEnabled (GL_FOG) == GL_TRUE);
-
-    memset (boundtexture.p, 0, numImageUnits * sizeof (GLuint));
-    currentUnit = 0;
-    memset (activeUnit, 0, sizeof (activeUnit));
-    if (extmgr->CS_GL_ARB_multitexture)
-    {
-      for (i = numImageUnits; i-- > 0; )
-      {
-        extmgr->glActiveTextureARB (GL_TEXTURE0_ARB + i);
-        enabled_GL_TEXTURE_1D[i] = (glIsEnabled (GL_TEXTURE_1D) == GL_TRUE);
-        enabled_GL_TEXTURE_2D[i] = (glIsEnabled (GL_TEXTURE_2D) == GL_TRUE);
-        enabled_GL_TEXTURE_3D[i] = (glIsEnabled (GL_TEXTURE_3D) == GL_TRUE);
-        enabled_GL_TEXTURE_CUBE_MAP[i] = (glIsEnabled (GL_TEXTURE_CUBE_MAP) == GL_TRUE);
-	if (extmgr->CS_GL_ARB_texture_rectangle
-	  || extmgr->CS_GL_EXT_texture_rectangle
-	  || extmgr->CS_GL_NV_texture_rectangle)
-	  enabled_GL_TEXTURE_RECTANGLE_ARB[i] = (glIsEnabled (GL_TEXTURE_RECTANGLE_ARB) == GL_TRUE);
-	else
-	  enabled_GL_TEXTURE_RECTANGLE_ARB[i] = false;
-      }
-      for (i = numTexCoords; i-- > 0; )
-      {
-        extmgr->glClientActiveTextureARB (GL_TEXTURE0_ARB + i);
-        enabled_GL_TEXTURE_COORD_ARRAY[i] = (glIsEnabled (GL_TEXTURE_COORD_ARRAY) == GL_TRUE);
-        glGetIntegerv (GL_TEXTURE_COORD_ARRAY_SIZE, (GLint*)&parameter_tsize[i]);
-        glGetIntegerv (GL_TEXTURE_COORD_ARRAY_STRIDE, (GLint*)&parameter_tstride[i]);
-        glGetIntegerv (GL_TEXTURE_COORD_ARRAY_TYPE, (GLint*)&parameter_ttype[i]);
-        glGetPointerv (GL_TEXTURE_COORD_ARRAY_POINTER, &parameter_tpointer[i]);
-	if (extmgr->CS_GL_ARB_vertex_buffer_object)
-	  glGetIntegerv (GL_TEXTURE_COORD_ARRAY_BUFFER_BINDING_ARB,
-	    (GLint*)&parameter_tvbo[i]);
-      }
-    } 
-    else 
-    {
-      enabled_GL_TEXTURE_1D[0] = (glIsEnabled (GL_TEXTURE_1D) == GL_TRUE);
-      enabled_GL_TEXTURE_2D[0] = (glIsEnabled (GL_TEXTURE_2D) == GL_TRUE);
-      enabled_GL_TEXTURE_3D[0] = (glIsEnabled (GL_TEXTURE_3D) == GL_TRUE);
-      enabled_GL_TEXTURE_CUBE_MAP[0] = (glIsEnabled (GL_TEXTURE_CUBE_MAP) == GL_TRUE);
-      enabled_GL_TEXTURE_COORD_ARRAY[0] = (glIsEnabled (GL_TEXTURE_COORD_ARRAY) == GL_TRUE);
-	if (extmgr->CS_GL_ARB_texture_rectangle
-	  || extmgr->CS_GL_EXT_texture_rectangle
-	  || extmgr->CS_GL_NV_texture_rectangle)
-	  enabled_GL_TEXTURE_RECTANGLE_ARB[0] = (glIsEnabled (GL_TEXTURE_RECTANGLE_ARB) == GL_TRUE);
-	else
-	  enabled_GL_TEXTURE_RECTANGLE_ARB[0] = false;
-      glGetIntegerv (GL_TEXTURE_COORD_ARRAY_SIZE, (GLint*)&parameter_tsize[0]);
-      glGetIntegerv (GL_TEXTURE_COORD_ARRAY_STRIDE, (GLint*)&parameter_tstride[0]);
-      glGetIntegerv (GL_TEXTURE_COORD_ARRAY_TYPE, (GLint*)&parameter_ttype[0]);
-      glGetPointerv (GL_TEXTURE_COORD_ARRAY_POINTER, &parameter_tpointer[0]);
-      if (extmgr->CS_GL_ARB_vertex_buffer_object)
-	glGetIntegerv (GL_TEXTURE_COORD_ARRAY_BUFFER_BINDING_ARB,
-	  (GLint*)&parameter_tvbo[0]);
-      for (i = 1 ; i < numImageUnits; i++)
-      {
-        enabled_GL_TEXTURE_1D[i] = enabled_GL_TEXTURE_1D[0];
-        enabled_GL_TEXTURE_2D[i] = enabled_GL_TEXTURE_2D[0];
-        enabled_GL_TEXTURE_3D[i] = enabled_GL_TEXTURE_3D[0];
-        enabled_GL_TEXTURE_CUBE_MAP[i] = enabled_GL_TEXTURE_CUBE_MAP[0];
-        enabled_GL_TEXTURE_COORD_ARRAY[i] = enabled_GL_TEXTURE_COORD_ARRAY[0];
-	enabled_GL_TEXTURE_RECTANGLE_ARB[i] = enabled_GL_TEXTURE_RECTANGLE_ARB[0];
-	parameter_tsize[i] = parameter_tsize[0];
-	parameter_tstride[i] = parameter_tstride[0];
-	parameter_ttype[i] = parameter_ttype[0];
-	parameter_tpointer[i] = parameter_tpointer[0];
-      }
-    }
-    enabled_GL_SCISSOR_TEST = (glIsEnabled (GL_SCISSOR_TEST) == GL_TRUE);
-    enabled_GL_VERTEX_ARRAY = (glIsEnabled (GL_VERTEX_ARRAY) == GL_TRUE);
-    enabled_GL_COLOR_ARRAY = (glIsEnabled (GL_COLOR_ARRAY) == GL_TRUE);
-    if (extmgr->CS_GL_EXT_secondary_color)
-      enabled_GL_SECONDARY_COLOR_ARRAY_EXT = 
-        (glIsEnabled (GL_SECONDARY_COLOR_ARRAY_EXT) == GL_TRUE);
-    else
-      enabled_GL_SECONDARY_COLOR_ARRAY_EXT = false;
-    enabled_GL_NORMAL_ARRAY = (glIsEnabled (GL_NORMAL_ARRAY) == GL_TRUE);
-    
-    if (extmgr->CS_GL_ARB_vertex_program)
-      enabled_GL_VERTEX_PROGRAM_POINT_SIZE_ARB =
-        (glIsEnabled (GL_VERTEX_PROGRAM_POINT_SIZE_ARB) == GL_TRUE);
-    else
-      enabled_GL_VERTEX_PROGRAM_POINT_SIZE_ARB = false;
-    if (extmgr->CS_GL_ARB_point_sprite)
-      enabled_GL_POINT_SPRITE_ARB =
-        (glIsEnabled (GL_POINT_SPRITE_ARB) == GL_TRUE);
-    else
-      enabled_GL_POINT_SPRITE_ARB = false;
-
-    memset (currentBufferID, 0, sizeof (currentBufferID));
-    {
-      enum { extVBO = 1, extPBO = 2 };
-      static const GLenum requiredExt[boCount] =
-      { extVBO, extVBO, extPBO, extPBO };
-      
-      int boExt = 0;
-      if (extmgr->CS_GL_ARB_vertex_buffer_object) boExt |= extVBO;
-      if (extmgr->CS_GL_ARB_pixel_buffer_object) boExt |= extPBO;
-      for (int b = 0; b < boCount; b++)
-      {
-	if (requiredExt[b] & boExt)
-	{
-	  static const GLenum localIndexToGLBufferBinding[boCount] =
-	  { GL_ARRAY_BUFFER_BINDING_ARB, GL_ELEMENT_ARRAY_BUFFER_BINDING_ARB, 
-	    GL_PIXEL_PACK_BUFFER_BINDING_ARB, GL_PIXEL_UNPACK_BUFFER_BINDING_ARB };
-	  glGetIntegerv (localIndexToGLBufferBinding[b], 
-	    (GLint*)&activeBufferID[b]);
-	}
-      }
-    }
-
-    glGetIntegerv (GL_VERTEX_ARRAY_SIZE, (GLint*)&parameter_vsize);
-    glGetIntegerv (GL_VERTEX_ARRAY_STRIDE, (GLint*)&parameter_vstride);
-    glGetIntegerv (GL_VERTEX_ARRAY_TYPE, (GLint*)&parameter_vtype);
-    glGetPointerv (GL_VERTEX_ARRAY_POINTER, &parameter_vpointer);
-    if (extmgr->CS_GL_ARB_vertex_buffer_object)
-      glGetIntegerv (GL_VERTEX_ARRAY_BUFFER_BINDING_ARB, (GLint*)&parameter_vvbo);
-
-    glGetIntegerv (GL_NORMAL_ARRAY_STRIDE, (GLint*)&parameter_nstride);
-    glGetIntegerv (GL_NORMAL_ARRAY_TYPE, (GLint*)&parameter_ntype);
-    glGetPointerv (GL_NORMAL_ARRAY_POINTER, &parameter_npointer);
-    if (extmgr->CS_GL_ARB_vertex_buffer_object)
-      glGetIntegerv (GL_NORMAL_ARRAY_BUFFER_BINDING_ARB, (GLint*)&parameter_nvbo);
-
-    glGetIntegerv (GL_COLOR_ARRAY_SIZE, (GLint*)&parameter_csize);
-    glGetIntegerv (GL_COLOR_ARRAY_STRIDE, (GLint*)&parameter_cstride);
-    glGetIntegerv (GL_COLOR_ARRAY_TYPE, (GLint*)&parameter_ctype);
-    glGetPointerv (GL_COLOR_ARRAY_POINTER, &parameter_cpointer);
-    if (extmgr->CS_GL_ARB_vertex_buffer_object)
-      glGetIntegerv (GL_COLOR_ARRAY_BUFFER_BINDING_ARB, (GLint*)&parameter_cvbo);
-    
-    if (extmgr->CS_GL_EXT_secondary_color)
-    {
-      glGetIntegerv (GL_SECONDARY_COLOR_ARRAY_SIZE_EXT, 
-        (GLint*)&parameter_scsize);
-      glGetIntegerv (GL_SECONDARY_COLOR_ARRAY_STRIDE_EXT, 
-        (GLint*)&parameter_scstride);
-      glGetIntegerv (GL_SECONDARY_COLOR_ARRAY_TYPE_EXT, 
-        (GLint*)&parameter_sctype);
-      glGetPointerv (GL_SECONDARY_COLOR_ARRAY_POINTER_EXT, 
-        &parameter_scpointer);
-      if (extmgr->CS_GL_ARB_vertex_buffer_object)
-	glGetIntegerv (GL_SECONDARY_COLOR_ARRAY_BUFFER_BINDING_ARB,
-	  (GLint*)&parameter_scvbo);
-      enabled_GL_COLOR_SUM_EXT = glIsEnabled (GL_COLOR_SUM_EXT) != GL_FALSE;
-    }
-    else
-    {
-      parameter_scsize = 0;
-      parameter_scstride = 0;
-      parameter_sctype = 0;
-      parameter_scpointer = 0;
-      enabled_GL_COLOR_SUM_EXT = false;
-    }
-    
-    glGetIntegerv (GL_UNPACK_ALIGNMENT, &pixelUnpackAlignment);
-    GLint v;
-    glGetIntegerv (GL_UNPACK_SWAP_BYTES, &v);
-    pixelUnpackSwapBytes = v != 0;
-  }
+  void InitCache();
 };
 
 
@@ -879,11 +660,14 @@ public:
   IMPLEMENT_CACHED_BOOL (GL_COLOR_SUM_EXT)
   IMPLEMENT_CACHED_BOOL (GL_VERTEX_PROGRAM_POINT_SIZE_ARB)
   IMPLEMENT_CACHED_BOOL (GL_POINT_SPRITE_ARB)
-  IMPLEMENT_CACHED_BOOL_CURRENTLAYER (GL_TEXTURE_1D)
-  IMPLEMENT_CACHED_BOOL_CURRENTLAYER (GL_TEXTURE_2D)
-  IMPLEMENT_CACHED_BOOL_CURRENTLAYER (GL_TEXTURE_3D)
-  IMPLEMENT_CACHED_BOOL_CURRENTLAYER (GL_TEXTURE_CUBE_MAP)
-  IMPLEMENT_CACHED_BOOL_CURRENTLAYER (GL_TEXTURE_RECTANGLE_ARB)
+  IMPLEMENT_CACHED_BOOL (GL_TEXTURE_CUBE_MAP_SEAMLESS)
+  IMPLEMENT_CACHED_BOOL (GL_SAMPLE_ALPHA_TO_COVERAGE_ARB)
+  IMPLEMENT_CACHED_BOOL (GL_SAMPLE_ALPHA_TO_ONE_ARB)
+  IMPLEMENT_CACHED_BOOL_IMAGEUNIT (GL_TEXTURE_1D)
+  IMPLEMENT_CACHED_BOOL_IMAGEUNIT (GL_TEXTURE_2D)
+  IMPLEMENT_CACHED_BOOL_IMAGEUNIT (GL_TEXTURE_3D)
+  IMPLEMENT_CACHED_BOOL_IMAGEUNIT (GL_TEXTURE_CUBE_MAP)
+  IMPLEMENT_CACHED_BOOL_IMAGEUNIT (GL_TEXTURE_RECTANGLE_ARB)
   IMPLEMENT_CACHED_PARAMETER_2 (glAlphaFunc, AlphaFunc, GLenum, alpha_func, GLclampf, alpha_ref)
   IMPLEMENT_CACHED_PARAMETER_1 (glCullFace, CullFace, GLenum, cull_mode)
   IMPLEMENT_CACHED_PARAMETER_1 (glDepthFunc, DepthFunc, GLenum, depth_func)
@@ -899,7 +683,7 @@ public:
   IMPLEMENT_CACHED_CLIENT_STATE (GL_COLOR_ARRAY)
   IMPLEMENT_CACHED_CLIENT_STATE (GL_SECONDARY_COLOR_ARRAY_EXT)
   IMPLEMENT_CACHED_CLIENT_STATE (GL_NORMAL_ARRAY)
-  IMPLEMENT_CACHED_CLIENT_STATE_LAYER (GL_TEXTURE_COORD_ARRAY)
+  IMPLEMENT_CACHED_CLIENT_STATE_TCS (GL_TEXTURE_COORD_ARRAY)
 
   IMPLEMENT_CACHED_PARAMETER_1 (glMatrixMode, MatrixMode, GLenum, matrixMode)
   
@@ -912,47 +696,82 @@ public:
   IMPLEMENT_CACHED_PARAMETER_4_BUF (extmgr->glSecondaryColorPointerEXT, 
     SecondaryColorPointerExt, GLint, scsize, GLenum, sctype, GLsizei, scstride, 
     GLvoid*, scpointer, scvbo);
-  IMPLEMENT_CACHED_PARAMETER_4_BUF_LAYER (glTexCoordPointer, TexCoordPointer, GLint, tsize,
+  IMPLEMENT_CACHED_PARAMETER_4_BUF_TCS (glTexCoordPointer, TexCoordPointer, GLint, tsize,
     GLenum, ttype, GLsizei, tstride, GLvoid*, tpointer, tvbo);
   
   // Special caches
   void SetTexture (GLenum target, GLuint texture)
   {
-    const int currentUnit = currentContext->currentUnit;
+    const int currentUnit = currentContext->currentImageUnit;
     if (texture != currentContext->boundtexture[currentUnit])
     {
-      ActivateTU (activateImage);
+      ActivateImageUnit ();
       currentContext->boundtexture[currentUnit] = texture;
       glBindTexture (target, texture);
     }
   }
   GLuint GetTexture (GLenum /*target*/)
   {
-    const int currentUnit = currentContext->currentUnit;
+    const int currentUnit = currentContext->currentImageUnit;
     return currentContext->boundtexture[currentUnit];
   }
   GLuint GetTexture (GLenum /*target*/, int unit)
   {
     return currentContext->boundtexture[unit];
   }
+  
   /**
-   * Select the currently active texture unit. 
+   * Select the currently active image unit. 
    * \remarks Doesn't check whether the  multitexture extension is actually 
    * supported, this must be done in calling code.
    */
-  void SetCurrentTU (int unit)
+  void SetCurrentImageUnit (int unit)
   {
-    currentContext->currentUnit = unit;   
+    CS_ASSERT(unit < currentContext->numImageUnits);
+    currentContext->currentImageUnit = unit;   
   }
-  /// Query active texture unit.
-  int GetCurrentTU ()
+  /// Query active image unit.
+  int GetCurrentImageUnit ()
   {
-    return currentContext->currentUnit;
+    return currentContext->currentImageUnit;
+  }
+  /**
+   * Activate the currently selected image unit. 
+   * Use this to bind textures.
+   * \remarks Doesn't check whether the  multitexture extension is actually 
+   *   supported, this must be done in calling code.
+   */
+  void ActivateImageUnit ()
+  {
+    int currentUnit = currentContext->currentImageUnit;
+    if (currentContext->activeUnit[texServer] != currentUnit)
+    {
+      GLuint tu = GL_TEXTURE0_ARB + currentUnit;
+      extmgr->glActiveTextureARB (tu);
+      currentContext->activeUnit[texServer] = currentUnit;
+    }
+  }
+  
+  /**
+   * Select the currently active texture coordinate unit. 
+   * Use this to change state of texture coordinate arrays, 
+   * change texture matrices, texture environments,
+   * texture coord generation and enabling/disabling textures.
+   * \remarks Doesn't check whether the  multitexture extension is actually 
+   * supported, this must be done in calling code.
+   */
+  void SetCurrentTCUnit (int unit)
+  {
+    CS_ASSERT(unit < currentContext->numTexCoords);
+    currentContext->currentTCUnit = unit;   
+  }
+  /// Query active texture coordinate set.
+  int GetCurrentTCUnit ()
+  {
+    return currentContext->currentTCUnit;
   }
   /// Flag that the active TU should be used for setting texture coords
   static const int activateTexCoord = 1 << texClient;
-  /// Flag that the active TU should be used for binding textures
-  static const int activateImage = 1 << texServer;
   /// Flag that the active TU should be used when changing the texture matrix
   static const int activateMatrix = 1 << texServer;
   /// Flag that the active TU should be used when changing the texture environment
@@ -962,18 +781,14 @@ public:
    * generation parameters.
    */
   static const int activateTexGen = 1 << texServer;
-  /// Flag that the active TU should be used when enabling/disabling texturing
-  static const int activateTexEnable = 1 << texServer;
   /**
-   * Activate the currently selected TU. 
-   * \a usage is a combination of UnitTexCoord and UnitImage and specifies,
-   * for what the current unit is to be used.
+   * Activate the currently selected coordinate set. 
    * \remarks Doesn't check whether the  multitexture extension is actually 
    *   supported, this must be done in calling code.
    */
-  void ActivateTU (uint usage)
+  void ActivateTCUnit (uint usage)
   {
-    int currentUnit = currentContext->currentUnit;
+    int currentUnit = currentContext->currentTCUnit;
     for (int i = 0; i < 2; i++)
     {
       if (currentContext->activeUnit[i] != currentUnit)
@@ -1109,6 +924,26 @@ public:
   }
   /** @} */
   
+  /**\name Clamp control
+   * @{ */
+  void SetClampColor (GLenum target, GLenum clamp)
+  {
+    int index = csGLStateCacheContext::GLClampTargetToCacheIndex (target);
+    CS_ASSERT (index >= 0);
+    if (clamp != currentContext->clampState[index])
+    {
+      extmgr->glClampColorARB (target, clamp);
+      currentContext->clampState[index] = clamp;
+    }
+  }
+  GLenum GetClampColor (GLenum target) const
+  {
+    int index = csGLStateCacheContext::GLClampTargetToCacheIndex (target);
+    CS_ASSERT (index >= 0);
+    return currentContext->clampState[index];
+  }
+  /** @} */
+  
   /// Query the number of texture image units supported by OpenGL
   GLint GetNumImageUnits() const { return currentContext->numImageUnits; }
   /// Query the number of texture coordinate sets supported by OpenGL
@@ -1116,7 +951,7 @@ public:
 };
 
 #undef IMPLEMENT_CACHED_BOOL
-#undef IMPLEMENT_CACHED_BOOL_CURRENTLAYER
+#undef IMPLEMENT_CACHED_BOOL_IMAGEUNIT
 #undef IMPLEMENT_CACHED_PARAMETER_1
 #undef IMPLEMENT_CACHED_PARAMETER_2
 #undef IMPLEMENT_CACHED_PARAMETER_3
@@ -1129,7 +964,7 @@ public:
 #undef IMPLEMENT_CACHED_CLIENT_STATE_LAYER
 
 #undef DECLARE_CACHED_BOOL
-#undef DECLARE_CACHED_BOOL_CURRENTLAYER
+#undef DECLARE_CACHED_BOOL_IMAGEUNIT
 #undef DECLARE_CACHED_PARAMETER_1
 #undef DECLARE_CACHED_PARAMETER_2
 #undef DECLARE_CACHED_PARAMETER_3
