@@ -25,6 +25,9 @@
 
 #include "csutil/alignedalloc.h"
 #include "csutil/memdebug.h"
+#include "csutil/ref.h"
+#include "csutil/threading/mutex.h"
+#include "iutil/allocator.h"
 
 /**\addtogroup util_memory
  * @{ */
@@ -134,7 +137,7 @@ namespace CS
      * \c SingleAllocation specifies whether no more than a single block is
      * allocated from the allocator at any time. Using that option saves 
      * (albeit a miniscule amount of) memory, but obviously is only safe when
-     * it's know that the single allocation constraint is satisfied (such as 
+     * it's known that the single allocation constraint is satisfied (such as 
      * allocators for csArray<>s).
      *
      * \warning The pointer returned may point into the instance data; be 
@@ -148,6 +151,9 @@ namespace CS
       bool SingleAllocation = false>
     class LocalBufferAllocator : public ExcessAllocator
     {
+    #ifdef CS_DEBUG
+      void* startThis;
+    #endif
       static const size_t localSize = N * sizeof (T);
       static const uint8 freePattern = 0xfa;
       static const uint8 newlyAllocatedSalt = 0xac;
@@ -163,6 +169,9 @@ namespace CS
         }
         else
           localBuf[localSize] = 0;
+      #ifdef CS_DEBUG
+        startThis = this;
+      #endif
       }
       LocalBufferAllocator (const ExcessAllocator& xalloc) : 
         ExcessAllocator (xalloc)
@@ -175,9 +184,13 @@ namespace CS
         }
         else
           localBuf[localSize] = 0;
+      #ifdef CS_DEBUG
+        startThis = this;
+      #endif
       }
-      T* Alloc (size_t allocSize)
+      void* Alloc (size_t allocSize)
       {
+        CS_ASSERT(startThis == this);
         if (SingleAllocation)
         {
       #ifdef CS_DEBUG
@@ -199,11 +212,11 @@ namespace CS
           memset (localBuf, newlyAllocatedSalt, localSize);
       #endif
 	  if (allocSize <= localSize)
-	    return (T*)localBuf;
+	    return (void*)localBuf;
 	  else
           {
             void* p = ExcessAllocator::Alloc (allocSize);
-	    return (T*)p;
+	    return p;
           }
         }
         else
@@ -225,11 +238,12 @@ namespace CS
         }
       }
     
-      void Free (T* mem)
+      void Free (void* mem)
       {
+        CS_ASSERT(startThis == this);
         if (SingleAllocation)
         {
-          if (mem != (T*)localBuf)
+          if (mem != (void*)localBuf)
 	    ExcessAllocator::Free (mem);
 	  else
 	  {
@@ -255,6 +269,118 @@ namespace CS
       #endif
         }
         else
+        {
+          if (mem != (void*)localBuf) 
+            ExcessAllocator::Free (mem);
+          else
+          {
+            localBuf[localSize] = 0;
+          }
+        }
+      }
+    
+      // The 'relevantcount' parameter should be the number of items
+      // in the old array that are initialized.
+      void* Realloc (void* p, size_t newSize)
+      {
+        CS_ASSERT(startThis == this);
+        if (p == 0) return Alloc (newSize);
+        if (p == localBuf)
+        {
+          if (newSize <= localSize)
+            return p;
+          else
+          {
+	    p = ExcessAllocator::Alloc (newSize);
+	    memcpy (p, localBuf, localSize);
+        #ifdef CS_DEBUG
+            memset (localBuf, freePattern, localSize);
+        #endif
+            if (!SingleAllocation) localBuf[localSize] = 0;
+	    return p;
+          }
+        }
+        else
+        {
+          if ((newSize <= localSize) && (SingleAllocation || !localBuf[localSize]))
+	  {
+	    memcpy (localBuf, p, newSize);
+	    ExcessAllocator::Free (p);
+            if (!SingleAllocation) localBuf[localSize] = 1;
+	    return localBuf;
+	  }
+	  else
+	    return ExcessAllocator::Realloc (p, newSize);
+        }
+      }
+
+      using ExcessAllocator::SetMemTrackerInfo;
+    };
+    
+    /**
+     * LocalBufferAllocator without safety checks, but suitable for use across
+     * modules.
+     *
+     * Since this class does not perform the same safety checks as
+     * LocalBufferAllocator it's good practice to start off with a normal
+     * (checked) LocalBufferAllocator and only move to using
+     * LocalBufferAllocatorUnchecked once you're sure there are no invalid
+     * usages.
+     */
+    template<typename T, size_t N, class ExcessAllocator = AllocatorMalloc,
+      bool SingleAllocation = false>
+    class LocalBufferAllocatorUnchecked : public ExcessAllocator
+    {
+      static const size_t localSize = N * sizeof (T);
+      uint8 localBuf[localSize + (SingleAllocation ? 0 : 1)];
+    public:
+      LocalBufferAllocatorUnchecked ()
+      {
+        if (!SingleAllocation) 
+          localBuf[localSize] = 0;
+      }
+      LocalBufferAllocatorUnchecked (const ExcessAllocator& xalloc) : 
+        ExcessAllocator (xalloc)
+      {
+        if (!SingleAllocation) 
+          localBuf[localSize] = 0;
+      }
+      T* Alloc (size_t allocSize)
+      {
+        if (SingleAllocation)
+        {
+	  if (allocSize <= localSize)
+	    return (T*)localBuf;
+	  else
+          {
+            void* p = ExcessAllocator::Alloc (allocSize);
+	    return (T*)p;
+          }
+        }
+        else
+        {
+          void* p;
+	  if ((allocSize <= localSize) && !localBuf[localSize])
+          {
+            localBuf[localSize] = 1;
+	    p = localBuf;
+          }
+	  else
+          {
+            p = ExcessAllocator::Alloc (allocSize);
+          }
+          return (T*)p;
+        }
+      }
+    
+      void Free (T* mem)
+      {
+        if (SingleAllocation)
+        {
+          if (mem != (T*)localBuf)
+	    ExcessAllocator::Free (mem);
+        }
+        else
         {
           if (mem != (T*)localBuf) 
             ExcessAllocator::Free (mem);
@@ -278,16 +404,13 @@ namespace CS
           {
 	    p = ExcessAllocator::Alloc (newSize);
 	    memcpy (p, localBuf, localSize);
-        #ifdef CS_DEBUG
-            memset (localBuf, freePattern, localSize);
-        #endif
             if (!SingleAllocation) localBuf[localSize] = 0;
 	    return p;
           }
         }
         else
         {
-          if ((newSize <= localSize) && !localBuf[localSize])
+          if ((newSize <= localSize) && (SingleAllocation || !localBuf[localSize]))
 	  {
 	    memcpy (localBuf, p, newSize);
 	    ExcessAllocator::Free (p);
@@ -505,6 +628,102 @@ namespace CS
 	(void)info;
       }
     };
+
+    /**
+     * Memory allocator forwarding to another allocator.
+     */
+    template<typename OtherAllocator>
+    class AllocatorRef
+    {
+      OtherAllocator& alloc;
+    public:
+      AllocatorRef (OtherAllocator& referencedAlloc)
+       : alloc (referencedAlloc) {}
+    
+      /// Allocate a block of memory of size \p n.
+      CS_ATTRIBUTE_MALLOC void* Alloc (const size_t n)
+      { return alloc.Alloc (n); }
+      /// Free the block \p p.
+      void Free (void* p) { alloc.Free (p); }
+      /// Resize the allocated block \p p to size \p newSize.
+      void* Realloc (void* p, size_t newSize)
+      { return alloc.Realloc (p, newSize); }
+      /// Set the information used for memory tracking.
+      void SetMemTrackerInfo (const char* info)
+      { alloc.SetMemTrackerInfo (info); }
+    };
+
+    /**
+     * Threadsafe allocator wrapper.
+     */
+    template <class Allocator>
+    class AllocatorSafe : protected Allocator
+    {
+    protected:
+      typedef Allocator WrappedAllocatorType;
+      typedef AllocatorSafe<Allocator> AllocatorSafeType;
+      /// Mutex to lock the wrapped allocator.
+      CS::Threading::RecursiveMutex mutex;
+
+    public:
+      AllocatorSafe () : Allocator ()
+      {
+      }
+
+      template<typename A1>
+      AllocatorSafe (const A1& a1) : Allocator (a1)
+      {
+      }
+
+      template<typename A1, typename A2>
+      AllocatorSafe (const A1& a1, const A2& a2) : Allocator (a1, a2)
+      {
+      }
+
+      void Free (void* p)
+      {
+        CS::Threading::RecursiveMutexScopedLock lock(mutex);
+        Allocator::Free(p);
+      }
+
+      CS_ATTRIBUTE_MALLOC void* Alloc (const size_t n)
+      {
+        CS::Threading::RecursiveMutexScopedLock lock(mutex);
+        return Allocator::Alloc(n);
+      }
+
+      void* Realloc (void* p, size_t newSize)
+      {
+        CS::Threading::RecursiveMutexScopedLock lock(mutex);
+        return Allocator::Realloc(p, newSize);
+      }
+
+      void SetMemTrackerInfo (const char* info)
+      {
+        CS::Threading::RecursiveMutexScopedLock lock(mutex);
+        Allocator::SetMemTrackerInfo(info);
+      }
+    };
+
+    /**
+     * Memory allocator forwarding to an iAllocator implementation.
+     */
+    class AllocatorInterface
+    {
+      csRef<iAllocator> alloc;
+    public:
+      AllocatorInterface (iAllocator* alloc)
+       : alloc (alloc) {}
+
+      CS_ATTRIBUTE_MALLOC void* Alloc (const size_t n)
+      { return alloc->Alloc (n); }
+      void Free (void* p) { alloc->Free (p); }
+      void* Realloc (void* p, size_t newSize)
+      { return alloc->Realloc (p, newSize); }
+      void SetMemTrackerInfo (const char* info)
+      { alloc->SetMemTrackerInfo (info); }
+    };
+
   } // namespace Memory
 } // namespace CS
 
