@@ -1014,12 +1014,30 @@ public:
    */
   static size_t MapToUpper (const utf32_char ch, utf32_char* dest, 
     size_t destSize, uint flags = 0);
+  /**
+   * Map a code point to its upper case equivalent(s).
+   * \param ch Code point to be mapped.
+   * \return The mapped code point.
+   * \remarks Always performs a 'simple' mapping (see #csUcMapSimple).
+   */
+  inline static utf32_char MapToUpper (const utf32_char ch)
+  {
+    utf32_char ret;
+    MapToUpper (ch, &ret, 1, csUcMapSimple);
+    return ret;
+  }
   /**
    * Map a code point to its lower case equivalent(s).
    * \copydoc MapToUpper(const utf32_char, utf32_char*, size_t, uint)
    */
   static size_t MapToLower (const utf32_char ch, utf32_char* dest, 
     size_t destSize, uint flags = 0);
+  inline static utf32_char MapToLower (const utf32_char ch)
+  {
+    utf32_char ret;
+    MapToLower (ch, &ret, 1, csUcMapSimple);
+    return ret;
+  }
   /**
    * Map a code point to its fold equivalent(s).
    * Fold mapping is useful for binary comparison of two Unicode strings.
@@ -1027,6 +1045,12 @@ public:
    */
   static size_t MapToFold (const utf32_char ch, utf32_char* dest, 
     size_t destSize, uint flags = 0);
+  inline static utf32_char MapToFold (const utf32_char ch)
+  {
+    utf32_char ret;
+    MapToFold (ch, &ret, 1, csUcMapSimple);
+    return ret;
+  }
   /** @} */
 };
 
