@@ -52,8 +52,8 @@ public:
 
 /**
  * The csEventNameRegistry transforms textual event names (e.g., 
- * "crystalspace.input.joystick.3.button") into easier-to-manage csEventIDs 
- * (which, in non-debug builds, are really just csStringIDs).
+ * "crystalspace.input.joystick.3.button") into easier-to-manage csEventID's 
+ * (which, in non-debug builds, are really just csStringID's).
  * Also offers easy methods for querying parentage relationships between
  * two event names.
  * 
@@ -66,7 +66,7 @@ class CS_CRYSTALSPACE_EXPORT csEventNameRegistry :
  private:
   /**
    * Do not allocate for yourself!  This object uses the singleton pattern
-   * and should be retrieved with the static csEventNameRegistry::GetRegistry 
+   * and should be retrieved with the static csEventNameRegistry::GetRegistry()
    * method.
    */
   csEventNameRegistry (iObjectRegistry *);
@@ -479,51 +479,6 @@ static inline csEventID csevCanvasOp (
 #define csevCommandLineHelp(reg)      \
   (csEventNameRegistry::GetID((reg), "crystalspace.application.commandlinehelp"))
 
-/**
- * Broadcasted before csevProcess on every frame.
- * This event will go away soon, since it was a kludge to
- * work around the lack of subscription priorities/scheduling.
- * Should be replaced with subscriptions to csevFrame with subscription 
- * ordering.
- */
-CS_CRYSTALSPACE_EXPORT CS_DEPRECATED_METHOD 
-  csEventID csevPreProcess(iObjectRegistry *reg);
-CS_CRYSTALSPACE_EXPORT CS_DEPRECATED_METHOD 
-  csEventID csevPreProcess(iEventNameRegistry *reg);
-
-/**
- * Broadcasted every frame.
- * This event will go away soon, replaced by csevFrame.
- */
-CS_CRYSTALSPACE_EXPORT CS_DEPRECATED_METHOD 
-  csEventID csevProcess(iObjectRegistry *reg);
-CS_CRYSTALSPACE_EXPORT CS_DEPRECATED_METHOD 
-  csEventID csevProcess(iEventNameRegistry *reg);
-
-/**
- * Broadcasted after csevProcess on every frame.
- * This event will go away soon, since it was a kludge to
- * work around the lack of subscription priorities/scheduling.
- * Should be replaced with subscriptions to csevFrame with subscription 
- * ordering.
- */
-CS_CRYSTALSPACE_EXPORT CS_DEPRECATED_METHOD
-  csEventID csevPostProcess(iObjectRegistry *reg);
-CS_CRYSTALSPACE_EXPORT CS_DEPRECATED_METHOD 
-  csEventID csevPostProcess(iEventNameRegistry *reg);
-
-/**
- * Broadcasted after csevPostProcess on every frame.
- * This event will go away soon, since it was a kludge to
- * work around the lack of subscription priorities/scheduling.
- * Should be replaced with subscriptions to csevFrame with subscription 
- * ordering.
- */
-CS_CRYSTALSPACE_EXPORT CS_DEPRECATED_METHOD 
-  csEventID csevFinalProcess(iObjectRegistry *reg);
-CS_CRYSTALSPACE_EXPORT CS_DEPRECATED_METHOD 
-  csEventID csevFinalProcess(iEventNameRegistry *reg);
-
 /** @} */
 
 #define CS_DECLARE_SYSTEM_EVENT_SHORTCUTS			\
@@ -531,11 +486,7 @@ CS_CRYSTALSPACE_EXPORT CS_DEPRECATED_METHOD
   csEventID SystemClose
 
 #define CS_DECLARE_FRAME_EVENT_SHORTCUTS			\
-  csEventID Frame;						\
-  csEventID PreProcess;						\
-  csEventID Process;						\
-  csEventID PostProcess;					\
-  csEventID FinalProcess
+  csEventID Frame
 
 #define CS_DECLARE_INPUT_EVENT_SHORTCUTS			\
   csEventID KeyboardEvent;					\
@@ -544,7 +495,7 @@ CS_CRYSTALSPACE_EXPORT CS_DEPRECATED_METHOD
 
 /**
  * Shortcut to declare class properties SystemOpen, SystemClose,
- * Frame, PreProcess, Process, PostProcess, FinalProcess.
+ * Frame.
  * Pair with CS_INITIALIZE_EVENT_SHORTCUTS(registry).
  */
 #define CS_DECLARE_EVENT_SHORTCUTS				\
@@ -559,10 +510,6 @@ CS_CRYSTALSPACE_EXPORT CS_DEPRECATED_METHOD
 
 #define CS_INITIALIZE_FRAME_EVENT_SHORTCUTS(object_reg) do {	\
     Frame = csevFrame ((object_reg));				\
-    PreProcess = csevPreProcess ((object_reg));			\
-    Process = csevProcess ((object_reg));			\
-    PostProcess = csevPostProcess ((object_reg));		\
-    FinalProcess = csevFinalProcess ((object_reg));		\
   } while (0)
 
 #define CS_INITIALIZE_INPUT_EVENT_SHORTCUTS(object_reg) do {	\
