@@ -134,6 +134,69 @@ namespace CS
 	}
       };
       
+      /**
+       * Reuse condition: a resource is reused if only one reference is held 
+       * to it. (The resource type must be a csRef<>.)
+       */
+      class ReuseIfOnlyOneRef
+      {
+      public:
+	struct AddParameter
+	{
+	  AddParameter () {}
+	};
+	struct StoredAuxiliaryInfo
+	{
+	  template<typename ResourceCacheType>
+	  StoredAuxiliaryInfo (const ResourceCacheType& cache, 
+	    const AddParameter& param) {}
+	};
+	
+	template<typename ResourceCacheType>
+	void MarkActive (const ResourceCacheType& cache,
+	    StoredAuxiliaryInfo& elementInfo)
+	{ }
+  
+	template<typename ResourceCacheType>
+	bool IsReusable (const ResourceCacheType& cache,
+	  const StoredAuxiliaryInfo& elementInfo,
+	  const typename ResourceCacheType::CachedType& data)
+	{
+	  return data->GetRefCount() == 1;
+	}
+      };
+      
+      /**
+       * Reuse condition: allow immediate reuse.
+       */
+      class ReuseAlways
+      {
+      public:
+	struct AddParameter
+	{
+	  AddParameter () {}
+	};
+	struct StoredAuxiliaryInfo
+	{
+	  template<typename ResourceCacheType>
+	  StoredAuxiliaryInfo (const ResourceCacheType& cache, 
+	    const AddParameter& param) {}
+	};
+	
+	template<typename ResourceCacheType>
+	void MarkActive (const ResourceCacheType& cache,
+	    StoredAuxiliaryInfo& elementInfo)
+	{ }
+  
+	template<typename ResourceCacheType>
+	bool IsReusable (const ResourceCacheType& cache,
+	  const StoredAuxiliaryInfo& elementInfo,
+	  const typename ResourceCacheType::CachedType& data)
+	{
+	  return true;
+	}
+      };
+      
       /**
        * Purge condition: a resource is purged after a certain time has passed
        */
@@ -180,6 +243,38 @@ namespace CS
 	}
       };
 
+      /**
+       * Purge condition: a resource is purged if only one reference is held 
+       * to it. (The resource type must be a csRef<>.)
+       */
+      class PurgeIfOnlyOneRef
+      {
+      public:
+	struct AddParameter
+	{
+	  AddParameter () {}
+	};
+	struct StoredAuxiliaryInfo
+	{
+	  template<typename ResourceCacheType>
+	  StoredAuxiliaryInfo (const ResourceCacheType& cache, 
+	    const AddParameter& param) {}
+	};
+	
+	template<typename ResourceCacheType>
+	void MarkActive (const ResourceCacheType& cache,
+	    StoredAuxiliaryInfo& elementInfo)
+	{ }
+  
+	template<typename ResourceCacheType>
+	bool IsPurgeable (const ResourceCacheType& cache,
+	  StoredAuxiliaryInfo& elementInfo,
+	  const typename ResourceCacheType::CachedType& data)
+	{
+	  return data->GetRefCount() == 1;
+	}
+      };
+      
     } // namespace ResourceCache
     
     /**
@@ -252,38 +347,38 @@ namespace CS
 	ElementWrapper (const ElementWrapper& other) : ptr (other.ptr) { }
 	
 	Element* operator->() { return ptr; }
-	operator Element*() const { return ptr; }
 	
-	// FIXME: Comparisons are fragile, rely on csComparator only using <
+	// Comparisons among ElementWrappers
 	bool operator== (const ElementWrapper& other) const
-	{ return ptr == other.ptr; }
+	{
+	  return ResourceSorting::IsEqual (ptr->data, other.ptr->data);
+	}
+	bool operator<= (const ElementWrapper& other) const
+	{
+	  return ResourceSorting::IsLargerEqual (ptr->data, other.ptr->data);
+	}
 	
-	bool operator< (const ElementWrapper& other) const
+	// Comparisons against ResourceSortingKeyType
+	bool operator== (const ResourceSortingKeyType& other) const
 	{
-	  // @@@ Make more efficient...
-	  if (ResourceSorting::IsEqual (ptr->data, other.ptr->data)) return false;
-	  if (ResourceSorting::IsLargerEqual (ptr->data, other.ptr->data)) return false;
-	  return true;
+	  return ResourceSorting::IsEqual (ptr->data, other);
 	}
-	bool operator< (const ResourceSortingKeyType& other) const
+	bool operator<= (const ResourceSortingKeyType& other) const
 	{
-	  // @@@ Make more efficient...
-	  if (ResourceSorting::IsEqual (ptr->data, other)) return false;
-	  if (ResourceSorting::IsLargerEqual (ptr->data, other)) return false;
-	  return true;
+	  return ResourceSorting::IsLargerEqual (ptr->data, other);
 	}
-	friend bool operator< (const ResourceSortingKeyType& key, 
-			       const ElementWrapper& el)
+	friend bool operator<= (const ResourceSortingKeyType& key, 
+			        const ElementWrapper& el)
 	{
-	  // @@@ Make more efficient...
-	  if (ResourceSorting::IsEqual (el.ptr->data, key)) return false;
-	  if (ResourceSorting::IsLargerEqual (el.ptr->data, key)) return true;
-	  return false;
+	  return ResourceSorting::IsLargerEqual (key, el.ptr->data);
 	}
 	
       };
       
       csBlockAllocator<Element> elementAlloc;
+      typedef csRedBlackTree<ElementWrapper,
+	  CS::Container::DefaultRedBlackTreeAllocator<ElementWrapper>,
+	  CS::Container::RedBlackTreeOrderingPartial> AvailableResourcesTree;
       // Tree of available resources
       struct AvailableResourcesWrapper : public ReuseCondition
       {
@@ -296,13 +391,13 @@ namespace CS
 	  RBTraverser (csBlockAllocator<Element>& elementAlloc) :
 	    elementAlloc (elementAlloc) {}
 	  
-	  void Process (Element* el)
+	  void operator() (ElementWrapper& el)
 	  {
-	    elementAlloc.Free (el);
+	    elementAlloc.Free (el.ptr);
 	  }
 	};
       public:
-	csRedBlackTree<ElementWrapper> v;
+	AvailableResourcesTree v;
         
         AvailableResourcesWrapper (const ReuseCondition& other,
           csBlockAllocator<Element>& elementAlloc)
@@ -332,7 +427,7 @@ namespace CS
 	  typename csList<ElementWrapper>::Iterator listIt (v);
 	  while (listIt.HasNext())
 	  {
-	    Element* el = listIt.Next();
+	    Element* el = listIt.Next().ptr;
 	    elementAlloc.Free (el);
 	  }
 	  v.DeleteAll();
@@ -361,11 +456,11 @@ namespace CS
 	SearchDataTraverser (T* entry, Element*& ret) 
 	  : entry (entry), ret (ret) {}
 	
-        bool Process (Element* el)
+        bool operator() (ElementWrapper& el)
 	{
-	  if (&(el->data) == entry)
+	  if (&(el.ptr->data) == entry)
 	  {
-	    ret = el;
+	    ret = el.ptr;
 	    return false;
 	  }
 	  return true;
@@ -383,7 +478,7 @@ namespace CS
 	typename csList<ElementWrapper>::Iterator listIt (activeResources.v);
 	while (listIt.HasNext())
 	{
-	  Element* el = listIt.Next();
+	  Element* el = listIt.Next().ptr;
 	  if (&(el->data) == entry) return el;
 	}
 	
@@ -411,6 +506,7 @@ namespace CS
       
       ~GenericResourceCache()
       {
+        availableResources.Destroy ();
       }
 
       /**
@@ -438,9 +534,9 @@ namespace CS
       public:
 	VerifyTraverser (Element* el) : el (el) {}
 	
-        bool Process (Element* el)
+        bool operator() (ElementWrapper& el)
 	{
-	  CS_ASSERT(el != this->el);
+	  CS_ASSERT(el.ptr != this->el);
 	  return true;
 	}
       };
@@ -471,11 +567,11 @@ namespace CS
 	
 	if (time >= lastPurgeAged + agedPurgeInterval)
 	{
-	  typename csRedBlackTree<ElementWrapper>::Iterator treeIt (
+	  typename AvailableResourcesTree::Iterator treeIt (
 	    availableResources.v.GetIterator ());
 	  while (treeIt.HasNext())
 	  {
-	    Element* el = treeIt.PeekNext ();
+	    Element* el = treeIt.PeekNext ().ptr;
 	    if (GetPurgeCondition().IsPurgeable (*this, 
 		el->GetPurgeAuxiliary(), el->data))
 	    {
@@ -500,7 +596,7 @@ namespace CS
 	  if (GetReuseCondition().IsReusable (*this,
 	      el->GetReuseAuxiliary(), el->data))
 	  {
-	    VerifyElementNotInTree (el);
+	    VerifyElementNotInTree (el.ptr);
 	    availableResources.v.Insert (el);
 	    activeResources.v.Delete (listIt);
 	  }
@@ -512,17 +608,17 @@ namespace CS
       T* Query (const ResourceSortingKeyType& key = 
 	ResourceSortingKeyType(), bool exact = false)
       {
-	const csRedBlackTree<ElementWrapper>& constTree = availableResources.v;
+	const AvailableResourcesTree& constTree = availableResources.v;
 	ElementWrapper const* el;
 	if (exact)
 	  el = constTree.Find (key);
 	else
-	  el = constTree.FindSmallestGreaterEqual (key);
+	  el = constTree.FindGreatestSmallerEqual (key);
 	if (el != 0)
 	{
 	  ElementWrapper myElement = *el;
 	  availableResources.v.DeleteExact (el);
-	  VerifyElementNotInTree (myElement);
+	  VerifyElementNotInTree (myElement.ptr);
 	  activeResources.v.PushFront (myElement);
 	  GetPurgeCondition().MarkActive (*this, myElement->GetPurgeAuxiliary());
 	  GetReuseCondition().MarkActive (*this, myElement->GetReuseAuxiliary());
