@@ -39,10 +39,13 @@ class CS_CRYSTALSPACE_EXPORT SimpleJobQueue :
 public:
   SimpleJobQueue ();
 
-  virtual void Enqueue (iJob* job);
+  virtual void Enqueue (iJob* job, bool lowPriority = false);
   virtual void PullAndRun (iJob* job);
   virtual void Unqueue (iJob* job, bool waitIfCurrent = true);
   virtual bool IsFinished ();
+  virtual int32 GetQueueCount();
+  void Wait(iJob*);
+  virtual void WaitAll ();
 };
 
 
