@@ -19,6 +19,7 @@
 #include "cssysdef.h"
 #include "csqint.h"
 
+#include "csgeom/vector3.h"
 #include "igraphic/image.h"
 #include "igraphic/imageio.h"
 #include "iutil/document.h"
@@ -29,7 +30,7 @@
 
 #include "csgeom/csrect.h"
 #include "csgeom/math.h"
-#include "csplugincommon/canvas/scrshot.h"
+#include "csgeom/projections.h"
 #include "csplugincommon/opengl/assumedstate.h"
 #include "csplugincommon/opengl/glcommon2d.h"
 #include "csplugincommon/opengl/glstates.h"
@@ -38,10 +39,9 @@
 
 csGraphics2DGLCommon::csGraphics2DGLCommon (iBase *iParent) :
   scfImplementationType (this, iParent), statecache (0), statecontext (0),
-    hasRenderTarget (false)
+    openComplete (false), hasRenderTarget (false)
 {
   EventOutlet = 0;
-  screen_shot = 0;
   multiFavorQuality = false;
   fontCache = 0;
   useCombineTE = false;
@@ -61,22 +61,6 @@ bool csGraphics2DGLCommon::Initialize (iObjectRegistry *object_reg)
    * both need settings from that file. */
   config.AddConfig (object_reg, "/config/r3dopengl.cfg");
 
-  // Guesstimate pixel format. We don't really care about it anyway.
-#if !defined(CS_LITTLE_ENDIAN)
-    pfmt.RedMask =   0x000000FF;
-    pfmt.GreenMask = 0x0000FF00;
-    pfmt.BlueMask =  0x00FF0000;
-    pfmt.AlphaMask = 0xFF000000;
-#else 
-    pfmt.AlphaMask = 0xFF000000;
-    pfmt.RedMask =   0x00FF0000;
-    pfmt.GreenMask = 0x0000FF00;
-    pfmt.BlueMask =  0x000000FF;
-#endif
-  pfmt.PixelBytes = 4;
-  pfmt.PalEntries = 0;
-  pfmt.complete ();
-
   ext.Initialize (object_reg, this);
 
   multiFavorQuality = config->GetBool ("Video.OpenGL.MultisampleFavorQuality");
@@ -93,8 +77,6 @@ csGraphics2DGLCommon::~csGraphics2DGLCommon ()
 {
   Close ();
   
-  delete[] screen_shot;
-
   while (ssPool)
   {
     csGLScreenShot* next = ssPool->poolNext;
@@ -163,17 +145,17 @@ bool csGraphics2DGLCommon::Open ()
       // Sanity check
       if ((vMajor < 1) || ((vMajor == 1) && (vMinor < 1)))
       {
-	Report (CS_REPORTER_SEVERITY_ERROR,
-	  "OpenGL >= 1.1 is required, but only %d.%d is present.",
-	  vMajor, vMinor);
+	      Report (CS_REPORTER_SEVERITY_ERROR,
+	        "OpenGL >= 1.1 is required, but only %d.%d is present.",
+	        vMajor, vMinor);
       }
       if ((vMajor >= 1) || ((vMajor == 1) && (vMinor >= 2)))
       {
-	//ext.InitGL_version_1_2 ();
+	      //ext.InitGL_version_1_2 ();
       }
       if ((vMajor >= 1) || ((vMajor == 1) && (vMinor >= 3)))
       {
-	//ext.InitGL_version_1_3 ();
+	      //ext.InitGL_version_1_3 ();
       }
     }
   }
@@ -186,16 +168,14 @@ bool csGraphics2DGLCommon::Open ()
     (ext.CS_GL_ARB_texture_env_combine || ext.CS_GL_EXT_texture_env_combine);
   if (useCombineTE)
   {
-    GLint texUnits;
-    glGetIntegerv(GL_MAX_TEXTURE_UNITS_ARB, &texUnits);
-    for (GLint u = texUnits - 1; u >= 0; u--)
+    for (GLint u = statecache->GetNumTexCoords(); u-- > 0; )
     {
-      statecache->SetCurrentTU (u);
-      statecache->ActivateTU (csGLStateCache::activateTexEnv);
+      statecache->SetCurrentTCUnit (u);
+      statecache->ActivateTCUnit (csGLStateCache::activateTexEnv);
       glTexEnvi (GL_TEXTURE_ENV, GL_TEXTURE_ENV_MODE, GL_COMBINE_ARB);
     }
-    statecache->ActivateTU (csGLStateCache::activateTexCoord);
   }
+  ext.InitGL_EXT_blend_func_separate ();
   ext.InitGL_ARB_multisample();
 
   if (ext.CS_GL_ARB_multisample)
@@ -206,31 +186,29 @@ bool csGraphics2DGLCommon::Open ()
     if (glmultisamp)
     {
       if (glmultisamp != currentFormat[glpfvMultiSamples])
-	Report (CS_REPORTER_SEVERITY_NOTIFY,
-	  "Multisample: actually %d samples",
-	  (int)glmultisamp);
+	      Report (CS_REPORTER_SEVERITY_NOTIFY,
+	        "Multisample: actually %d samples", (int)glmultisamp);
 
       ext.InitGL_NV_multisample_filter_hint();
       if (ext.CS_GL_NV_multisample_filter_hint)
       {
-	glHint (GL_MULTISAMPLE_FILTER_HINT_NV,
-	  multiFavorQuality ? GL_NICEST : GL_FASTEST);
+        glHint (GL_MULTISAMPLE_FILTER_HINT_NV,
+          multiFavorQuality ? GL_NICEST : GL_FASTEST);
 	
-	GLint msHint;
-	glGetIntegerv (GL_MULTISAMPLE_FILTER_HINT_NV, &msHint);
-	Report (CS_REPORTER_SEVERITY_NOTIFY,
-	  "Multisample settings: %s",
-	  ((msHint == GL_NICEST) ? "quality" :
-	  ((msHint == GL_FASTEST) ? "performance" : "unknown")));
+        GLint msHint;
+        glGetIntegerv (GL_MULTISAMPLE_FILTER_HINT_NV, &msHint);
+        Report (CS_REPORTER_SEVERITY_NOTIFY,
+          "Multisample settings: %s", ((msHint == GL_NICEST) ? "quality" :
+          ((msHint == GL_FASTEST) ? "performance" : "unknown")));
       }
     }
     else
     {
       Report (CS_REPORTER_SEVERITY_NOTIFY,
-	"Multisample: disabled");
+	      "Multisample: disabled");
     }
   }
-
+  
   GLFontCache->Setup();
 
   CS::PluginCommon::GL::SetAssumedState (statecache, &ext);
@@ -242,6 +220,7 @@ bool csGraphics2DGLCommon::Open ()
   glViewport (0, 0, vpWidth, vpHeight);
   Clear (0);
 
+  openComplete = true;
   return true;
 }
 
@@ -253,11 +232,12 @@ void csGraphics2DGLCommon::Close ()
   delete statecache; statecache = 0;
   ext.Close ();
   driverdb.Close ();
+  openComplete = false;
 }
 
 void csGraphics2DGLCommon::SetClipRect (int xmin, int ymin, int xmax, int ymax)
 {
-  ((csGLFontCache*)fontCache)->FlushText ();
+  if (fontCache) ((csGLFontCache*)fontCache)->FlushText ();
 
   csGraphics2D::SetClipRect (xmin, ymin, xmax, ymax);
   glScissor (vpLeft + ClipX1, fbHeight - (vpTop + ClipY2),
@@ -313,7 +293,12 @@ bool csGraphics2DGLCommon::BeginDraw ()
   statecache->SetColorMask (true, true, true, true);
     
   statecache->Enable_GL_BLEND ();		      
-  statecache->SetBlendFunc (GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);
+  if (ext.CS_GL_EXT_blend_func_separate)
+    // Blending mode that's premultiplied alpha friendly
+    statecache->SetBlendFuncSeparate (GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA,
+				      GL_ONE, GL_ONE_MINUS_SRC_ALPHA);
+  else
+    statecache->SetBlendFunc (GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);
 
   return true;
 }
@@ -409,7 +394,7 @@ void csGraphics2DGLCommon::OpenDriverDB (const char* phase)
   if (!dbfile)
   {
     Report (CS_REPORTER_SEVERITY_WARNING, 
-      "Could not open driver database file '%s'", driverDB);
+      "Could not open driver database file %s", CS::Quote::Single (driverDB));
     return;
   }
 
@@ -449,6 +434,21 @@ void csGraphics2DGLCommon::Report (int severity, const char* msg, ...)
   va_end (args);
 }
 
+const char* csGraphics2DGLCommon::GetHWRenderer()
+{
+	return (char*)glGetString (GL_RENDERER);
+}
+
+const char* csGraphics2DGLCommon::GetHWGLVersion()
+{
+	return (char*)glGetString (GL_VERSION);
+}
+
+const char* csGraphics2DGLCommon::GetHWVendor()
+{
+	return (char*)glGetString (GL_VENDOR);
+}
+
 const char* csGraphics2DGLCommon::GetRendererString (const char* str)
 {
   if (strcmp (str, "renderer") == 0)
@@ -497,11 +497,6 @@ void csGraphics2DGLCommon::Clear (int color)
   glClear (GL_COLOR_BUFFER_BIT);
 }
 
-void csGraphics2DGLCommon::SetRGB (int i, int r, int g, int b)
-{
-  csGraphics2D::SetRGB (i, r, g, b);
-}
-
 void csGraphics2DGLCommon::DrawLine (
   float x1, float y1, float x2, float y2, int color)
 {
@@ -565,6 +560,70 @@ void csGraphics2DGLCommon::DrawLine (
   if (gl_alphaTest) statecache->Enable_GL_ALPHA_TEST ();
 }
 
+void csGraphics2DGLCommon::DrawLineProjected
+  (const csVector3& _v1, const csVector3& _v2, float fov, int color)
+{
+  csVector3 v1 (_v1), v2 (_v2);
+  if (!DrawLineNearClip (v1, v2)) return;
+
+  float iz1 = fov / v1.z;
+  int px1 = csQint (v1.x * iz1 + (vpWidth / 2));
+  int py1 = vpHeight - 1 - csQint (v1.y * iz1 + (vpHeight / 2));
+  float iz2 = fov / v2.z;
+  int px2 = csQint (v2.x * iz2 + (vpWidth / 2));
+  int py2 = vpHeight - 1 - csQint (v2.y * iz2 + (vpHeight / 2));
+
+  DrawLine (px1, py1, px2, py2, color);
+}
+
+void csGraphics2DGLCommon::DrawLineProjected
+  (const csVector3& _v1, const csVector3& _v2,
+   const CS::Math::Matrix4& projection, int color)
+{
+  csVector3 v1 (_v1), v2 (_v2);
+  if (!DrawLineNearClip (v1, v2)) return;
+
+  csVector4 v1p (projection * csVector4 (v1));
+  v1p /= v1p.w;
+  csVector4 v2p (projection * csVector4 (v2));
+  v2p /= v2p.w;
+
+  int px1 = csQint ((v1p.x + 1) * (vpWidth / 2));
+  int py1 = vpHeight - 1 - csQint ((v1p.y + 1) * (vpHeight / 2));
+  int px2 = csQint ((v2p.x + 1) * (vpWidth / 2));
+  int py2 = vpHeight - 1 - csQint ((v2p.y + 1) * (vpHeight / 2));
+
+  DrawLine (px1, py1, px2, py2, color);
+}
+
+bool csGraphics2DGLCommon::DrawLineNearClip (csVector3 & v1, csVector3 & v2)
+{
+  if (v1.z < SMALL_Z && v2.z < SMALL_Z)
+    return false;
+
+  if (v1.z < SMALL_Z)
+  {
+    // x = t*(x2-x1)+x1;
+    // y = t*(y2-y1)+y1;
+    // z = t*(z2-z1)+z1;
+    float t = (SMALL_Z - v1.z) / (v2.z - v1.z);
+    v1.x = t * (v2.x - v1.x) + v1.x;
+    v1.y = t * (v2.y - v1.y) + v1.y;
+    v1.z = SMALL_Z;
+  }
+  else if (v2.z < SMALL_Z)
+  {
+    // x = t*(x2-x1)+x1;
+    // y = t*(y2-y1)+y1;
+    // z = t*(z2-z1)+z1;
+    float t = (SMALL_Z - v1.z) / (v2.z - v1.z);
+    v2.x = t * (v2.x - v1.x) + v1.x;
+    v2.y = t * (v2.y - v1.y) + v1.y;
+    v2.z = SMALL_Z;
+  }
+  return true;
+}
+
 void csGraphics2DGLCommon::DrawBox (int x, int y, int w, int h, int color)
 {
   ((csGLFontCache*)fontCache)->FlushText ();
@@ -661,133 +720,32 @@ void csGraphics2DGLCommon::Blit (int x, int y, int w, int h,
   if (gl_alphaTest) statecache->Enable_GL_ALPHA_TEST ();
 }
 
-unsigned char* csGraphics2DGLCommon::GetPixelAt (int x, int y)
+void csGraphics2DGLCommon::GetPixel (int x, int y, uint8 &oR, uint8 &oG, uint8 &oB)
 {
-  ((csGLFontCache*)fontCache)->FlushText ();
-
-  /// left as Height-y-1 to keep within offscreen bitmap.
-  /// but for opengl itself you'd need Height-y.
-  return screen_shot ?
-    (screen_shot + pfmt.PixelBytes * ((vpHeight - y - 1) * vpWidth + x)) : 0;
+  uint8 dummy;
+  csGraphics2DGLCommon::GetPixel (x, y, oR, oG, oB, dummy);
 }
 
-csImageArea *csGraphics2DGLCommon::SaveArea (int x, int y, int w, int h)
+void csGraphics2DGLCommon::GetPixel (int x, int y, uint8 &oR, uint8 &oG, uint8 &oB, uint8 &oA)
 {
-  ((csGLFontCache*)fontCache)->FlushText ();
-
-  // For the time being copy data into system memory.
-#ifndef GL_VERSION_1_2
-  if (pfmt.PixelBytes != 1 && pfmt.PixelBytes != 4)
-    return 0;
-#endif
-  // Convert to Opengl co-ordinate system
-  y = vpHeight - (y + h);
-
-  if (x < 0)
-  { w += x; x = 0; }
-  if (x + w > vpWidth)
-    w = vpWidth - x;
-  if (y < 0)
-  { h += y; y = 0; }
-  if (y + h > vpHeight)
-    h = vpHeight - y;
-  if ((w <= 0) || (h <= 0))
-    return 0;
-
-  csImageArea *Area = new csImageArea (x, y, w, h);
-  if (!Area)
-    return 0;
-  int actual_width = pfmt.PixelBytes * w;
-  GLubyte* dest = new GLubyte [actual_width * h];
-  Area->data = (char *)dest;
-  if (!dest)
-  {
-    delete Area;
-    return 0;
-  }
-  statecache->Disable_GL_TEXTURE_2D ();
-  bool gl_alphaTest = (glIsEnabled(GL_ALPHA_TEST) == GL_TRUE);
-  if (gl_alphaTest) statecache->Disable_GL_ALPHA_TEST ();
-  //csGLStates::Disable_GL_DITHER ();
-  GLenum format, type;
-  switch (pfmt.PixelBytes)
-  {
-    case 1:
-      format = GL_COLOR_INDEX;
-      type = GL_UNSIGNED_BYTE;
-      break;
-#ifdef GL_VERSION_1_2
-    case 2:
-      format = GL_RGB;
-      type = GL_UNSIGNED_SHORT_5_6_5;
-      break;
-#endif
-    case 4:
-      format = GL_RGBA;
-      type = GL_UNSIGNED_BYTE;
-      break;
-    default:
-      delete Area;
-      return 0; // invalid format
-  }
-  glReadPixels (x, y, w, h, format, type, dest);
-
-  if (gl_alphaTest) statecache->Enable_GL_ALPHA_TEST ();
-  return Area;
-}
-
-void csGraphics2DGLCommon::RestoreArea (csImageArea *Area, bool Free)
-{
-  ((csGLFontCache*)fontCache)->FlushText ();
-
-  statecache->Disable_GL_TEXTURE_2D ();
-  bool gl_alphaTest = (glIsEnabled(GL_ALPHA_TEST) == GL_TRUE);
-  if (gl_alphaTest) statecache->Disable_GL_ALPHA_TEST ();
-  //csGLStates::Disable_GL_DITHER ();
-  if (Area)
-  {
-    GLenum format, type;
-    switch (pfmt.PixelBytes)
-    {
-      case 1:
-        format = GL_COLOR_INDEX;
-        type = GL_UNSIGNED_BYTE;
-        break;
-#ifdef GL_VERSION_1_2
-      case 2:
-        format = GL_RGB;
-        type = GL_UNSIGNED_SHORT_5_6_5;
-        break;
-#endif
-      case 4:
-        format = GL_RGBA;
-        type = GL_UNSIGNED_BYTE;
-        break;
-      default:
-        return; // invalid format
-    }
-    glRasterPos2i (Area->x, Area->y);
-    glDrawPixels (Area->w, Area->h, format, type, Area->data);
-    glFlush ();
-    if (Free)
-      FreeArea (Area);
-  } /* endif */
-
-  if (gl_alphaTest) statecache->Enable_GL_ALPHA_TEST ();
+  uint8 px[4];
+  if (!hasRenderTarget)
+    y = vpHeight - y;
+  glReadPixels (x, y, 1, 1, GL_RGBA,
+    GL_UNSIGNED_BYTE, px);
+  oR = px[0];
+  oG = px[1];
+  oB = px[2];
+  oA = px[3];
 }
 
 csPtr<iImage> csGraphics2DGLCommon::ScreenShot ()
 {
   ((csGLFontCache*)fontCache)->FlushText ();
 
-/*#ifndef GL_VERSION_1_2
-  if (pfmt.PixelBytes != 1 && pfmt.PixelBytes != 4)
-    return 0;
-#endif*/
-
   // Need to resolve pixel alignment issues
   int screen_width = vpWidth * (4);
-  if (!screen_shot) screen_shot = new uint8 [screen_width * vpHeight];
+  uint8* screen_shot = new uint8 [screen_width * vpHeight];
   //if (!screen_shot) return 0;
 
   glReadPixels (0, 0, vpWidth, vpHeight, GL_RGBA,
@@ -795,6 +753,8 @@ csPtr<iImage> csGraphics2DGLCommon::ScreenShot ()
 
   csGLScreenShot* ss = GetScreenShot ();
   ss->SetData (screen_shot);
+  
+  delete[] screen_shot;
 
   return ss;
 }
@@ -875,22 +835,22 @@ bool csGraphics2DGLCommon::DebugCommand (const char* cmdstr)
       csRef<iDataBuffer> buf = imgsaver->Save (images[i], "image/png");
       if (!buf)
       {
-	Report (CS_REPORTER_SEVERITY_WARNING,
-	  "Could not save font cache page.");
+	      Report (CS_REPORTER_SEVERITY_WARNING,
+	        "Could not save font cache page.");
       }
       else
       {
-	outfn.Format ("%s%zu.png", dir, i);
-	if (!vfs->WriteFile (outfn, (char*)buf->GetInt8 (), buf->GetSize ()))
-	{
-	  Report (CS_REPORTER_SEVERITY_WARNING,
-	    "Could not write to %s.", outfn.GetData ());
-	}
-	else
-	{
-	  Report (CS_REPORTER_SEVERITY_NOTIFY,
-	    "Dumped font cache page to %s", outfn.GetData ());
-	}
+	      outfn.Format ("%s%zu.png", dir, i);
+	      if (!vfs->WriteFile (outfn, (char*)buf->GetInt8 (), buf->GetSize ()))
+	      {
+	        Report (CS_REPORTER_SEVERITY_WARNING,
+	          "Could not write to %s.", outfn.GetData ());
+	      }
+	      else
+	      {
+	        Report (CS_REPORTER_SEVERITY_NOTIFY,
+	          "Dumped font cache page to %s", outfn.GetData ());
+	      }
       }
     }
 
@@ -931,7 +891,10 @@ bool csGraphics2DGLCommon::Resize (int width, int height)
   }
   fbWidth = width;
   fbHeight = height;
-  EventOutlet->Broadcast (csevCanvasResize(object_reg, this), (intptr_t)this);
+  if (openComplete)
+    /* csGraphics2D::Open() causes a resize due to fitting to the working area;
+     * don't emit a resize event in that case */
+    EventOutlet->Broadcast (csevCanvasResize(object_reg, this), (intptr_t)this);
   return true;
 }
 
@@ -1023,7 +986,7 @@ void csGraphics2DGLCommon::csGLPixelFormatPicker::ReadPickerValue (
       int val;
       if (sscanf (currentVal, "%d%c", &val, &dummy) == 1)
       {
-	values.Push (val);
+	      values.Push (val);
       }
       currentVal = comma ? comma + 1 : 0;
     }
