@@ -277,7 +277,7 @@ static void ShowBlockInfo (csMemTrackerEntry& me)
   csPrintf ("BLOCK: start=%08p size=%zu freed=%d\n", me.start,
         me.size, (int)me.freed);
 # ifdef CS_EXTENSIVE_MEMDEBUG_IMPLEMENT
-  csPrintf ("       alloced at '%s' %d\n", me.alloc_file, me.alloc_line);
+  csPrintf ("       alloced at %s %d\n", CS::Quote::Single (me.alloc_file), me.alloc_line);
 # endif
 }
 
@@ -998,7 +998,8 @@ namespace CS
 	{
 	  if (!iSCF::SCF)
 	  {
-	    csPrintf ("iSCF::SCF not set yet!\n");
+	    csPrintf ("%s for %s: iSCF::SCF not set yet!\n",
+	      CS_FUNCTION_NAME, Class);
 	    return;
 	  }
 	
@@ -1069,6 +1070,14 @@ namespace CS
 	  else if (dcount < 0)
 	    mti->totalDeallocCount++;
 	}
+	const char* GetInfo (csMemTrackerModule* m, void* p)
+	{
+	  if (!m) return 0;
+	  csMemTrackerModule::BlockInfo* bi =
+	    m->blockSizes.GetElementPointer (p);
+	  if (!bi) return 0;
+	  return bi->info;
+	}
       } // namespace Impl
     } // namespace MemTracker
   } // namespace Debug
