@@ -36,5 +36,20 @@ namespace CS {
       return memorySize;
 
     } // End GetPhysicalMemorySize()      
+
+    size_t GetMaxVirtualSize()
+    {
+      static size_t memorySize = 0;
+      static bool cacheValid = false;
+
+      if (!cacheValid)
+      {
+        memorySize = CS::Platform::Implementation::GetMaxVirtualSize();
+        cacheValid = true;
+      }
+
+      return memorySize;
+
+    }
   } // End namespace Platform
 } // End namespace CS
