@@ -20,140 +20,59 @@
 #include "cssysdef.h"
 #include "csqint.h"
 
-#include "csgfx/imagecubemapmaker.h"
-#include "csgfx/rgbpixel.h"
-#include "cstool/proctex.h"
-#include "csutil/array.h"
-#include "csutil/cscolor.h"
-#include "csutil/scanstr.h"
-#include "iengine/engine.h"
-#include "iengine/material.h"
-#include "iengine/region.h"
-#include "iengine/texture.h"
-#include "igraphic/animimg.h"
-#include "imap/ldrctxt.h"
-#include "imap/reader.h"
-#include "iutil/document.h"
-#include "iutil/objreg.h"
-#include "iutil/strset.h"
-#include "iutil/vfs.h"
-#include "ivaria/keyval.h"
-#include "ivaria/reporter.h"
-#include "ivideo/graph3d.h"
-#include "ivideo/material.h"
-#include "ivideo/shader/shader.h"
-#include "ivideo/texture.h"
+#include "csgfx/imagememory.h"
+#include "cstool/vfsdirchange.h"
+#include "csutil/stringquote.h"
 
+#include "igraphic/animimg.h"
+#include "imap/services.h"
+#include "itexture/iproctex.h"
+#include "iutil/stringarray.h"
+#include "iutil/vfs.h"
+#include "ivaria/reporter.h"
+#include "ivideo/material.h"
+
+#include "csthreadedloader.h"
 #include "loadtex.h"
-#include "csloader.h"
 
 CS_PLUGIN_NAMESPACE_BEGIN(csparser)
 {
-
-#define PLUGIN_LEGACY_TEXTYPE_PREFIX  "crystalspace.texture.loader."
-
-  bool csLoader::ParseMaterialList (iLoaderContext* ldr_context,
-    iDocumentNode* node, const char* prefix)
+  THREADED_CALLABLE_IMPL4(csThreadedLoader, ParseTexture, csRef<iLoaderContext> ldr_context,
+    csRef<iDocumentNode> node, csSafeCopyArray<ProxyTexture>* proxyTextures, const char* path)
   {
-    if (!Engine) return false;
+    csVfsDirectoryChanger dirChange(vfs);
+    dirChange.ChangeTo(path);
 
-    csRef<iDocumentNodeIterator> it = node->GetNodes ();
-    while (it->HasNext ())
-    {
-      csRef<iDocumentNode> child = it->Next ();
-      if (child->GetType () != CS_NODE_ELEMENT) continue;
-      const char* value = child->GetValue ();
-      csStringID id = xmltokens.Request (value);
-      switch (id)
-      {
-      case XMLTOKEN_MATERIAL:
-        if (!ParseMaterial (ldr_context, child, prefix))
-          return false;
-        break;
-      default:
-        SyntaxService->ReportBadToken (child);
-        return false;
-      }
-    }
-
-    return true;
-  }
-
-  bool csLoader::ParseTextureList (iLoaderContext* ldr_context, iDocumentNode* node)
-  {
-    if (!ImageLoader)
-    {
-      SyntaxService->ReportError (
-        "crystalspace.maploader.parse.textures",
-        node, "Image loader is missing!");
-      return false;
-    }
-    static bool proctex_deprecated_warned = false;
-
-    csRef<iDocumentNodeIterator> it = node->GetNodes ();
-    while (it->HasNext ())
-    {
-      csRef<iDocumentNode> child = it->Next ();
-      if (child->GetType () != CS_NODE_ELEMENT) continue;
-      const char* value = child->GetValue ();
-      csStringID id = xmltokens.Request (value);
-      switch (id)
-      {
-      case XMLTOKEN_PROCTEX:
-        /*        if (!ParseProcTex (child))
-        return false;
-        break;*/
-        if (!proctex_deprecated_warned)
-        {
-          SyntaxService->Report (
-            "crystalspace.maploader.parse.texture",
-            CS_REPORTER_SEVERITY_NOTIFY,
-            child,
-            "Use of <proctex> is deprecated. "
-            "Procedural textures can now be specified with the <texture> node as well.");
-          proctex_deprecated_warned = true;
-        }
-      case XMLTOKEN_TEXTURE:
-        if (!ParseTexture (ldr_context, child))
-          return false;
-        break;
-      case XMLTOKEN_CUBEMAP:
-        if (!ParseCubemap (ldr_context, child))
-          return false;
-        break;
-      case XMLTOKEN_TEXTURE3D:
-        if (!ParseTexture3D (ldr_context, child))
-          return false;
-        break;
-      default:
-        SyntaxService->ReportBadToken (child);
-        return false;
-      }
-    }
-
-    return true;
-  }
-
-  iTextureWrapper* csLoader::ParseTexture (iLoaderContext* ldr_context,
-    iDocumentNode* node)
-  {
     const char* txtname = node->GetAttributeValue ("name");
-    if (ldr_context->CheckDupes ())
+
+    iTextureWrapper* t = ldr_context->FindTexture (txtname, false);
+    if (t)
     {
-      iTextureWrapper* t = Engine->FindTexture (txtname);
-      if (t)
-      {
-        AddToRegionOrCollection (ldr_context, t->QueryObject ());
-        return t;
-      }
+      ldr_context->AddToCollection(t->QueryObject ());
+      ret->SetResult(scfQueryInterfaceSafe<iBase>(t));
+      return true;
     }
 
-    static bool deprecated_warned = false;
+    if(!AddLoadingTexture(txtname))
+    {
+      // Fixme.
+      while(!t)
+      {
+        t = ldr_context->FindTexture (txtname, false);
+      }
+
+      ldr_context->AddToCollection(t->QueryObject ());
+      ret->SetResult(scfQueryInterfaceSafe<iBase>(t));
+      return true;
+    }
 
     csRef<iTextureWrapper> tex;
     csRef<iLoaderPlugin> plugin;
 
     csString filename = node->GetAttributeValue ("file");
+    enum { None, Filename, Color } imageSourceType = None;
+    if (!filename.IsEmpty()) imageSourceType = Filename;
+    csColor4 singleColor (0, 0, 0);
     csColor transp (0, 0, 0);
     bool do_transp = false;
     bool keep_image = false;
@@ -182,7 +101,10 @@ CS_PLUGIN_NAMESPACE_BEGIN(csparser)
         {
           bool for2d;
           if (!SyntaxService->ParseBool (child, for2d, true))
-            return 0;
+          {
+            RemoveLoadingTexture(txtname);
+            return false;
+          }
           if (for2d)
             context.SetFlags (context.GetFlags() | CS_TEXTURE_2D);
           else
@@ -193,7 +115,10 @@ CS_PLUGIN_NAMESPACE_BEGIN(csparser)
         {
           bool for3d;
           if (!SyntaxService->ParseBool (child, for3d, true))
-            return 0;
+          {
+            RemoveLoadingTexture(txtname);
+            return false;
+          }
           if (for3d)
             context.SetFlags (context.GetFlags() | CS_TEXTURE_3D);
           else
@@ -203,7 +128,7 @@ CS_PLUGIN_NAMESPACE_BEGIN(csparser)
       case XMLTOKEN_TRANSPARENT:
         do_transp = true;
         if (!SyntaxService->ParseColor (child, transp))
-          return 0;
+          return false;
         break;
       case XMLTOKEN_FILE:
         {
@@ -212,37 +137,59 @@ CS_PLUGIN_NAMESPACE_BEGIN(csparser)
           {
             SyntaxService->ReportError (
               "crystalspace.maploader.parse.texture",
-              child, "Expected VFS filename for 'file'!");
-            return 0;
+              child, "Expected VFS filename for %s!",
+	      CS::Quote::Single ("file"));
+            RemoveLoadingTexture(txtname);
+            return false;
           }
+          if (imageSourceType == Color)
+          {
+            SyntaxService->ReportError (
+              "crystalspace.maploader.parse.texture",
+              child, "<file> is specified, but <color> was specified earlier");
+          }
+          imageSourceType = Filename;
           filename = fname;
         }
         break;
+      case XMLTOKEN_COLOR:
+        {
+          if (!SyntaxService->ParseColor (child, singleColor))
+          {
+            RemoveLoadingTexture(txtname);
+            return false;
+          }
+          if (imageSourceType == Filename)
+          {
+            SyntaxService->ReportError (
+              "crystalspace.maploader.parse.texture",
+              child, "<color> is specified, but <file> was specified earlier");
+          }
+          imageSourceType = Color;
+        }
+        break;
       case XMLTOKEN_MIPMAP:
         {
           bool mm;
           if (!SyntaxService->ParseBool (child, mm, true))
-            return 0;
+          {
+            RemoveLoadingTexture(txtname);
+            return false;
+          }
           if (!mm)
             context.SetFlags (context.GetFlags() | CS_TEXTURE_NOMIPMAPS);
           else
             context.SetFlags (context.GetFlags() & ~CS_TEXTURE_NOMIPMAPS);
         }
         break;
-      case XMLTOKEN_DITHER:
-        {
-          SyntaxService->Report (
-            "crystalspace.maploader.parse.texture",
-            CS_REPORTER_SEVERITY_NOTIFY,
-            node,
-            "\"dither\" flag has been dropped");
-        }
-        break;
       case XMLTOKEN_NPOTS:
         {
           bool npots;
           if (!SyntaxService->ParseBool (child, npots, true))
-            return 0;
+          {
+            RemoveLoadingTexture(txtname);
+            return false;
+          }
           if (npots)
             context.SetFlags (context.GetFlags() | CS_TEXTURE_NPOTS);
           else
@@ -252,7 +199,10 @@ CS_PLUGIN_NAMESPACE_BEGIN(csparser)
       case XMLTOKEN_KEEPIMAGE:
         {
           if (!SyntaxService->ParseBool (child, keep_image, true))
-            return 0;
+          {
+            RemoveLoadingTexture(txtname);
+            return false;
+          }
         }
         break;
       case XMLTOKEN_PARAMS:
@@ -265,7 +215,8 @@ CS_PLUGIN_NAMESPACE_BEGIN(csparser)
           SyntaxService->ReportError (
             "crystalspace.maploader.parse.texture",
             child, "Expected plugin ID for <type>!");
-          return 0;
+          RemoveLoadingTexture(txtname);
+          return false;
         }
         break;
       case XMLTOKEN_SIZE:
@@ -281,13 +232,16 @@ CS_PLUGIN_NAMESPACE_BEGIN(csparser)
         break;
       case XMLTOKEN_ALWAYSANIMATE:
         if (!SyntaxService->ParseBool (child, always_animate, true))
-          return 0;
+          return false;
         break;
       case XMLTOKEN_CLAMP:
         {
           bool c;
           if (!SyntaxService->ParseBool (child, c, true))
-            return 0;
+          {
+            RemoveLoadingTexture(txtname);
+            return false;
+          }
           if (c)
             context.SetFlags (context.GetFlags() | CS_TEXTURE_CLAMP);
           else
@@ -298,7 +252,10 @@ CS_PLUGIN_NAMESPACE_BEGIN(csparser)
         {
           bool c;
           if (!SyntaxService->ParseBool (child, c, true))
-            return 0;
+          {
+            RemoveLoadingTexture(txtname);
+            return false;
+          }
           if (c)
             context.SetFlags (context.GetFlags() & ~CS_TEXTURE_NOFILTER);
           else
@@ -314,79 +271,109 @@ CS_PLUGIN_NAMESPACE_BEGIN(csparser)
         {
           csAlphaMode am;
           if (!SyntaxService->ParseAlphaMode (child, 0, am, false))
-            return 0;
+          {
+            RemoveLoadingTexture(txtname);
+            return false;
+          }
           overrideAlphaType = true;
           alphaType = am.alphaType;
         }
         break;
       default:
-        SyntaxService->ReportBadToken (child);
-        return 0;
+        {
+          SyntaxService->ReportBadToken (child);
+          RemoveLoadingTexture(txtname);
+          return false;
+        }
       }
     }
 
     csString texClass = context.GetClass();
 
-    // Proxy texture loading if the loader isn't specified
-    // and we don't need to load them immediately.
-    if(txtname && type.IsEmpty() && ldr_context->GetKeepFlags() == KEEP_USED &&
-       !ldr_context->GetRegion())
+    if (imageSourceType != Color)
     {
-      if (filename.IsEmpty())
+      // Proxy texture loading if the loader isn't specified
+      // and we don't need to load them immediately.
+      if(txtname && type.IsEmpty() && ldr_context->GetKeepFlags() == KEEP_USED &&
+	ldr_context->GetCollection())
       {
-        filename = txtname;
+	if (filename.IsEmpty())
+	{
+	  filename = txtname;
+	}
+  
+	// Get absolute path (on VFS) of the file.
+	csRef<iDataBuffer> absolutePath = vfs->ExpandPath(filename);
+	filename = absolutePath->GetData();
+  
+	ProxyTexture proxTex;
+	proxTex.img.AttachNew (new ProxyImage (this, filename, object_reg));
+	proxTex.always_animate = always_animate;
+  
+	tex = Engine->GetTextureList()->CreateTexture (proxTex.img);
+	tex->SetTextureClass(context.GetClass());
+	tex->SetFlags(context.GetFlags());
+	tex->QueryObject()->SetName(txtname);
+	AddTextureToList(tex);
+	RemoveLoadingTexture(txtname);
+  
+	proxTex.alphaType = csAlphaMode::alphaNone;
+	if(overrideAlphaType)
+	{
+	  proxTex.alphaType = alphaType;
+	}
+  
+	if(keep_image)
+	  tex->SetKeepImage(true);
+  
+	proxTex.keyColour.do_transp = do_transp;
+	if(do_transp)
+	{
+	  proxTex.keyColour.colours = transp;
+	}
+  
+	proxTex.textureWrapper = tex;
+	ldr_context->AddToCollection(proxTex.textureWrapper->QueryObject());
+	proxyTextures->Push(proxTex);
+	ret->SetResult(scfQueryInterfaceSafe<iBase>(proxTex.textureWrapper));
+  
+	return true;
       }
 
-      // Get absolute path (on VFS) of the file.
-      csRef<iDataBuffer> absolutePath = VFS->ExpandPath(filename);
-      filename = absolutePath->GetData();
-
-      ProxyTexture proxTex;
-      proxTex.img.AttachNew (new ProxyImage (this, filename));
-      proxTex.always_animate = always_animate;
-
-      tex = Engine->GetTextureList()->NewTexture (proxTex.img);
-      tex->SetTextureClass(context.GetClass());
-      tex->SetFlags(context.GetFlags());
-      tex->QueryObject()->SetName(txtname);
-
-      proxTex.alphaType = csAlphaMode::alphaNone;
-      if(overrideAlphaType)
+      // @@@ some more comments
+      if (type.IsEmpty () && filename.IsEmpty ())
       {
-        proxTex.alphaType = alphaType;
+	filename = txtname;
       }
-
-      if(keep_image)
-        tex->SetKeepImage(true);
-
-      proxTex.keyColour.do_transp = do_transp;
-      if(do_transp)
-      {
-        proxTex.keyColour.colours = transp;
-      }
-
-      proxTex.textureWrapper = tex;
-      AddToRegionOrCollection (ldr_context, proxTex.textureWrapper->QueryObject());
-      proxyTextures.Push(proxTex);
-
-      return tex;
     }
 
-    // @@@ some more comments
-    if (type.IsEmpty () && filename.IsEmpty ())
-    {
-      filename = txtname;
-    }
-
-    iTextureManager* tm;
-    tm = G3D ? G3D->GetTextureManager() : 0;
+    iTextureManager* texman;
+    texman = g3d ? g3d->GetTextureManager() : 0;
     int Format;
-    Format = tm ? tm->GetTextureFormat () : CS_IMGFMT_TRUECOLOR;
-    if (!filename.IsEmpty ())
+    Format = texman ? texman->GetTextureFormat () : CS_IMGFMT_TRUECOLOR;
+    csRef<iLoaderPlugin> BuiltinImageTexLoader;
+    if (imageSourceType == Color)
     {
-      csRef<iImage> image = LoadImage (filename, Format);
+      csRGBpixel singlePixel (int (singleColor.red * 255),
+        int (singleColor.green * 255),
+        int (singleColor.blue * 255),
+        int (singleColor.alpha * 255));
+      csRef<iImage> image;
+      image.AttachNew (new csImageMemory (1, 1, (const void*)&singlePixel, Format));
       context.SetImage (image);
-      if (image && type.IsEmpty ())
+    }
+    else if (!filename.IsEmpty ())
+    {
+      csRef<iThreadReturn> ret = csPtr<iThreadReturn>(new csLoaderReturn(threadman));
+      if(!LoadImageTC (ret, false, vfs->GetCwd(), filename, Format, false))
+      {
+        SyntaxService->Report("crystalspace.maploader.parse.texture",
+          CS_REPORTER_SEVERITY_WARNING, node, "Could not load image %s!", filename.GetData());
+      }
+
+      csRef<iImage> image = scfQueryInterfaceSafe<iImage>(ret->GetResultRefPtr());
+      context.SetImage (image);
+      if (image.IsValid() && type.IsEmpty ())
       {
         // special treatment for animated textures
         csRef<iAnimatedImage> anim = scfQueryInterface<iAnimatedImage> (image);
@@ -394,59 +381,32 @@ CS_PLUGIN_NAMESPACE_BEGIN(csparser)
         {
           type = PLUGIN_TEXTURELOADER_ANIMIMG;
         }
-        else
-        {
-          // shortcut, no need to go through the plugin list facility
-          if (!BuiltinImageTexLoader)
-          {
-            csImageTextureLoader* itl = new csImageTextureLoader (0);
-            itl->Initialize (object_reg);
-            BuiltinImageTexLoader.AttachNew (itl);
-          }
-          plugin = BuiltinImageTexLoader;
-        }
       }
     }
+    /* If an image but no texture type is given use the builtin image texture
+       loader */
+    if ((context.GetImage() != 0) && type.IsEmpty ())
+    {
+      csImageTextureLoader* itl = new csImageTextureLoader (0);
+      itl->Initialize (object_reg);
+      BuiltinImageTexLoader.AttachNew(itl);
+      plugin = BuiltinImageTexLoader;
+    }
 
-    iLoaderPlugin* Plug; Plug = 0;
     iBinaryLoaderPlugin* Binplug;
     if ((!type.IsEmpty ()) && !plugin)
     {
       iDocumentNode* defaults = 0;
-      if (!loaded_plugins.FindPlugin (type, Plug, Binplug, defaults))
-      {
-        if ((type == "dots") ||
-          (type == "plasma") ||
-          (type == "water") ||
-          (type == "fire"))
-        {
-          // old style proctex type
-          if (!deprecated_warned)
-          {
-            SyntaxService->Report (
-              "crystalspace.maploader.parse.texture",
-              CS_REPORTER_SEVERITY_NOTIFY,
-              node,
-              "Deprecated syntax used for proctex! "
-              "Specify a plugin classid or map the old types to their "
-              "plugin counterparts in the <plugins> node.");
-            deprecated_warned = true;
-          }
-
-          csString newtype = PLUGIN_LEGACY_TEXTYPE_PREFIX;
-          newtype += type;
-          type = newtype;
-
-          loaded_plugins.FindPlugin (type, Plug, Binplug, defaults);
-        }
-      }
-      plugin = Plug;
+      iLoaderPlugin* Plugin;
+      loaded_plugins.FindPlugin (type, Plugin, Binplug, defaults);
+      plugin = Plugin;
 
       if (defaults != 0)
       {
         ReportWarning (
           "crystalspace.maploader.parse.texture",
-          node, "'defaults' section is ignored for textures!");
+          node, "%s section is ignored for textures!",
+	  CS::Quote::Single ("defaults"));
       }
     }
 
@@ -455,7 +415,8 @@ CS_PLUGIN_NAMESPACE_BEGIN(csparser)
       SyntaxService->Report (
         "crystalspace.maploader.parse.texture",
         CS_REPORTER_SEVERITY_WARNING,
-        node, "Could not get plugin '%s', using default", (const char*)type);
+        node, "Could not get plugin %s, using default",
+	CS::Quote::Single ((const char*)type));
 
       if (!BuiltinImageTexLoader)
       {
@@ -477,22 +438,11 @@ CS_PLUGIN_NAMESPACE_BEGIN(csparser)
       SyntaxService->Report (
         "crystalspace.maploader.parse.texture",
         CS_REPORTER_SEVERITY_WARNING,
-        node, "Could not load texture '%s', using checkerboard instead", txtname);
+        node, "Could not load texture %s, using checkerboard instead",
+	CS::Quote::Single (txtname));
 
-      /*if (!BuiltinCheckerTexLoader)
-      {
-      csCheckerTextureLoader* ctl = new csCheckerTextureLoader (0);
-      ctl->Initialize (object_reg);
-      BuiltinCheckerTexLoader.AttachNew (ctl);
-      }
-      csRef<iBase> b = BuiltinCheckerTexLoader->Parse (ParamsNode,
-      0, ldr_context, static_cast<iBase*> (&context));*/
-      if (!BuiltinErrorTexLoader)
-      {
-        csMissingTextureLoader* mtl = new csMissingTextureLoader (0);
-        mtl->Initialize (object_reg);
-        BuiltinErrorTexLoader.AttachNew (mtl);
-      }
+      csRef<iLoaderPlugin> BuiltinErrorTexLoader;
+      BuiltinErrorTexLoader.AttachNew(new csMissingTextureLoader (object_reg));
       csRef<iBase> b = BuiltinErrorTexLoader->Parse (ParamsNode,
         0, ldr_context, static_cast<iBase*> (&context));
       if (!b.IsValid())
@@ -505,7 +455,8 @@ CS_PLUGIN_NAMESPACE_BEGIN(csparser)
             CS_REPORTER_SEVERITY_ERROR,
             node, "Could not create default texture!");
           noMissingWarned = true;
-          return 0;
+          RemoveLoadingTexture(txtname);
+          return false;
         }
       }
       tex = scfQueryInterface<iTextureWrapper> (b);
@@ -527,44 +478,56 @@ CS_PLUGIN_NAMESPACE_BEGIN(csparser)
       csRef<iProcTexture> ipt = scfQueryInterface<iProcTexture> (tex);
       if (ipt)
         ipt->SetAlwaysAnimate (always_animate);
-      AddToRegionOrCollection (ldr_context, tex->QueryObject ());
+      ldr_context->AddToCollection(tex->QueryObject ());
 
       size_t i;
       for (i = 0 ; i < key_nodes.GetSize () ; i++)
       {
         if (!ParseKey (key_nodes[i], tex->QueryObject()))
-          return 0;
+        {
+          RemoveLoadingTexture(txtname);
+          return false;
+        }
       }
     }
 
-    if (tm)
+    if (texman)
     {
-      if (!tex->GetTextureHandle ()) tex->Register (tm);
+      if (!tex->GetTextureHandle ()) tex->Register (texman);
     }
-    return tex;
+
+    AddTextureToList(tex);
+    RemoveLoadingTexture(txtname);
+    ret->SetResult(scfQueryInterfaceSafe<iBase>(tex));
+    return true;
   }
 
-  iMaterialWrapper* csLoader::ParseMaterial (iLoaderContext* ldr_context,
-    iDocumentNode* node, const char *prefix)
+  bool csThreadedLoader::ParseMaterial (iLoaderContext* ldr_context,
+    iDocumentNode* node, csWeakRefArray<iMaterialWrapper> &materialArray, const char *prefix)
   {
-    if (!Engine) return 0;
-
     const char* matname = node->GetAttributeValue ("name");
-    if (ldr_context->CheckDupes ())
+    iMaterialWrapper* m = ldr_context->FindMaterial (matname, false);
+    if (m)
     {
-      iMaterialWrapper* m = Engine->FindMaterial (matname);
-      if (m)
-      {
-        AddToRegionOrCollection (ldr_context, m->QueryObject ());
-        return m;
-      }
+      ldr_context->AddToCollection(m->QueryObject ());
+      return true;
     }
 
-    iTextureWrapper* texh = 0;
-    bool col_set = false;
+    if(!AddLoadingMaterial(matname))
+    {
+      // Fixme.
+      while(!m)
+      {
+        m = ldr_context->FindMaterial (matname, false);
+      }
+
+      ldr_context->AddToCollection(m->QueryObject ());
+      return true;
+    }
+
+    csRef<iTextureWrapper> texh = 0;
     csColor col;
 
-    bool shaders_mentioned = false;	// If true there were shaders.
     csArray<csStringID> shadertypes;
     csArray<iShader*> shaders;
     csRefArray<csShaderVariable> shadervars;
@@ -589,29 +552,24 @@ CS_PLUGIN_NAMESPACE_BEGIN(csparser)
           texh = ldr_context->FindTexture (txtname);
           if (!texh)
           {
-            ReportError (
-              "crystalspace.maploader.parse.material",
-              "Cannot find texture '%s' for material `%s'", txtname, matname);
-            return 0;
+            csRef<iThreadReturn> itr = csPtr<iThreadReturn>(new csLoaderReturn(threadman));
+            LoadTextureTC(itr, false, vfs->GetCwd(), txtname, txtname, CS_TEXTURE_3D, 0, true, false, true,
+              ldr_context->GetCollection(), ldr_context->GetKeepFlags(), true);
+            texh = scfQueryInterface<iTextureWrapper>(itr->GetResultRefPtr());
+            if(!texh)
+            {
+              ReportError (
+                "crystalspace.maploader.parse.material",
+                "Cannot find texture %s for material %s",
+		CS::Quote::Single (txtname), CS::Quote::Single (matname));
+              RemoveLoadingMaterial(matname);
+              return false;
+            }
           }
         }
         break;
-      case XMLTOKEN_COLOR:
-        {
-          col_set = true;
-          if (!SyntaxService->ParseColor (child, col))
-            return 0;
-        }
-        break;
-      case XMLTOKEN_DIFFUSE:
-      case XMLTOKEN_AMBIENT:
-      case XMLTOKEN_REFLECTION:
-        ReportWarning ("crystalspace.maploader.parse.material",
-          child, "Syntax not supported any more. Use shader variables instead");
-        break;
       case XMLTOKEN_SHADER:
         {
-          shaders_mentioned = true;
           csRef<iShaderManager> shaderMgr = 
             csQueryRegistry<iShaderManager> (object_reg);
           if (!shaderMgr)
@@ -637,8 +595,6 @@ CS_PLUGIN_NAMESPACE_BEGIN(csparser)
             break;
           }
           shadertypes.Push (stringSet->Request(shadertype));
-          //shaders.Push (shader);
-          //csRef<iShaderWrapper> wrapper = shaderMgr->CreateWrapper (shader);
           shaders.Push (shader);
         }
         break;
@@ -656,21 +612,17 @@ CS_PLUGIN_NAMESPACE_BEGIN(csparser)
         }
         break;
       default:
-        SyntaxService->ReportBadToken (child);
-        return 0;
+        {
+          SyntaxService->ReportBadToken (child);
+          RemoveLoadingMaterial(matname);
+          return false;
+        }
       }
     }
 
     csRef<iMaterial> material = Engine->CreateBaseMaterial (texh);
 
-    if (col_set)
-    {
-      csShaderVariable* flatSV = material->GetVariableAdd (
-        stringSet->Request (CS_MATERIAL_VARNAME_FLATCOLOR));
-      flatSV->SetValue (col);
-    }
-
-    iMaterialWrapper *mat;
+    csRef<iMaterialWrapper> mat;
 
     if (prefix)
     {
@@ -678,13 +630,15 @@ CS_PLUGIN_NAMESPACE_BEGIN(csparser)
       strcpy (prefixedname, prefix);
       strcat (prefixedname, "_");
       strcat (prefixedname, matname);
-      mat = Engine->GetMaterialList ()->NewMaterial (material, prefixedname);
+      mat = Engine->GetMaterialList ()->CreateMaterial (material, prefixedname);
       delete [] prefixedname;
     }
     else
     {
-      mat = Engine->GetMaterialList ()->NewMaterial (material, matname);
+      mat = Engine->GetMaterialList ()->CreateMaterial (material, matname);
     }
+    AddMaterialToList(mat);
+    RemoveLoadingMaterial(matname);
 
     size_t i;
     for (i=0; i<shaders.GetSize (); i++)
@@ -698,87 +652,13 @@ CS_PLUGIN_NAMESPACE_BEGIN(csparser)
     for (i = 0 ; i < key_nodes.GetSize () ; i++)
     {
       if (!ParseKey (key_nodes[i], mat->QueryObject()))
-        return 0;
+        return false;
     }
-    AddToRegionOrCollection (ldr_context, mat->QueryObject ());
+    ldr_context->AddToCollection(mat->QueryObject ());
 
     materialArray.Push(mat);
 
-    return mat;
+    return true;
   }
-
-  /// Parse a Cubemap texture definition and add the texture to the engine
-  iTextureWrapper* csLoader::ParseCubemap (iLoaderContext* ldr_context,
-    iDocumentNode* node)
-  {
-    static bool cubemapDeprecationWarning = false;
-    if (!cubemapDeprecationWarning)
-    {
-      cubemapDeprecationWarning = true;
-      SyntaxService->Report ("crystalspace.maploader.parse.texture",
-        CS_REPORTER_SEVERITY_WARNING, node,
-        "'<cubemap>...' is deprecated, use '<texture><type>"
-        PLUGIN_TEXTURELOADER_CUBEMAP "</type><params>...' instead");
-    }
-
-    csRef<csCubemapTextureLoader> plugin;
-    plugin.AttachNew (new csCubemapTextureLoader (0));
-    plugin->Initialize (object_reg);
-
-    csRef<TextureLoaderContext> context;
-    const char* txtname = node->GetAttributeValue ("name");
-    context.AttachNew (new TextureLoaderContext (txtname));
-
-    csRef<iBase> b = plugin->Parse (node, 0/*ssource*/, ldr_context, context);
-    csRef<iTextureWrapper> tex;
-    if (b) tex = scfQueryInterface<iTextureWrapper> (b);
-
-    if (tex)
-    {
-      tex->QueryObject ()->SetName (txtname);
-      AddToRegionOrCollection (ldr_context, tex->QueryObject ());
-      iTextureManager* tm = G3D ? G3D->GetTextureManager() : 0;
-      if (tm) tex->Register (tm);
-    }
-
-    return tex;
-  }
-
-  iTextureWrapper* csLoader::ParseTexture3D (iLoaderContext* ldr_context,
-    iDocumentNode* node)
-  {
-    static bool volmapDeprecationWarning = false;
-    if (!volmapDeprecationWarning)
-    {
-      volmapDeprecationWarning = true;
-      SyntaxService->Report ("crystalspace.maploader.parse.texture",
-        CS_REPORTER_SEVERITY_WARNING, node,
-        "'<texture3d>...' is deprecated, use '<texture><type>"
-        PLUGIN_TEXTURELOADER_TEX3D "</type><params>...' instead");
-    }
-
-    csRef<csTexture3DLoader> plugin;
-    plugin.AttachNew (new csTexture3DLoader (0));
-    plugin->Initialize (object_reg);
-
-    csRef<TextureLoaderContext> context;
-    const char* txtname = node->GetAttributeValue ("name");
-    context.AttachNew (new TextureLoaderContext (txtname));
-
-    csRef<iBase> b = plugin->Parse (node, 0/*ssource*/, ldr_context, context);
-    csRef<iTextureWrapper> tex;
-    if (b) tex = scfQueryInterface<iTextureWrapper> (b);
-
-    if (tex)
-    {
-      tex->QueryObject ()->SetName (txtname);
-      AddToRegionOrCollection (ldr_context, tex->QueryObject ());
-      iTextureManager* tm = G3D ? G3D->GetTextureManager() : 0;
-      if (tm) tex->Register (tm);
-    }
-
-    return tex;
-  }
-
 }
 CS_PLUGIN_NAMESPACE_END(csparser)
