@@ -28,6 +28,7 @@
 #include "csutil/csendian.h"
 #include "csutil/databuf.h"
 #include "csutil/dirtyaccessarray.h"
+#include "csutil/stringquote.h"
 #include "cstool/rbuflock.h"
 
 #include "syntxldr.h"
@@ -60,6 +61,34 @@ struct vhFloat
   { node->SetAttributeAsFloat (attr, v); }  
 };
 
+namespace
+{
+  /* Wrapper class to provide operations for FillBuffer(), BufferWriter,
+     BufferParser */
+  struct Half
+  {
+    uint16 v;
+    
+    Half () {}
+    Half (float f) : v (csIEEEfloat::FromNativeRTZ (f)) {}
+    
+    operator float() const
+    {
+      return csIEEEfloat::ToNative (v);
+    }
+    
+    Half& operator= (float f)
+    {
+      v = csIEEEfloat::FromNativeRTZ (f);
+      return *this;
+    }
+    bool operator< (const Half& other) const
+    {
+      return csIEEEfloat::ToNative (v) < csIEEEfloat::ToNative (other.v);
+    }
+  };
+}
+
 template <class ValGetter>
 struct BufferParser
 {
@@ -78,8 +107,8 @@ struct BufferParser
 	&& (strcmp (child->GetValue(), "e") != 0)
 	&& (strcmp (child->GetValue(), "dongledome") != 0))
       {
-	GetBufferParseError()->Format ("unexpected node '%s'", 
-	  child->GetValue());
+	GetBufferParseError()->Format ("unexpected node %s", 
+	  CS::Quote::Single (child->GetValue()));
 	return GetBufferParseError()->GetData();
       }
       for (int c = 0; c < compNum; c++)
@@ -94,8 +123,6 @@ struct BufferParser
   }
 };
 
-
-
 template<typename T>
 static csRef<iRenderBuffer> FillBuffer (const csDirtyAccessArray<T>& buf,
                                         csRenderBufferComponentType compType,
@@ -180,7 +207,8 @@ csRef<iRenderBuffer> csTextSyntaxService::ParseRenderBuffer (iDocumentNode* node
     csRef<iDataBuffer> data = vfs->ReadFile (filename, false);
     if (!data.IsValid())
     {
-      ReportError (msgid, node, "could not read from '%s'", filename);
+      ReportError (msgid, node, "could not read from %s",
+		   CS::Quote::Single (filename));
       return 0;
     }
     return ReadRenderBuffer (data, KeepSaveInfo() ? filename : 0);
@@ -189,13 +217,15 @@ csRef<iRenderBuffer> csTextSyntaxService::ParseRenderBuffer (iDocumentNode* node
   const char* componentType = node->GetAttributeValue ("type");
   if (componentType == 0)
   {
-    ReportError (msgid, node, "no 'type' attribute");
+    ReportError (msgid, node, "no %s attribute",
+		 CS::Quote::Single ("type"));
     return 0;
   }
   int componentNum = node->GetAttributeValueAsInt ("components");
   if (componentNum <= 0)
   {
-    ReportError (msgid, node, "bogus 'components' attribute: %d", componentNum);
+    ReportError (msgid, node, "bogus %s attribute: %d",
+		 CS::Quote::Single ("components"), componentNum);
     return 0;
   }
   
@@ -318,9 +348,20 @@ csRef<iRenderBuffer> csTextSyntaxService::ParseRenderBuffer (iDocumentNode* node
       buffer = FillBuffer<double> (buf, CS_BUFCOMP_DOUBLE, componentNum, indexBuf);
     }
   }
+  else if ((strcmp (componentType, "half") == 0) 
+    || (strcmp (componentType, "h") == 0))
+  {
+    csDirtyAccessArray<Half> buf;
+    err = BufferParser<vhFloat>::Parse (node, componentNum, buf);
+    if (err == 0)
+    {
+      buffer = FillBuffer<Half> (buf, CS_BUFCOMP_HALF, componentNum, indexBuf);
+    }
+  }
   else
   {
-    ReportError (msgid, node, "unknown value for 'type': %s", componentType);
+    ReportError (msgid, node, "unknown value for %s: %s",
+		 CS::Quote::Single ("type"), componentType);
     return 0;
   }
   if (err != 0)
@@ -342,7 +383,8 @@ bool csTextSyntaxService::ParseRenderBuffer (iDocumentNode* node, iRenderBuffer*
     csRef<iDataBuffer> data = vfs->ReadFile (filename, false);
     if (!data.IsValid())
     {
-      ReportError (msgid, node, "could not read from '%s'", filename);
+      ReportError (msgid, node, "could not read from %s",
+		   CS::Quote::Single (filename));
       return false;
     }
     //return ReadRenderBuffer (data, KeepSaveInfo() ? filename : 0);
@@ -353,23 +395,28 @@ bool csTextSyntaxService::ParseRenderBuffer (iDocumentNode* node, iRenderBuffer*
   const char* componentType = node->GetAttributeValue ("type");
   if (componentType == 0)
   {
-    ReportError (msgid, node, "no 'type' attribute");
+    ReportError (msgid, node, "no %s attribute",
+		 CS::Quote::Single ("type"));
     return false;
   }  
 
   int componentNum = node->GetAttributeValueAsInt ("components");
   if (componentNum <= 0)
   {
-    ReportError (msgid, node, "bogus 'components' attribute: %d", componentNum);
+    ReportError (msgid, node, "bogus %s attribute: %d",
+		 CS::Quote::Single ("components"), componentNum);
     return false;
   }
   if (componentNum != buffer->GetComponentCount ())
   {
-    ReportError (msgid, node, "'components' attribute %d does not match expected %d", 
+    ReportError (msgid, node, "%s attribute %d does not match expected %d", 
+      CS::Quote::Single ("components"),
       componentNum, buffer->GetComponentCount ());
     return false;
   }
 
+  bool normalized = node->GetAttributeValueAsBool ("normalized", false);
+
   bool indexBuf = false;
   {
     const char* index = node->GetAttributeValue("indices");
@@ -400,7 +447,8 @@ bool csTextSyntaxService::ParseRenderBuffer (iDocumentNode* node, iRenderBuffer*
     csDirtyAccessArray<int> buf;
     err = BufferParser<vhInt>::Parse (node, componentNum, buf);
 
-    if (buffer->GetComponentType () != CS_BUFCOMP_INT)
+    if (buffer->GetComponentType () !=
+      (normalized ? CS_BUFCOMP_INT_NORM : CS_BUFCOMP_INT))
     {
       ReportError (msgid, node, "component type %s does not match expected %s",
         componentType, "int");
@@ -425,7 +473,8 @@ bool csTextSyntaxService::ParseRenderBuffer (iDocumentNode* node, iRenderBuffer*
     csDirtyAccessArray<uint> buf;
     err = BufferParser<vhInt>::Parse (node, componentNum, buf);
     
-    if (buffer->GetComponentType () != CS_BUFCOMP_UNSIGNED_INT)
+    if (buffer->GetComponentType () !=
+      (normalized ? CS_BUFCOMP_UNSIGNED_INT_NORM : CS_BUFCOMP_UNSIGNED_INT))
     {
       ReportError (msgid, node, "component type %s does not match expected %s",
         componentType, "uint");
@@ -449,7 +498,8 @@ bool csTextSyntaxService::ParseRenderBuffer (iDocumentNode* node, iRenderBuffer*
     csDirtyAccessArray<char> buf;
     err = BufferParser<vhInt>::Parse (node, componentNum, buf);
     
-    if (buffer->GetComponentType () != CS_BUFCOMP_BYTE)
+    if (buffer->GetComponentType () !=
+      (normalized ? CS_BUFCOMP_BYTE_NORM : CS_BUFCOMP_BYTE))
     {
       ReportError (msgid, node, "component type %s does not match expected %s",
         componentType, "byte");
@@ -473,7 +523,8 @@ bool csTextSyntaxService::ParseRenderBuffer (iDocumentNode* node, iRenderBuffer*
     csDirtyAccessArray<unsigned char> buf;
     err = BufferParser<vhInt>::Parse (node, componentNum, buf);
     
-    if (buffer->GetComponentType () != CS_BUFCOMP_UNSIGNED_BYTE)
+    if (buffer->GetComponentType () !=
+      (normalized ? CS_BUFCOMP_UNSIGNED_BYTE_NORM : CS_BUFCOMP_UNSIGNED_BYTE))
     {
       ReportError (msgid, node, "component type %s does not match expected %s",
         componentType, "ubyte");
@@ -497,7 +548,8 @@ bool csTextSyntaxService::ParseRenderBuffer (iDocumentNode* node, iRenderBuffer*
     csDirtyAccessArray<short> buf;
     err = BufferParser<vhInt>::Parse (node, componentNum, buf);
     
-    if (buffer->GetComponentType () != CS_BUFCOMP_SHORT)
+    if (buffer->GetComponentType () !=
+      (normalized ? CS_BUFCOMP_SHORT_NORM : CS_BUFCOMP_SHORT))
     {
       ReportError (msgid, node, "component type %s does not match expected %s",
         componentType, "short");
@@ -521,7 +573,8 @@ bool csTextSyntaxService::ParseRenderBuffer (iDocumentNode* node, iRenderBuffer*
     csDirtyAccessArray<unsigned short> buf;
     err = BufferParser<vhInt>::Parse (node, componentNum, buf);
     
-    if (buffer->GetComponentType () != CS_BUFCOMP_UNSIGNED_SHORT)
+    if (buffer->GetComponentType () != 
+      (normalized ? CS_BUFCOMP_UNSIGNED_SHORT_NORM : CS_BUFCOMP_UNSIGNED_SHORT))
     {
       ReportError (msgid, node, "component type %s does not match expected %s",
         componentType, "ushort");
@@ -587,9 +640,34 @@ bool csTextSyntaxService::ParseRenderBuffer (iDocumentNode* node, iRenderBuffer*
       buffer->CopyInto (buf.GetArray (), buf.GetSize ()/componentNum);
     }
   }
+  else if ((strcmp (componentType, "half") == 0) 
+    || (strcmp (componentType, "h") == 0))
+  {
+    csDirtyAccessArray<Half> buf;
+    err = BufferParser<vhFloat>::Parse (node, componentNum, buf);
+    
+    if (buffer->GetComponentType () != CS_BUFCOMP_HALF)
+    {
+      ReportError (msgid, node, "component type %s does not match expected %s",
+        componentType, "half");
+      return false;
+    }
+
+    if (err == 0)
+    {
+      if (buf.GetSize () > buffer->GetElementCount ()*componentNum)
+      {
+        ReportError (msgid, node, "too many elements: %d", buf.GetSize ());
+        return false;
+      }
+
+      buffer->CopyInto (buf.GetArray (), buf.GetSize ()/componentNum);
+    }
+  }
   else
   {
-    ReportError (msgid, node, "unknown value for 'type': %s", componentType);
+    ReportError (msgid, node, "unknown value for %s: %s",
+		 CS::Quote::Single ("type"), componentType);
     return 0;
   }
   if (err != 0)
@@ -632,7 +710,8 @@ bool csTextSyntaxService::WriteRenderBuffer (iDocumentNode* node, iRenderBuffer*
       csRef<iVFS> vfs = csQueryRegistry<iVFS> (object_reg);
       if (!vfs->WriteFile (filename, saveData->GetData(), saveData->GetSize()))
       {
-        ReportError (msgid, node, "could not write to '%s'", filename);
+        ReportError (msgid, node, "could not write to %s",
+		     CS::Quote::Single (filename));
         return false;
       }
       node->SetAttribute ("file", filename);
@@ -717,6 +796,14 @@ bool csTextSyntaxService::WriteRenderBuffer (iDocumentNode* node, iRenderBuffer*
         lock.GetSize ()*componentCount);
       break;
     }
+  case CS_BUFCOMP_HALF:
+    {
+      node->SetAttribute ("type", "half");
+      csRenderBufferLock<Half> lock (buffer, CS_BUF_LOCK_READ);
+      err = BufferWriter<vhFloat>::Write (node, componentCount, (Half*)lock, 
+        lock.GetSize ()*componentCount);
+      break;
+    }
     
   default:
     return false;
@@ -728,6 +815,84 @@ bool csTextSyntaxService::WriteRenderBuffer (iDocumentNode* node, iRenderBuffer*
   return true;
 }
 
+// Buffer component types, as stored on disk
+enum
+{
+  diskBUFCOMP_BYTE = 0,
+  diskBUFCOMP_UNSIGNED_BYTE,
+  diskBUFCOMP_SHORT,
+  diskBUFCOMP_UNSIGNED_SHORT,
+  diskBUFCOMP_INT,
+  diskBUFCOMP_UNSIGNED_INT,
+  diskBUFCOMP_FLOAT,
+  diskBUFCOMP_DOUBLE,
+  
+  diskBUFCOMP_NORMALIZED = 8,
+  
+  diskBUFCOMP_BYTE_NORM = CS_BUFCOMP_BYTE | CS_BUFCOMP_NORMALIZED,
+  diskBUFCOMP_UNSIGNED_BYTE_NORM =
+      CS_BUFCOMP_UNSIGNED_BYTE | CS_BUFCOMP_NORMALIZED,
+  diskBUFCOMP_SHORT_NORM =
+      CS_BUFCOMP_SHORT | CS_BUFCOMP_NORMALIZED,
+  diskBUFCOMP_UNSIGNED_SHORT_NORM =
+      CS_BUFCOMP_UNSIGNED_SHORT | CS_BUFCOMP_NORMALIZED,
+  diskBUFCOMP_INT_NORM = CS_BUFCOMP_INT | CS_BUFCOMP_NORMALIZED,
+  diskBUFCOMP_UNSIGNED_INT_NORM = 
+      CS_BUFCOMP_UNSIGNED_INT | CS_BUFCOMP_NORMALIZED,
+
+  /* When this was added value 8 was already pegged as the NORMALIZED flag,
+     so the next free bit (16) was used instead */
+  diskBUFCOMP_HALF = 16,
+  
+  diskBUFCOMP_BASE_TYPECOUNT
+};
+
+static uint8 ComponentTypeToDisk (unsigned int compType)
+{
+  uint8 compTypeDisk = 0;
+  
+  switch (compType & ~(CS_BUFCOMP_NORMALIZED))
+  {
+  case CS_BUFCOMP_BYTE:			compTypeDisk = diskBUFCOMP_BYTE;		break;
+  case CS_BUFCOMP_UNSIGNED_BYTE:	compTypeDisk = diskBUFCOMP_UNSIGNED_BYTE;	break;
+  case CS_BUFCOMP_SHORT:		compTypeDisk = diskBUFCOMP_SHORT;		break;
+  case CS_BUFCOMP_UNSIGNED_SHORT:	compTypeDisk = diskBUFCOMP_UNSIGNED_SHORT;	break;
+  case CS_BUFCOMP_INT:			compTypeDisk = diskBUFCOMP_INT;			break;
+  case CS_BUFCOMP_UNSIGNED_INT:		compTypeDisk = diskBUFCOMP_UNSIGNED_INT;	break;
+  case CS_BUFCOMP_FLOAT:		compTypeDisk = diskBUFCOMP_FLOAT;		break;
+  case CS_BUFCOMP_DOUBLE:		compTypeDisk = diskBUFCOMP_DOUBLE;		break;
+  case CS_BUFCOMP_HALF:			compTypeDisk = diskBUFCOMP_HALF;		break;
+  }
+  
+  if (compType & CS_BUFCOMP_NORMALIZED)
+    compTypeDisk |= diskBUFCOMP_NORMALIZED;
+  
+  return compTypeDisk;
+}
+
+static unsigned int DiskToComponentType (uint8 compTypeDisk)
+{
+  unsigned int compType = 0;
+  
+  switch (compTypeDisk & ~(diskBUFCOMP_NORMALIZED))
+  {
+  case diskBUFCOMP_BYTE:		compType = CS_BUFCOMP_BYTE;		break;
+  case diskBUFCOMP_UNSIGNED_BYTE:	compType = CS_BUFCOMP_UNSIGNED_BYTE;	break;
+  case diskBUFCOMP_SHORT:		compType = CS_BUFCOMP_SHORT;		break;
+  case diskBUFCOMP_UNSIGNED_SHORT:	compType = CS_BUFCOMP_UNSIGNED_SHORT;	break;
+  case diskBUFCOMP_INT:			compType = CS_BUFCOMP_INT;		break;
+  case diskBUFCOMP_UNSIGNED_INT:	compType = CS_BUFCOMP_UNSIGNED_INT;	break;
+  case diskBUFCOMP_FLOAT:		compType = CS_BUFCOMP_FLOAT;		break;
+  case diskBUFCOMP_DOUBLE:		compType = CS_BUFCOMP_DOUBLE;		break;
+  case diskBUFCOMP_HALF:		compType = CS_BUFCOMP_HALF;		break;
+  }
+  
+  if (compTypeDisk & diskBUFCOMP_NORMALIZED)
+    compType |= CS_BUFCOMP_NORMALIZED;
+  
+  return compType;
+}
+
 struct RenderBufferHeaderCommon
 {
   enum 
@@ -875,7 +1040,7 @@ public:
     if (IsEdited())
       return GetWrappedBuffer()->GetComponentType();
     else
-      return (csRenderBufferComponentType)header->compType;
+      return (csRenderBufferComponentType)DiskToComponentType (header->compType);
   }
 
   virtual csRenderBufferType GetBufferType() const
@@ -888,8 +1053,11 @@ public:
     if (IsEdited())
       return GetWrappedBuffer()->GetSize();
     else
-      return csRenderBufferComponentSizes[header->compType] 
+    {
+      unsigned int compType = DiskToComponentType (header->compType);
+      return csRenderBufferComponentSizes[compType & ~CS_BUFCOMP_NORMALIZED] 
 	* csLittleEndian::Convert (header->elementCount) * header->compCount;
+    }
   }
 
   virtual size_t GetStride() const 
@@ -902,7 +1070,10 @@ public:
     if (IsEdited())
       return GetWrappedBuffer()->GetElementDistance();
     else
-      return header->compCount * csRenderBufferComponentSizes[header->compType];
+    {
+      unsigned int compType = DiskToComponentType (header->compType);
+      return header->compCount * csRenderBufferComponentSizes[compType & ~CS_BUFCOMP_NORMALIZED];
+    }
   }
 
   virtual size_t GetOffset() const
@@ -1042,16 +1213,14 @@ static void ConvertBufferDataToLE (csRenderBufferComponentType compType,
 }
 #endif
 
-static const size_t RenderBufferComponentSizesOnDisk[CS_BUFCOMP_TYPECOUNT] = 
+static const size_t RenderBufferComponentSizesOnDisk[CS_BUFCOMP_BASE_TYPECOUNT] = 
 {
   sizeof (int8), sizeof (uint8), 
   sizeof (int16), sizeof (uint16),
   sizeof (int32), sizeof (uint32),
   sizeof (uint32),
   sizeof (uint64),
-  sizeof (int8), sizeof (uint8), 
-  sizeof (int16), sizeof (uint16),
-  sizeof (int32), sizeof (uint32)
+  sizeof (uint16)
 };
 
 csRef<iRenderBuffer> csTextSyntaxService::ReadRenderBuffer (iDataBuffer* buf, 
@@ -1071,8 +1240,9 @@ csRef<iRenderBuffer> csTextSyntaxService::ReadRenderBuffer (iDataBuffer* buf,
 
   const size_t totalElements = 
     header->compCount * csLittleEndian::Convert (header->elementCount);
+  unsigned int compType = DiskToComponentType (header->compType);
   const size_t totalSizeDisk = 
-    totalElements * RenderBufferComponentSizesOnDisk[header->compType];
+    totalElements * RenderBufferComponentSizesOnDisk[compType & ~CS_BUFCOMP_NORMALIZED];
 
   // Sanity check
   if (buf->GetSize() < (totalSizeDisk + headerSize))
@@ -1080,7 +1250,7 @@ csRef<iRenderBuffer> csTextSyntaxService::ReadRenderBuffer (iDataBuffer* buf,
 
 #if !defined(CS_LITTLE_ENDIAN) || !defined(CS_IEEE_DOUBLE_FORMAT)
   const size_t totalSize = 
-    totalElements * csRenderBufferComponentSizes[header->compType];
+    totalElements * csRenderBufferComponentSizes[compType & ~CS_BUFCOMP_NORMALIZED];
 
   // Buffer data is in little endian format, floats in IEEE, need to convert...
   csRef<iDataBuffer> newData;
@@ -1091,7 +1261,7 @@ csRef<iRenderBuffer> csTextSyntaxService::ReadRenderBuffer (iDataBuffer* buf,
   void* src = buf->GetData() + headerSize;
   void* dst = newData->GetData() + headerSize;
 
-  switch (header->compType)
+  switch (compType & ~CS_BUFCOMP_NORMALIZED)
   {
     case CS_BUFCOMP_FLOAT:
       ConvertFloatBufferDataFromLE ((uint32*)src, (float*)dst, totalElements);
@@ -1099,8 +1269,11 @@ csRef<iRenderBuffer> csTextSyntaxService::ReadRenderBuffer (iDataBuffer* buf,
     case CS_BUFCOMP_DOUBLE:
       ConvertFloatBufferDataFromLE ((uint64*)src, (double*)dst, totalElements);
       break;
+    case CS_BUFCOMP_HALF:
+      ConvertBufferDataToLE ((uint16*)src, (uint16*)dst, totalElements);
+      break;
     default:
-      ConvertBufferDataToLE ((csRenderBufferComponentType)header->compType,
+      ConvertBufferDataToLE ((csRenderBufferComponentType)compType,
         src, dst, totalElements);
   }
 
@@ -1130,7 +1303,7 @@ csRef<iDataBuffer> csTextSyntaxService::StoreRenderBuffer (iRenderBuffer* rbuf)
   const size_t totalElements = 
     rbuf->GetComponentCount() * rbuf->GetElementCount();
   const size_t totalSizeDisk = 
-    totalElements * RenderBufferComponentSizesOnDisk[rbuf->GetComponentType()];
+    totalElements * RenderBufferComponentSizesOnDisk[rbuf->GetComponentType() & ~CS_BUFCOMP_NORMALIZED];
 
 #if !defined(CS_LITTLE_ENDIAN) || !defined(CS_IEEE_DOUBLE_FORMAT)
   // Buffer data is in little endian format, floats in IEEE, need to convert...
@@ -1139,7 +1312,7 @@ csRef<iDataBuffer> csTextSyntaxService::StoreRenderBuffer (iRenderBuffer* rbuf)
   void* src = srcData;
   void* dst = newData->GetData();
 
-  switch (rbuf->GetComponentType ())
+  switch (rbuf->GetComponentType () & ~CS_BUFCOMP_NORMALIZED)
   {
     case CS_BUFCOMP_FLOAT:
       ConvertFloatBufferDataToLE ((float*)src, (uint32*)dst, totalElements);
@@ -1147,6 +1320,9 @@ csRef<iDataBuffer> csTextSyntaxService::StoreRenderBuffer (iRenderBuffer* rbuf)
     case CS_BUFCOMP_DOUBLE:
       ConvertFloatBufferDataToLE ((double*)src, (uint64*)dst, totalElements);
       break;
+    case CS_BUFCOMP_HALF:
+      ConvertBufferDataToLE ((uint16*)src, (uint16*)dst, totalElements);
+      break;
     default:
       ConvertBufferDataToLE (rbuf->GetComponentType(), src, dst, 
         totalElements);
@@ -1164,7 +1340,7 @@ csRef<iDataBuffer> csTextSyntaxService::StoreRenderBuffer (iRenderBuffer* rbuf)
 
   RenderBufferHeaderCommon commonHeader;
   commonHeader.magic = 0;
-  commonHeader.compType = rbuf->GetComponentType();
+  commonHeader.compType = ComponentTypeToDisk (rbuf->GetComponentType());
   commonHeader.compCount = rbuf->GetComponentCount();
   commonHeader.elementCount = 
     csLittleEndian::Convert (uint32 (rbuf->GetElementCount()));
