@@ -201,7 +201,7 @@ const char* csBinaryDocAttribute::GetValue ()
 	{
   	  char buf[50];
 	  cs_snprintf (buf, sizeof (buf) - 1,
-	    "%g", csLongToFloat (csLittleEndian::UInt32 (attrPtr->value)));
+	    "%g", node->doc->ConvertToFloat (csLittleEndian::UInt32 (attrPtr->value)));
 	  cs_free (vstr); 
 	  vstr = CS::StrDup (buf);
 	  vsptr = attrPtr;
@@ -231,7 +231,7 @@ int csBinaryDocAttribute::GetValueAsInt ()
       }
     case BD_VALUE_TYPE_FLOAT:
       {
-	return (int)csLongToFloat (csLittleEndian::UInt32 (attrPtr->value));
+	return (int)node->doc->ConvertToFloat (csLittleEndian::UInt32 (attrPtr->value));
       }
     default:
       return 0;
@@ -256,7 +256,7 @@ float csBinaryDocAttribute::GetValueAsFloat ()
       }
     case BD_VALUE_TYPE_FLOAT:
       {
-	return csLongToFloat (csLittleEndian::UInt32 (attrPtr->value));
+	return node->doc->ConvertToFloat (csLittleEndian::UInt32 (attrPtr->value));
       }
     default:
       return 0.0f;
@@ -287,7 +287,7 @@ bool csBinaryDocAttribute::GetValueAsBool ()
       }
     case BD_VALUE_TYPE_FLOAT:
       {
-	return (csLongToFloat (csLittleEndian::UInt32 (attrPtr->value))== 0);
+	return (node->doc->ConvertToFloat (csLittleEndian::UInt32 (attrPtr->value))== 0);
       }
     default:
       return false;
@@ -353,7 +353,7 @@ void csBinaryDocAttribute::SetValue (const char* val)
     {
       attrPtr->flags = (attrPtr->flags & ~BD_VALUE_TYPE_MASK) | 
 	BD_VALUE_TYPE_FLOAT;
-      attrPtr->value = csLittleEndian::UInt32 (csFloatToLong (f));
+      attrPtr->value = csLittleEndian::UInt32 (csIEEEfloat::FromNative (f));
     }
     else 
     {
@@ -384,13 +384,13 @@ void csBinaryDocAttribute::SetValueAsFloat (float f)
     cs_free (vstr); vstr = 0;
     attrPtr->flags = (attrPtr->flags & ~BD_VALUE_TYPE_MASK) | 
       BD_VALUE_TYPE_FLOAT;
-    attrPtr->value = csLittleEndian::UInt32 (csFloatToLong (f));
+    attrPtr->value = csLittleEndian::UInt32 (csIEEEfloat::FromNative (f));
   }
 }
 
 void csBinaryDocAttribute::Store (csMemFile* nodesFile)
 {
-  bdNodeAttribute diskAttr;
+  bdNodeAttributePOD diskAttr;
   size_t attrSize = sizeof (diskAttr);
 
   diskAttr.flags = attrPtr->flags & BD_VALUE_TYPE_MASK;
@@ -413,13 +413,18 @@ void csBinaryDocAttribute::Store (csMemFile* nodesFile)
     diskAttr.value = attrPtr->value;
 
   bool putFlagsInName = false;
-  const size_t nameLen = strlen (attrPtr->GetNameStr (node->doc));
-  if (nameLen < MAX_IMM_ATTR_NAME_STR)
+  const size_t nameSize = strlen (attrPtr->GetNameStr (node->doc))+1;
+  if (nameSize <= MAX_IMM_ATTR_NAME_STR)
   {
     diskAttr.flags |= BD_ATTR_NAME_IMMEDIATE;
     diskAttr.nameID = 0;
-    strcpy ((char*)&diskAttr.nameID, attrPtr->GetNameStr (node->doc));
-    putFlagsInName = (nameLen < MAX_IMM_ATTR_NAME_STR_W_FLAGS);
+    /* We really want to copy into nameID.
+     * Yes, it may spill over, that's by design.
+     * But since fortify will complain when that happens, directly
+     * copy into an offset of the diskAttr struct. */
+    memcpy ((char*)&diskAttr + offsetof (bdNodeAttributePOD, nameID),
+      attrPtr->GetNameStr (node->doc), nameSize);
+    putFlagsInName = (nameSize <= MAX_IMM_ATTR_NAME_STR_W_FLAGS);
   }
   else
   {
@@ -755,9 +760,8 @@ void csBinaryDocNode::DecRef ()
 {
   /* In case we're freed the doc's node pool will be accessed; to make sure
      it's valid keep a ref to the doc while we're (potentially) destructed */
-  doc->csBinaryDocument::IncRef();
+  csRef<csBinaryDocument> tmp(doc);
   scfPooledImplementationType::DecRef();
-  doc->csBinaryDocument::DecRef();
 }
 
 csBinaryDocNode::csBinaryDocNode (csBdNode* ptr,
@@ -829,7 +833,7 @@ const char* csBinaryDocNode::nodeValueStr (csBdNode* nodeData)
 	{
   	  char buf[50];
 	  cs_snprintf (buf, sizeof (buf) - 1, "%g", 
-	    csLongToFloat (csLittleEndian::UInt32 (nodeData->value)));
+	    doc->ConvertToFloat (csLittleEndian::UInt32 (nodeData->value)));
 	  cs_free (vstr);
 	  vstr = CS::StrDup (buf);
 	  vsptr = nodeData;
@@ -859,7 +863,7 @@ int csBinaryDocNode::nodeValueInt (csBdNode* nodeData)
       }
     case BD_VALUE_TYPE_FLOAT:
       {
-	return (int)csLongToFloat (csLittleEndian::UInt32 (nodeData->value));
+	return (int)doc->ConvertToFloat (csLittleEndian::UInt32 (nodeData->value));
       }
     default:
       return 0;
@@ -884,7 +888,7 @@ float csBinaryDocNode::nodeValueFloat (csBdNode* nodeData)
       }
     case BD_VALUE_TYPE_FLOAT:
       {
-	return csLongToFloat (csLittleEndian::UInt32 (nodeData->value));
+	return doc->ConvertToFloat (csLittleEndian::UInt32 (nodeData->value));
       }
     default:
       return 0.0f;
@@ -1012,7 +1016,7 @@ void csBinaryDocNode::SetValue (const char* value)
     {
       nodeData->flags = (nodeData->flags & ~BD_VALUE_TYPE_MASK) | 
 	BD_VALUE_TYPE_FLOAT;
-      nodeData->value = csLittleEndian::UInt32 (csFloatToLong (f));
+      nodeData->value = csLittleEndian::UInt32 (csIEEEfloat::FromNative (f));
     }
     else 
     {
@@ -1041,7 +1045,7 @@ void csBinaryDocNode::SetValueAsFloat (float value)
     cs_free (vstr); vstr = 0;
     nodeData->flags = (nodeData->flags & ~BD_VALUE_TYPE_MASK) | 
       BD_VALUE_TYPE_FLOAT;
-    nodeData->value = csLittleEndian::UInt32 (csFloatToLong (value));
+    nodeData->value = csLittleEndian::UInt32 (csIEEEfloat::FromNative (value));
   }
 }
 
@@ -1278,7 +1282,7 @@ csRef<iDocumentAttribute> csBinaryDocNode::GetAttribute (const char* name)
   return 0;
 }
 
-const char* csBinaryDocNode::GetAttributeValue (const char* name)
+const char* csBinaryDocNode::GetAttributeValue (const char* name, const char* defaultValue)
 {
   getAttributeValueStrKeeper = GetAttribute (name);
   if (getAttributeValueStrKeeper)
@@ -1287,11 +1291,11 @@ const char* csBinaryDocNode::GetAttributeValue (const char* name)
   }
   else
   {
-    return 0;
+    return defaultValue;
   }
 }
 
-int csBinaryDocNode::GetAttributeValueAsInt (const char* name)
+int csBinaryDocNode::GetAttributeValueAsInt (const char* name, int defaultValue)
 {
   csRef<iDocumentAttribute> attr = GetAttribute (name);
   if (attr)
@@ -1300,11 +1304,11 @@ int csBinaryDocNode::GetAttributeValueAsInt (const char* name)
   }
   else
   {
-    return 0;
+    return defaultValue;
   }
 }
 
-float csBinaryDocNode::GetAttributeValueAsFloat (const char* name)
+float csBinaryDocNode::GetAttributeValueAsFloat (const char* name, float defaultValue)
 {
   csRef<iDocumentAttribute> attr = GetAttribute (name);
   if (attr)
@@ -1313,12 +1317,12 @@ float csBinaryDocNode::GetAttributeValueAsFloat (const char* name)
   }
   else
   {
-    return 0.0f;
+    return defaultValue;
   }
 }
 
 bool csBinaryDocNode::GetAttributeValueAsBool (const char* name,
-					       bool defaultvalue)
+					       bool defaultValue)
 {
   csRef<iDocumentAttribute> attr = GetAttribute (name);
   if (attr)
@@ -1327,7 +1331,7 @@ bool csBinaryDocNode::GetAttributeValueAsBool (const char* name,
   }
   else
   {
-    return defaultvalue;
+    return defaultValue;
   }
 }
 
@@ -1396,10 +1400,16 @@ void csBinaryDocNode::Store (csMemFile* nodesFile)
 	(diskNode.flags & ~BD_VALUE_TYPE_MASK) | BD_VALUE_TYPE_STR_IMMEDIATE;
     // Hack: cram one more byte into value, if possible
     if ((newFlags & BE (0xff)) == 0) maximmvalue++;
-    if (strlen (nodeData->vstr) < maximmvalue)
+    size_t vstrsize = strlen (nodeData->vstr)+1;
+    if (vstrsize <= maximmvalue)
     {
       diskNode.value = 0;
-      strcpy ((char*)&diskNode.value, nodeData->vstr);
+      /* We really want to copy into value.
+       * Yes, it may spill over, that's by design.
+       * But since fortify will complain when that happens, copy into the
+       * diskNode struct directly, exploiting that value is the first
+       * member. */
+      memcpy (&diskNode, nodeData->vstr, vstrsize);
       diskNode.flags = (diskNode.flags & ~BD_VALUE_TYPE_MASK) | 
 	BD_VALUE_TYPE_STR_IMMEDIATE;
     }
@@ -1500,7 +1510,8 @@ void csBinaryDocNode::Store (csMemFile* nodesFile)
 // =================================================
 
 csBinaryDocument::csBinaryDocument () : scfImplementationType (this),
-  root (0), attrAlloc (0), nodeAlloc (0), outStrHash (0)
+  oldStyleFloats (false), root (0), attrAlloc (2000), nodeAlloc (2000),
+  outStrHash (0)
 {
 }
 
@@ -1508,32 +1519,26 @@ csBinaryDocument::~csBinaryDocument ()
 {
   if (root && (root->flags & BD_NODE_MODIFIED))
     delete root;
-  delete attrAlloc;
-  delete nodeAlloc;
 }
 
 csBdAttr* csBinaryDocument::AllocBdAttr ()
 {
-  if (!attrAlloc) attrAlloc = new csBlockAllocator<csBdAttr> (2000);
-  return attrAlloc->Alloc();
+  return attrAlloc.Alloc();
 }
 
 void csBinaryDocument::FreeBdAttr (csBdAttr* attr)
 {
-  CS_ASSERT(attrAlloc);
-  attrAlloc->Free (attr);
+  attrAlloc.Free (attr);
 }
 
 csBdNode* csBinaryDocument::AllocBdNode ()
 {
-  if (!nodeAlloc) nodeAlloc = new csBlockAllocator<csBdNode> (2000);
-  return nodeAlloc->Alloc();
+  return nodeAlloc.Alloc();
 }
 
 void csBinaryDocument::FreeBdNode (csBdNode* node)
 {
-  CS_ASSERT(nodeAlloc);
-  nodeAlloc->Free (node);
+  nodeAlloc.Free (node);
 }
 
 #include "csutil/custom_new_disable.h"
@@ -1570,7 +1575,7 @@ uint32 csBinaryDocument::GetOutStringID (const char* str)
   val = outStrHash->Request (str);
   if (val == csInvalidStringID)
   {
-    val = (csStringID)(outStrStorage->GetPos() - outStrTabOfs);
+    val = (CS::StringIDValue)(outStrStorage->GetPos() - outStrTabOfs);
     outStrStorage->Write (str, strlen (str) + 1);
     outStrHash->Register (str, val);
   }
@@ -1591,6 +1596,7 @@ void csBinaryDocument::Clear ()
   data = 0; 
   dataStart = 0;
   root = 0;
+  oldStyleFloats = false;
 }
 
 csRef<iDocumentNode> csBinaryDocument::CreateRoot ()
@@ -1625,7 +1631,8 @@ const char* csBinaryDocument::Parse (iDataBuffer* buf, bool /* collapse */)
     return "Not enough data";
   }
   bdHeader *head = (bdHeader*)buf->GetData();
-  if (head->magic != (uint32)BD_HEADER_MAGIC)
+  if ((head->magic != (uint32)BD_HEADER_MAGIC)
+      && (head->magic != (uint32)BD_HEADER_MAGIC_OLDFLOAT))
   {
     return "Not a binary CS document";
   }
@@ -1640,6 +1647,7 @@ const char* csBinaryDocument::Parse (iDataBuffer* buf, bool /* collapse */)
   }
   
   Clear();
+  oldStyleFloats = head->magic == (uint32)BD_HEADER_MAGIC_OLDFLOAT;
   root = 0;
   data = buf;
   dataStart = data->GetUint8();
