@@ -29,14 +29,13 @@
 #include "csutil/weakref.h"
 #include "csutil/leakguard.h"
 #include "csutil/hash.h"
+#include "csutil/threading/rwmutex.h"
 #include "iutil/selfdestruct.h"
 #include "csgfx/shadervarcontext.h"
 #include "imesh/object.h"
-#include "imesh/lighting.h"
 #include "iengine/mesh.h"
 #include "iengine/imposter.h"
 #include "iengine/viscull.h"
-#include "iengine/shadcast.h"
 #include "ivideo/graph3d.h"
 #include "ivideo/shader/shader.h"
 
@@ -46,13 +45,20 @@
 #include "scenenode.h"
 #include "light.h"
 
+struct iMeshLoaderIterator;
 struct iMeshWrapper;
 struct iMovable;
 struct iRenderView;
 struct iSharedVariable;
 class csEngine;
-class csLight;
-class csMeshWrapper;
+class csImposterManager;
+
+CS_PLUGIN_NAMESPACE_BEGIN(Engine)
+{
+  class csLight;
+  class csMeshWrapper;
+  class csMovable;
+  class csMovableSectorList;
 
 /**
  * General list of meshes.
@@ -60,8 +66,10 @@ class csMeshWrapper;
 class csMeshList : public scfImplementation1<csMeshList, iMeshList>
 {
 private:
-  csRefArrayObject<iMeshWrapper> list;
+  csRefArrayObject<iMeshWrapper, CS::Container::ArrayAllocDefault,
+    csArrayCapacityVariableGrow> list;
   csHash<iMeshWrapper*, csString> meshes_hash;
+  mutable CS::Threading::ReadWriteMutex meshLock;
 
   class NameChangeListener : public scfImplementation1<NameChangeListener,
   	iObjectNameChangeListener>
@@ -94,17 +102,15 @@ public:
   csMeshList (int cap, int thresshold);
   virtual ~csMeshList ();
 
-  /// Find a mesh in <name>:<childname>:<childname> notation.
-  iMeshWrapper *FindByNameWithChild (const char *Name) const;
-
   /// Override PrepareMesh
   virtual void PrepareMesh (iMeshWrapper*) { }
   /// Override FreeMesh
   virtual void FreeMesh (iMeshWrapper*) { }
 
-  virtual int GetCount () const { return (int)list.GetSize () ; }
-  virtual iMeshWrapper *Get (int n) const { return list.Get (n); }
+  virtual int GetCount () const;
+  virtual iMeshWrapper *Get (int n) const;
   virtual int Add (iMeshWrapper *obj);
+  void AddBatch (csRef<iMeshLoaderIterator> itr);
   virtual bool Remove (iMeshWrapper *obj);
   virtual bool Remove (int n);
   virtual void RemoveAll ();
@@ -112,18 +118,6 @@ public:
   virtual iMeshWrapper *FindByName (const char *Name) const;
 };
 
-struct LSIAndDist
-{
-  csLightSectorInfluence* lsi;
-  // An indication of how powerful this light affects the object.
-  // Higher values mean more influence.
-  float influence;
-};
-
-struct ExtraRenderMeshData
-{
-    csZBufMode      zBufMode;
-};
 
 #include "csutil/deprecated_warn_off.h"
 
@@ -136,12 +130,13 @@ class csMeshWrapper :
                                iMeshWrapper,
                                scfFakeInterface<iShaderVariableContext>,
                                iVisibilityObject,
-    		               iSceneNode,
+    		                       iSceneNode,
                                iSelfDestruct>,
   public CS::Graphics::OverlayShaderVariableContextImpl
 {
   friend class csMovable;
   friend class csMovableSectorList;
+  friend class ::csImposterManager;
 
 protected:
   /**
@@ -166,19 +161,9 @@ protected:
    */
   CS::Graphics::RenderPriority render_priority;
 
-  /// For NR: Cached value from DrawTest
-  bool draw_test;
-  /// For NR: Cached light test
-  bool in_light;
-  /// For NR: Should we draw anything in drawshadow at all?
-  bool cast_hardware_shadow;
-  /// For NR: should we draw last
-  bool draw_after_fancy_stuff;
-
   // Used to store extra rendermeshes that something might attach to this
-  // mesh (ie, for decals)
+  // mesh (ie, for decals or lines)
   csDirtyAccessArray<csRenderMesh*> extraRenderMeshes;
-  csArray<ExtraRenderMeshData> extraRenderMeshData;
 
   /**
    * This value indicates the last time that was used to do animation.
@@ -203,30 +188,12 @@ protected:
   csRef<csLODListener> var_min_render_dist_listener;
   csRef<csLODListener> var_max_render_dist_listener;
 
-  csEngine* engine;
-
 protected:
   virtual void InternalRemove() { SelfDestruct(); }
 
 private:
   /// Mesh object corresponding with this csMeshWrapper.
   csRef<iMeshObject> meshobj;
-  /// For optimization purposes we keep the iLightingInfo interface here.
-  csRef<iLightingInfo> light_info;
-  /**
-   * For optimization purposes we keep the iShadowReceiver interface here.
-   * Also for maintaining the special version of the shadow receiver that
-   * multiplexes in case of static lod.
-   */
-  csRef<iShadowReceiver> shadow_receiver;
-  bool shadow_receiver_valid;
-  /**
-   * For optimization purposes we keep the iShadowCaster interface here.
-   * Also for maintaining the special version of the shadow caster that
-   * multiplexes in case of static lod.
-   */
-  csRef<iShadowCaster> shadow_caster;
-  bool shadow_caster_valid;
 
   /**
    * For optimization purposes we keep the portal container interface here
@@ -246,29 +213,12 @@ private:
   /// Z-buf mode to use for drawing this object.
   csZBufMode zbufMode;
 
-  csImposterMesh *imposter_mesh;
+  /// Whether or not an imposter is currently being used for this mesh.
+  bool using_imposter;
 
-  /**
-   * Array of lights affecting this mesh object. This is calculated
-   * by the csLightManager class.
-   */
-  csDirtyAccessArray<iLightSectorInfluence*> relevant_lights;
-#define MAX_INF_LIGHTS 100
-  static LSIAndDist relevant_lights_cache[MAX_INF_LIGHTS];
-
-  // This is a mirror of 'relevant_lights' which we use to detect if
-  // a light has been removed or changed. It is only used in case we
-  // are not updating all the time (if CS_LIGHTINGUPDATE_ALWAYSUPDATE is
-  // not set).
-  struct LightRef
-  {
-    csWeakRef<iLightSectorInfluence> lsi;
-    uint32 light_nr;
-  };
-  csSafeCopyArray<LightRef> relevant_lights_ref;
-  bool relevant_lights_valid;
-  size_t relevant_lights_max;
-  csFlags relevant_lights_flags;
+  /// Whether we're drawing to an imposter texture.
+  /// TODO: Work out a better way to detect this.
+  csWeakRef<iBase> drawing_imposter;
 
   // In case the mesh has CS_ENTITY_NOCLIP set then this will
   // contain the value of the last frame number and camera pointer.
@@ -276,6 +226,47 @@ private:
   iCamera* last_camera;
   uint last_frame_number;
 
+  // An infinite bounding box.
+  static csBox3 infBBox;
+
+  // Data used when instancing is used on this mesh
+  struct InstancingData
+  {
+    // Shadervars for instancing.
+    csRef<csShaderVariable> fadeFactors;
+    csRef<csShaderVariable> transformVars;
+    bool instancingTransformsDirty;
+    struct InstancingBbox
+    {
+      csBox3 oldBox;
+      csBox3 newBox;
+    };
+    csArray<InstancingBbox> instancingBoxes;
+    struct RenderMeshesSet : public CS::NonCopyable
+    {
+      int n;
+      csRenderMesh** meshArray;
+      csRenderMesh* meshes;
+      
+      RenderMeshesSet ();
+      ~RenderMeshesSet ();
+      void CopyOriginalMeshes (int n, csRenderMesh** meshes);
+    };
+    csFrameDataHolder<RenderMeshesSet> instancingRMs;
+    
+    InstancingData() : instancingTransformsDirty (false) {}
+  };
+  typedef csBlockAllocator<InstancingData> InstancingAlloc;
+  CS_DECLARE_STATIC_CLASSVAR_REF(instancingAlloc, GetInstancingAlloc,
+    InstancingAlloc)
+  InstancingData* instancing;
+  InstancingData* GetInstancingData();
+  
+  bool DoInstancing() const { return instancing != 0; }
+  /* Given a box in object space return a box that contains all boxes
+     transformed by instance transforms. */
+  csBox3 AdjustBboxForInstances (const csBox3& origBox) const;
+  csRenderMesh** FixupRendermeshesForInstancing (int n, csRenderMesh** meshes);
 public:
   CS_LEAKGUARD_DECLARE (csMeshWrapper);
 
@@ -284,6 +275,8 @@ public:
   /// Culler flags.
   csFlags culler_flags;
 
+  csEngine* engine;
+
   /**
    * Clear this object from all sector portal lists.
    * If a sector is given then it will only clear for that sector.
@@ -325,11 +318,8 @@ public:
   csMeshWrapper (csEngine* engine, iMeshObject* meshobj = 0);
 
   /// Set the mesh factory.
-  virtual void SetFactory (iMeshFactoryWrapper* factory)
-  {
-    csMeshWrapper::factory = factory;
-    SetParentContext (factory ? factory->GetSVContext() : 0);
-  }
+  virtual void SetFactory (iMeshFactoryWrapper* factory);
+
   /// Get the mesh factory.
   virtual iMeshFactoryWrapper* GetFactory () const
   {
@@ -343,8 +333,6 @@ public:
 
   virtual iPortalContainer* GetPortalContainer () const
   { return portal_container; }
-  virtual iShadowReceiver* GetShadowReceiver ();
-  virtual iShadowCaster* GetShadowCaster ();
 
   /// For iVisibilityObject: Get the object model.
   virtual iObjectModel* GetObjectModel ()
@@ -361,6 +349,16 @@ public:
   // For iVisibilityObject:
   virtual csFlags& GetCullerFlags () { return culler_flags; }
 
+  // For iVisibilityObject:
+  virtual const csBox3& GetBBox () const
+  {
+    // 'Always visible' mesh objects have an infinite bounding box.
+    if (flags.Check (CS_ENTITY_ALWAYSVISIBLE))
+      return infBBox;
+
+    return wor_bbox;
+  }
+
   /**
    * Get the movable instance for this object.
    * It is very important to call GetMovable().UpdateMove()
@@ -423,18 +421,6 @@ public:
     return draw_cb_vector.Get (idx);
   }
 
-  virtual void SetLightingUpdate (int flags, int num_lights);
-
-  /**
-   * Get the array of relevant lights for this object.
-   */
-  const csArray<iLightSectorInfluence*>& GetRelevantLights (
-  	int maxLights, bool desireSorting);
-  /**
-   * Forcibly invalidate relevant lights.
-   */
-  void InvalidateRelevantLights () { relevant_lights_valid = false; }
-
   /**
    * Draw this mesh object given a camera transformation.
    * If needed the skeleton state will first be updated.
@@ -458,14 +444,9 @@ public:
    * This list is used for special cases (like decals) where additional
    * things need to be renderered for the mesh in an abstract way.
    */
+  virtual size_t AddExtraRenderMesh(CS::Graphics::RenderMesh* renderMesh);
   virtual size_t AddExtraRenderMesh(CS::Graphics::RenderMesh* renderMesh, 
           csZBufMode zBufMode);
-  virtual void AddExtraRenderMesh(CS::Graphics::RenderMesh* renderMesh, 
-    CS::Graphics::RenderPriority priority, csZBufMode zBufMode)
-  {
-    renderMesh->renderPrio = priority;
-    AddExtraRenderMesh (renderMesh, zBufMode);
-  }
 
   /**
    * Grabs any additional render meshes this mesh might have on top
@@ -481,11 +462,6 @@ public:
   virtual size_t GetExtraRenderMeshCount () const
   { return extraRenderMeshes.GetSize(); }
 
-  /** 
-   * Gets the priority of a specific extra rendermesh.
-   */
-  virtual CS::Graphics::RenderPriority GetExtraRenderMeshPriority(size_t idx) const;
-
   /**
    * Gets the z-buffer mode of a specific extra rendermesh
    */
@@ -510,17 +486,17 @@ public:
    */
   virtual void HardTransform (const csReversibleTransform& t);
 
-  /// This is the function to check distances.  Fn above may not be needed.
-  bool CheckImposterRelevant (iRenderView *rview);
-
-  /**
-   * Gets the imposters rendermesh
-   */
-  csRenderMesh** GetImposter (iRenderView *rview);
+  /// Check if we should use an imposter (distance based).
+  bool UseImposter (iRenderView *rview);
 
   void SetLODFade (float fade);
   void UnsetLODFade ();
 
+  void SetDefaultEnvironmentTexture ();
+
+  virtual csShaderVariable* AddInstance(csVector3& position, csMatrix3& rotation);
+  virtual void RemoveInstance(csShaderVariable* instance);
+
   //---------- Bounding volume and beam functions -----------------//
 
   virtual csSphere GetRadius () const;
@@ -602,10 +578,6 @@ public:
     return this;
   }
   virtual iMeshWrapper* FindChildByName (const char* name);
-  virtual iLightingInfo* GetLightingInfo () const
-  {
-    return light_info;
-  }
   virtual csFlags& GetFlags ()
   {
     return flags;
@@ -637,4 +609,7 @@ public:
 
 #include "csutil/deprecated_warn_on.h"
 
+}
+CS_PLUGIN_NAMESPACE_END(Engine)
+
 #endif // __CS_MESHOBJ_H__
