@@ -55,7 +55,7 @@
 
 #include "stencil.h"
 
-CS_IMPLEMENT_PLUGIN
+
 
 csStencilShadowCacheEntry::csStencilShadowCacheEntry (
   csStencilShadowStep* parent, iMeshWrapper* mesh) :
@@ -125,12 +125,12 @@ void csStencilShadowCacheEntry::SetActiveLight (iLight *light,
 
     unsigned int *buf = (unsigned int *)entry->shadow_index_buffer->Lock (
     	CS_BUF_LOCK_NORMAL);
-    entry->edge_start = triangle_count*3;
+    entry->edge_start = (int)triangle_count*3;
     int indexRange = entry->index_range = entry->edge_start;
 
     /* setup shadow caps */
-    size_t i;
-    for (i = 0; i < (size_t)entry->edge_start; i ++) buf[i] = i;
+    uint i;
+    for (i = 0; i < (uint)entry->edge_start; i ++) buf[i] = i;
 
     csVector4 lightPos4 = entry->meshLightPos;
     lightPos4.w = 0;
@@ -376,7 +376,7 @@ void csStencilShadowCacheEntry::UpdateBuffers ()
 //---------------------------------------------------------------------------
 
 csStencilShadowStep::csStencilShadowStep (csStencilShadowType* type) :  
-  scfImplementationType (this), shadowMeshes (0, 128)
+  scfImplementationType (this), shadowMeshes (0)
 {
   csStencilShadowStep::type = type;
   enableShadows = false;
@@ -414,6 +414,9 @@ bool csStencilShadowStep::Initialize (iObjectRegistry* objreg)
     (object_reg, "crystalspace.shared.stringset");
   base_id = strings->Request ("base");
   shadows_id = strings->Request ("shadows");
+
+  svNameStringset = csQueryRegistryTagInterface<iShaderVarStringSet>
+    (object_reg, "crystalspace.shader.variablenameset");
   return true;
 }
 
@@ -461,13 +464,13 @@ void csStencilShadowStep::DrawShadow (iRenderView* rview, iLight* light,
   // but just in case, no need to draw if no edges are drawn
   if (edge_start < index_range) 
   {
-    csRef<iShaderVarStack> stacks;
-    stacks.AttachNew (new scfArray<iShaderVarStack>);
+    csShaderVariableStack stack;
+    stack.Setup (svNameStringset->GetSize ());
 
     shadowCacheEntry->UpdateBuffers ();
-    shmgr->PushVariables (stacks);
-    shader->SetupPass (shaderTicket, &rmesh, modes, stacks);
-    rmesh.variablecontext->PushVariables (stacks);
+    shmgr->PushVariables (stack);
+    shader->SetupPass (shaderTicket, &rmesh, modes, stack);
+    rmesh.variablecontext->PushVariables (stack);
     if (shadowCacheEntry->ShadowCaps())
     {
       rmesh.indexstart = 0;
@@ -476,39 +479,39 @@ void csStencilShadowStep::DrawShadow (iRenderView* rview, iLight* light,
         @@@ Try to get rid of drawing the mesh twice
        */
       g3d->SetShadowState (CS_SHADOW_VOLUME_FAIL1);
-      g3d->DrawMesh (&rmesh, modes, stacks);
+      g3d->DrawMesh (&rmesh, modes, stack);
       g3d->SetShadowState (CS_SHADOW_VOLUME_FAIL2);
-      g3d->DrawMesh (&rmesh, modes, stacks);
+      g3d->DrawMesh (&rmesh, modes, stack);
     }
     else 
     {
       rmesh.indexstart = edge_start;
       rmesh.indexend = index_range;
       g3d->SetShadowState (CS_SHADOW_VOLUME_PASS1);
-      g3d->DrawMesh (&rmesh, rmesh, stacks);
+      g3d->DrawMesh (&rmesh, rmesh, stack);
       g3d->SetShadowState (CS_SHADOW_VOLUME_PASS2);
-      g3d->DrawMesh (&rmesh, rmesh, stacks);
+      g3d->DrawMesh (&rmesh, rmesh, stack);
     }
     shader->TeardownPass (shaderTicket);
   }
 }
 
 void csStencilShadowStep::Perform (iRenderView* /*rview*/, iSector* /*sector*/,
-  iShaderVarStack* /*stacks*/)
+  csShaderVariableStack& /*stacks*/)
 {
   /// TODO: Report error (no light)
   return;
 }
 
 void csStencilShadowStep::Perform (iRenderView* rview, iSector* sector,
-	iLight* light, iShaderVarStack* stacks)
+	iLight* light, csShaderVariableStack& stack)
 {
   iShader* shadow;
   if (!enableShadows || ((shadow = type->GetShadow ()) == 0))
   {
     for (size_t i = 0; i < steps.GetSize (); i++)
     {
-      steps[i]->Perform (rview, sector, light, stacks);
+      steps[i]->Perform (rview, sector, light, stack);
     }
     return;
   }
@@ -596,7 +599,7 @@ void csStencilShadowStep::Perform (iRenderView* rview, iSector* sector,
     float maxRadius;
     csRenderMeshModes modes;
     modes.z_buf_mode = CS_ZBUF_TEST;
-    size_t shaderTicket = shadow->GetTicket (modes, stacks);
+    size_t shaderTicket = shadow->GetTicket (modes, stack);
     for (size_t p = 0; p < shadow->GetNumberOfPasses (shaderTicket); p ++) 
     {
       shadow->ActivatePass (shaderTicket, p);
@@ -646,7 +649,7 @@ void csStencilShadowStep::Perform (iRenderView* rview, iSector* sector,
 
   for (size_t i = 0; i < steps.GetSize (); i++)
   {
-    steps[i]->Perform (rview, sector, light, stacks);
+    steps[i]->Perform (rview, sector, light, stack);
   }
 
   g3d->SetShadowState (CS_SHADOW_VOLUME_FINISH);
