@@ -13,7 +13,7 @@
 
   You should have received a copy of the GNU Library General Public
   License along with this library; if not, write to the Free
-  Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+  Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.3
 */
 
 #ifndef __CS_ANIMESH_H__
@@ -33,18 +33,27 @@
 #include "imesh/animesh.h"
 #include "imesh/object.h"
 #include "iutil/comp.h"
+#include "ivaria/decal.h"
 
 #include "morphtarget.h"
 
-struct iEngine;
-struct iLightManager;
-
 CS_PLUGIN_NAMESPACE_BEGIN(Animesh)
 {
 
   class FactorySubmesh;
   class FactorySocket;
 
+  struct Subset
+  {
+    csArray<size_t> vertices;      // the vertices belonging to this subset
+    size_t vertexCount;            // the number of vertices belonging to this subset
+      
+    Subset ()
+    : vertexCount (0)
+    {}
+  };
+
+
   class AnimeshObjectType : 
     public scfImplementation2<AnimeshObjectType, 
                               iMeshObjectType, 
@@ -58,32 +67,33 @@ CS_PLUGIN_NAMESPACE_BEGIN(Animesh)
 
     //-- iComponent
     virtual bool Initialize (iObjectRegistry*);
-  
-    csRef<iEngine> engine;
-    csRef<iLightManager> lightmgr;
+
+  private:
+    iObjectRegistry* object_reg;
   };
 
 
   class AnimeshObjectFactory :
-    public scfImplementation2<AnimeshObjectFactory,
-                              iAnimatedMeshFactory,
-                              iMeshObjectFactory>
+    public scfImplementationExt2<AnimeshObjectFactory,
+                                 csObjectModel,
+                                 CS::Mesh::iAnimatedMeshFactory,
+                                 iMeshObjectFactory>
   {
   public:
     AnimeshObjectFactory (AnimeshObjectType* objType);
 
-    //-- iAnimatedMeshFactory
-    virtual iAnimatedMeshFactorySubMesh* CreateSubMesh (iRenderBuffer* indices,
+    //-- CS::Mesh::iAnimatedMeshFactory
+    virtual CS::Mesh::iAnimatedMeshSubMeshFactory* CreateSubMesh (iRenderBuffer* indices,
       const char* name, bool visible);
-    virtual iAnimatedMeshFactorySubMesh* CreateSubMesh (
+    virtual CS::Mesh::iAnimatedMeshSubMeshFactory* CreateSubMesh (
       const csArray<iRenderBuffer*>& indices, 
       const csArray<csArray<unsigned int> >& boneIndices,
       const char* name,
       bool visible);
-    virtual iAnimatedMeshFactorySubMesh* GetSubMesh (size_t index) const;
+    virtual CS::Mesh::iAnimatedMeshSubMeshFactory* GetSubMesh (size_t index) const;
     virtual size_t FindSubMesh (const char* name) const;
     virtual size_t GetSubMeshCount () const;
-    virtual void DeleteSubMesh (iAnimatedMeshFactorySubMesh* mesh);
+    virtual void DeleteSubMesh (CS::Mesh::iAnimatedMeshSubMeshFactory* mesh);
 
     virtual uint GetVertexCount () const;
 
@@ -102,24 +112,34 @@ CS_PLUGIN_NAMESPACE_BEGIN(Animesh)
 
     virtual void Invalidate ();
 
-    virtual void SetSkeletonFactory (iSkeletonFactory2* skeletonFactory);
-    virtual iSkeletonFactory2* GetSkeletonFactory () const;
+    virtual void SetSkeletonFactory (CS::Animation::iSkeletonFactory* skeletonFactory);
+    virtual CS::Animation::iSkeletonFactory* GetSkeletonFactory () const;
     virtual void SetBoneInfluencesPerVertex (uint num);
     virtual uint GetBoneInfluencesPerVertex () const;
-    virtual csAnimatedMeshBoneInfluence* GetBoneInfluences ();
+    virtual CS::Mesh::AnimatedMeshBoneInfluence* GetBoneInfluences ();
 
-    virtual iAnimatedMeshMorphTarget* CreateMorphTarget (const char* name);
-    virtual iAnimatedMeshMorphTarget* GetMorphTarget (uint target);
+    virtual CS::Mesh::iAnimatedMeshMorphTarget* CreateMorphTarget (const char* name);
+    virtual CS::Mesh::iAnimatedMeshMorphTarget* GetMorphTarget (uint target);
     virtual uint GetMorphTargetCount () const;
     virtual void ClearMorphTargets ();
     virtual uint FindMorphTarget (const char* name) const;
 
-    virtual void CreateSocket (BoneID bone, 
+    virtual void CreateSocket (CS::Animation::BoneID bone, 
       const csReversibleTransform& transform, const char* name);
     virtual size_t GetSocketCount () const;
-    virtual iAnimatedMeshSocketFactory* GetSocket (size_t index) const;
+    virtual CS::Mesh::iAnimatedMeshSocketFactory* GetSocket (size_t index) const;
     virtual uint FindSocket (const char* name) const;
 
+    virtual void SetBoneBoundingBox (CS::Animation::BoneID bone, const csBox3& box); 
+    virtual const csBox3& GetBoneBoundingBox (CS::Animation::BoneID bone) const; 
+
+    virtual size_t AddSubset ();
+    virtual void AddSubsetVertex (const size_t subset, const size_t vertexIndex);
+    virtual size_t GetSubsetVertex (const size_t subset, const size_t vertexIndex) const;
+    virtual size_t GetSubsetVertexCount (const size_t subset) const;
+    virtual size_t GetSubsetCount () const;
+    virtual void ClearSubsets ();
+
     //-- iMeshObjectFactory
     virtual csFlags& GetFlags ();
 
@@ -143,21 +163,31 @@ CS_PLUGIN_NAMESPACE_BEGIN(Animesh)
     virtual void SetMixMode (uint mode);
     virtual uint GetMixMode () const;
 
+    //-- iObjectModel
+    virtual const csBox3& GetObjectBoundingBox ();
+    virtual void SetObjectBoundingBox (const csBox3& bbox);
+    virtual void GetRadius (float& radius, csVector3& center);
+
     //-- Private
     inline uint GetVertexCountP () const
     {
       return vertexCount;
     }
-  
-  private: 
 
-    // required but stupid stuff..
+    void ComputeTangents ();
+
+  private: 
+    void UpdateBoundingBoxes ();
+
+    void UpdateSubsets ();
+    void RebuildMorphTargets ();
+
+    // Main mesh factory properties
     AnimeshObjectType* objectType;
     iMeshFactoryWrapper* logParent;
     csRef<iMaterialWrapper> material;
     csFlags factoryFlags;
     uint mixMode;
-    csBox3 factoryBB;
 
     // Main data storage...
     uint vertexCount;
@@ -167,31 +197,55 @@ CS_PLUGIN_NAMESPACE_BEGIN(Animesh)
     csRef<iRenderBuffer> tangentBuffer;
     csRef<iRenderBuffer> binormalBuffer;
     csRef<iRenderBuffer> colorBuffer;
-    csDirtyAccessArray<csAnimatedMeshBoneInfluence> boneInfluences;
+    csDirtyAccessArray<CS::Mesh::AnimatedMeshBoneInfluence> boneInfluences;
     csRef<iRenderBuffer> masterBWBuffer;
     csRef<iRenderBuffer> boneWeightAndIndexBuffer[2];
 
-    csRef<iSkeletonFactory2> skeletonFactory;
+    csRef<CS::Animation::iSkeletonFactory> skeletonFactory;
 
     // Submeshes
     csRefArray<FactorySubmesh> submeshes;
 
-    csRefArray<MorphTarget> morphTargets;
+    csRefArray<MorphTarget> morphTargets;         // Morph targets without optimization
+    csRefArray<MorphTarget> subsetMorphTargets;   // Optimized morph targets (e.i. without null offsets)
     csHash<uint, csString> morphTargetNames;
 
     // Sockets
     csRefArray<FactorySocket> sockets;
 
+    // Whether or not the user has manually set any of the bone bounding boxes
+    bool userBoneBBoxes;
+
+    // Global bounding box of the whole mesh, and list of the bounding boxes
+    // linked to each bones
+    csBox3 globalBBox;
+    csHash<csBox3, CS::Animation::BoneID> boneBBoxes;
+
+    // Subsets
+    csArray<Subset> subsets; 
+    bool userSubsets;
+
+    struct SubsetTargets
+    {
+      csArray<uint> morphTargets;   // the indices of the morph targets influencing this subset
+      uint morphTargetCount;        // the number of morph targets influencing this subset
+
+      SubsetTargets ()
+      : morphTargetCount (0)
+      {}
+    };
+
     friend class AnimeshObject;
   };
 
   class FactorySubmesh : 
     public scfImplementation1<FactorySubmesh, 
-                              iAnimatedMeshFactorySubMesh>
+                              CS::Mesh::iAnimatedMeshSubMeshFactory>
   {
   public:
     FactorySubmesh (const char* name)
-      : scfImplementationType (this), material(0), name(name)
+      : scfImplementationType (this), material(0), name(name), visible (true),
+	zbufMode (CS_ZBUF_USE)
     {}
 
     virtual iRenderBuffer* GetIndices (size_t set)
@@ -244,29 +298,44 @@ CS_PLUGIN_NAMESPACE_BEGIN(Animesh)
     /// Get the submesh name.
     virtual const char* GetName () const { return name.GetData(); }
 
-    /// Whether we're visible by default.
+    virtual void SetRendering (bool doRender)
+      { visible = doRender; }
+    virtual bool IsRendering () const
+      { return visible; }
+    virtual void SetRenderPriority (CS::Graphics::RenderPriority rp)
+      { renderPriority = rp; }
+    virtual CS::Graphics::RenderPriority GetRenderPriority () const
+      { return renderPriority; }
+    virtual void SetZBufMode (csZBufMode mode)
+      { zbufMode = mode; }
+    virtual csZBufMode GetZBufMode () const
+      { return zbufMode; }
+
     bool visible;
+    CS::Graphics::RenderPriority renderPriority;
+    csZBufMode zbufMode;
   };
 
 
   class FactorySocket :
     public scfImplementation1<FactorySocket,
-                              iAnimatedMeshSocketFactory>
+                              CS::Mesh::iAnimatedMeshSocketFactory>
   {
   public:
-    FactorySocket (AnimeshObjectFactory* factory, BoneID bone, const char* name,
+    FactorySocket (AnimeshObjectFactory* factory, CS::Animation::BoneID bone, const char* name,
                    csReversibleTransform transform);
 
-    //-- iAnimatedMeshSocketFactory
+    //-- CS::Mesh::iAnimatedMeshSocketFactory
     virtual const char* GetName () const;
+    virtual void SetName (const char*);
     virtual const csReversibleTransform& GetTransform () const;
     virtual void SetTransform (csReversibleTransform& tf);
-    virtual BoneID GetBone () const;
-    virtual void SetBone (BoneID bone);
-    virtual iAnimatedMeshFactory* GetFactory ();
+    virtual CS::Animation::BoneID GetBone () const;
+    virtual void SetBone (CS::Animation::BoneID bone);
+    virtual CS::Mesh::iAnimatedMeshFactory* GetFactory ();
 
     AnimeshObjectFactory* factory;
-    BoneID bone;
+    CS::Animation::BoneID bone;
     csString name;
     csReversibleTransform transform;        
   };
@@ -275,24 +344,31 @@ CS_PLUGIN_NAMESPACE_BEGIN(Animesh)
   class AnimeshObject :
     public scfImplementationExt2<AnimeshObject,
                                  csObjectModel,
-                                 iAnimatedMesh,
-                                 iMeshObject>
+                                 CS::Mesh::iAnimatedMesh,
+                                 iMeshObject>, public iDecalAnimationControl
   {
   public:
     AnimeshObject (AnimeshObjectFactory* factory);
 
-    //-- iAnimatedMesh
-    virtual void SetSkeleton (iSkeleton2* skeleton);
-    virtual iSkeleton2* GetSkeleton () const;
+    //-- CS::Mesh::iAnimatedMesh
+    virtual void SetSkeleton (CS::Animation::iSkeleton* skeleton);
+    virtual CS::Animation::iSkeleton* GetSkeleton () const;
 
-    virtual iAnimatedMeshSubMesh* GetSubMesh (size_t index) const;
+    virtual CS::Mesh::iAnimatedMeshSubMesh* GetSubMesh (size_t index) const;
     virtual size_t GetSubMeshCount () const;
 
     virtual void SetMorphTargetWeight (uint target, float weight);
     virtual float GetMorphTargetWeight (uint target) const;
+    virtual void ClearMorphTargetWeights ();
 
     virtual size_t GetSocketCount () const;
-    virtual iAnimatedMeshSocket* GetSocket (size_t index) const;
+    virtual CS::Mesh::iAnimatedMeshSocket* GetSocket (size_t index) const;
+
+    virtual CS::Mesh::iAnimatedMeshFactory* GetAnimatedMeshFactory () const;
+    virtual iRenderBufferAccessor* GetRenderBufferAccessor () const;
+
+    virtual void SetBoneBoundingBox (CS::Animation::BoneID bone, const csBox3& box); 
+    virtual const csBox3& GetBoneBoundingBox (CS::Animation::BoneID bone) const; 
 
     //-- iMeshObject
     virtual iMeshObjectFactory* GetFactory () const;
@@ -314,6 +390,7 @@ CS_PLUGIN_NAMESPACE_BEGIN(Animesh)
     virtual void HardTransform (const csReversibleTransform& t);
     virtual bool SupportsHardTransform () const;
 
+    virtual bool HitBeamBBoxes (const csVector3& start, const csVector3& end);
     virtual bool HitBeamOutline (const csVector3& start,
       const csVector3& end, csVector3& isect, float* pr);
     virtual bool HitBeamObject (const csVector3& start, const csVector3& end,
@@ -334,13 +411,13 @@ CS_PLUGIN_NAMESPACE_BEGIN(Animesh)
     virtual void SetMixMode (uint mode);
     virtual uint GetMixMode () const;
 
-    virtual void InvalidateMaterialHandles ();
-
     virtual void PositionChild (iMeshObject* child,csTicks current_time);
 
     virtual void BuildDecal(const csVector3* pos, float decalRadius,
       iDecalBuilder* decalBuilder);
 
+    virtual void UnsetObjectBoundingBox ();
+
     //-- iObjectModel
     virtual const csBox3& GetObjectBoundingBox ();
     virtual void SetObjectBoundingBox (const csBox3& bbox);
@@ -350,8 +427,14 @@ CS_PLUGIN_NAMESPACE_BEGIN(Animesh)
     void PreGetBuffer (csRenderBufferHolder* holder, 
       csRenderBufferName buffer);    
 
+    //-- iDecalAnimationControl
+    virtual void UpdateDecal (iDecalTemplate* decalTemplate,
+			      size_t baseIndex,
+			      csArray<size_t>& indices,
+			      csRenderBuffer& vertices,
+			      csRenderBuffer& normals);
+
   private:
-    //
     void SetupSubmeshes ();
     void SetupSockets ();
     void UpdateLocalBoneTransforms ();
@@ -369,9 +452,8 @@ CS_PLUGIN_NAMESPACE_BEGIN(Animesh)
     void MorphVertices ();
 
     void PreskinLF ();
-    void UpdateLighting ();
-    void UpdateLighting (iLight* light);
-    template<typename Attenuation> void UpdateLighting (iLight* light);
+
+    void UpdateBoundingBoxes ();
 
     class RenderBufferAccessor :
       public scfImplementation1<RenderBufferAccessor, 
@@ -383,15 +465,15 @@ CS_PLUGIN_NAMESPACE_BEGIN(Animesh)
       {}
 
       void PreGetBuffer (csRenderBufferHolder* holder, 
-	csRenderBufferName buffer)
+        csRenderBufferName buffer)
       { meshObject->PreGetBuffer (holder, buffer); }
       
       AnimeshObject* meshObject;
     };
-
+ 
     class Submesh : 
       public scfImplementation1<Submesh, 
-                                iAnimatedMeshSubMesh>
+                                CS::Mesh::iAnimatedMeshSubMesh>
     {
     public:
       Submesh (AnimeshObject* meshObject, FactorySubmesh* factorySubmesh)
@@ -399,7 +481,7 @@ CS_PLUGIN_NAMESPACE_BEGIN(Animesh)
         factorySubmesh (factorySubmesh), material(0), isRendering (factorySubmesh->visible)
       {}
 
-      virtual iAnimatedMeshFactorySubMesh* GetFactorySubMesh ()
+      virtual CS::Mesh::iAnimatedMeshSubMeshFactory* GetFactorySubMesh ()
       {
         return factorySubmesh;
       }
@@ -440,19 +522,19 @@ CS_PLUGIN_NAMESPACE_BEGIN(Animesh)
     };    
 
     class Socket : public scfImplementation1<Socket, 
-                                             iAnimatedMeshSocket>
+                                             CS::Mesh::iAnimatedMeshSocket>
     {
     public:
       Socket (AnimeshObject* object, FactorySocket* factorySocket);
 
-      //-- iAnimatedMeshSocket
+      //-- CS::Mesh::iAnimatedMeshSocket
       virtual const char* GetName () const;
-      virtual iAnimatedMeshSocketFactory* GetFactory ();
+      virtual CS::Mesh::iAnimatedMeshSocketFactory* GetFactory ();
       virtual const csReversibleTransform& GetTransform () const;
       virtual void SetTransform (csReversibleTransform& tf);
       virtual const csReversibleTransform GetFullTransform () const;     
-      virtual BoneID GetBone () const;
-      virtual iAnimatedMesh* GetMesh () const;
+      virtual CS::Animation::BoneID GetBone () const;
+      virtual CS::Mesh::iAnimatedMesh* GetMesh () const;
       virtual iSceneNode* GetSceneNode () const;
       virtual void SetSceneNode (iSceneNode* sn);
 
@@ -460,8 +542,8 @@ CS_PLUGIN_NAMESPACE_BEGIN(Animesh)
 
       AnimeshObject* object;
       FactorySocket* factorySocket;
-      BoneID bone;      
-      csReversibleTransform transform;      
+      CS::Animation::BoneID bone;
+      csReversibleTransform transform;
       csReversibleTransform socketBoneTransform;
       iSceneNode* sceneNode;
     };
@@ -472,13 +554,13 @@ CS_PLUGIN_NAMESPACE_BEGIN(Animesh)
     uint mixMode;
     csFlags meshObjectFlags;
 
-    csRef<iSkeleton2> skeleton;
+    csRef<CS::Animation::iSkeleton> skeleton;
     unsigned int skeletonVersion;
-    csTicks lastTick;
+    bool animationInitialized;
 
     // Hold the bone transforms
     csRef<csShaderVariable> boneTransformArray;
-    csRef<csSkeletalState2> lastSkeletonState;
+    csRef<CS::Animation::AnimatedMeshState> lastSkeletonState;
 
     csRenderMeshHolder rmHolder;
     csDirtyAccessArray<CS::Graphics::RenderMesh*> renderMeshList;
@@ -486,7 +568,16 @@ CS_PLUGIN_NAMESPACE_BEGIN(Animesh)
     csRefArray<Submesh> submeshes;
     csRefArray<Socket> sockets;
 
+    // Whether or not the user has manually set the global bounding box of the mesh
+    bool userGlobalBBox;
+
+    // Global bounding box of the whole mesh, and list of the bounding boxes
+    // linked to each bones
+    csBox3 globalBBox;
+    csHash<csBox3, CS::Animation::BoneID> boneBBoxes;
+
     // Holder for skinned vertex buffers
+    csRef<RenderBufferAccessor> bufferAccessor;
     csRef<iRenderBuffer> skinnedVertices;
     csRef<iRenderBuffer> skinnedNormals;
     csRef<iRenderBuffer> skinnedTangents;
@@ -495,15 +586,18 @@ CS_PLUGIN_NAMESPACE_BEGIN(Animesh)
     csRef<iRenderBuffer> postMorphVertices;
 
     csArray<float> morphTargetWeights;
+    unsigned int morphVersion;
+    bool morphStateChanged;
 
-    // Version numbers for the software skinning
-    unsigned int skinVertexVersion, skinNormalVersion, skinTangentVersion, skinBinormalVersion;
-    // Things we skinned in software last frame
-    bool skinVertexLF, skinNormalLF, skinTangentLF, skinBinormalLF;
-    
-    // Used for SW lighting
-    iMovable*  lighting_movable;
-    csRef<iRenderBuffer> colorsLit;
+    // Version numbers for the skinning
+    unsigned int skinVertexVersion, skinNormalVersion, skinTangentBinormalVersion;
+    unsigned int morphVertexVersion;
+    // Things we skinned last frame
+    bool skinVertexLF, skinNormalLF, skinTangentBinormalLF;
+
+    // LOD on the animation
+    csTicks lastUpdate;
+    char accumulatedFrames;
   };
 
 }
