@@ -34,6 +34,7 @@
 #include "iengine/material.h"
 #include "iutil/vfs.h"
 #include "csutil/csstring.h"
+#include "iutil/stringarray.h"
 #include "iutil/object.h"
 #include "iutil/document.h"
 #include "iutil/objreg.h"
@@ -42,7 +43,7 @@
 #include "imap/ldrctxt.h"
 #include "ivaria/reporter.h"
 
-CS_IMPLEMENT_PLUGIN
+
 
 enum
 {
@@ -203,7 +204,8 @@ csRef<iEmitGen3D> csEmitLoader::ParseEmit (iDocumentNode* node,
 	if (weight == 0)
 	{
 	  synldr->ReportError ("crystalspace.emitloader.parse",
-		child, "'weight' cannot be given in this context!");
+		child, "%s cannot be given in this context!",
+		CS::Quote::Single ("weight"));
 	  return 0;
 	}
 	*weight = child->GetContentsValueAsFloat ();
@@ -224,7 +226,8 @@ csRef<iEmitGen3D> csEmitLoader::ParseEmit (iDocumentNode* node,
 	  if (!synldr->ParseBox (child, box))
 	  {
 	    synldr->ReportError ("crystalspace.emitloader.parse",
-		  child, "Error parsing box for 'emitbox'!");
+		  child, "Error parsing box for %s!",
+		  CS::Quote::Single ("emitbox"));
 	    return 0;
 	  }
 	  csRef<iEmitBox> ebox = fstate->CreateBox ();
@@ -274,7 +277,8 @@ csRef<iEmitGen3D> csEmitLoader::ParseEmit (iDocumentNode* node,
 	  if (!synldr->ParseBox (child, box))
 	  {
 	    synldr->ReportError ("crystalspace.emitloader.parse",
-		  child, "Error parsing box for 'emitline'!");
+		  child, "Error parsing box for %s!",
+		  CS::Quote::Single ("emitline"));
 	    return 0;
 	  }
 	  csRef<iEmitLine> eline = fstate->CreateLine ();
@@ -288,7 +292,8 @@ csRef<iEmitGen3D> csEmitLoader::ParseEmit (iDocumentNode* node,
 	  if (!synldr->ParseBox (child, box))
 	  {
 	    synldr->ReportError ("crystalspace.emitloader.parse",
-		  child, "Error parsing box for 'emitcylinder'!");
+		  child, "Error parsing box for %s!",
+		  CS::Quote::Single ("emitcylinder"));
 	    return 0;
 	  }
 	  p = child->GetAttributeValueAsFloat ("p");
@@ -304,7 +309,8 @@ csRef<iEmitGen3D> csEmitLoader::ParseEmit (iDocumentNode* node,
 	  if (!synldr->ParseBox (child, box))
 	  {
 	    synldr->ReportError ("crystalspace.emitloader.parse",
-		  child, "Error parsing box for 'emitcylindertangent'!");
+		  child, "Error parsing box for %s!",
+		  CS::Quote::Single ("emitcylindertangent"));
 	    return 0;
 	  }
 	  p = child->GetAttributeValueAsFloat ("p");
@@ -321,7 +327,8 @@ csRef<iEmitGen3D> csEmitLoader::ParseEmit (iDocumentNode* node,
 	  if (!minnode)
 	  {
 	    synldr->ReportError ("crystalspace.emitloader.parse",
-		child, "'min' is missing in 'emitspheretangent'!");
+		child, "%s is missing in %s!",
+		CS::Quote::Single ("min"), CS::Quote::Single ("emitspheretangent"));
 	    return 0;
 	  }
 	  a.x = minnode->GetAttributeValueAsFloat ("x");
@@ -365,12 +372,14 @@ csPtr<iBase> csEmitLoader::Parse (iDocumentNode* node,
 	{
 	  const char* factname = child->GetContentsValue ();
 	  iMeshFactoryWrapper* fact = ldr_context->FindMeshFactory (factname);
-	  if (!fact)
-	  {
-	    synldr->ReportError ("crystalspace.emitloader.parse",
-		child, "Cannot find factory '%s' for emit!", factname);
-	    return 0;
-	  }
+
+    if(!fact)
+    {
+      synldr->ReportError ("crystalspace.emitloader.parse",
+        child, "Cannot find factory %s for emit!", CS::Quote::Single (factname));
+        return 0;
+    }
+
 	  mesh = fact->GetMeshObjectFactory ()->NewInstance ();
           partstate = scfQueryInterface<iParticleState> (mesh);
           emitstate = scfQueryInterface<iEmitState> (mesh);
@@ -378,8 +387,8 @@ csPtr<iBase> csEmitLoader::Parse (iDocumentNode* node,
 	  {
       	    synldr->ReportError (
 		"crystalspace.emitstate.parse.badfactory",
-		child, "Factory '%s' doesn't appear to be an emit factory!",
-		factname);
+		child, "Factory %s doesn't appear to be an emit factory!",
+		CS::Quote::Single (factname));
 	    return 0;
 	  }
           emitfactorystate = scfQueryInterface<iEmitFactoryState> (
@@ -393,7 +402,7 @@ csPtr<iBase> csEmitLoader::Parse (iDocumentNode* node,
 	  if (!mat)
 	  {
 	    synldr->ReportError ("crystalspace.emitloader.parse",
-		child, "Cannot find material '%s' for emit!", matname);
+		child, "Cannot find material %s for emit!", CS::Quote::Single (matname));
             return 0;
 	  }
 	  mesh->SetMaterialWrapper (mat);
@@ -446,7 +455,8 @@ csPtr<iBase> csEmitLoader::Parse (iDocumentNode* node,
 	  if (!alphanode)
 	  {
 	    synldr->ReportError ("crystalspace.emitloader.parse",
-		child, "Missing 'alpha' in 'aging'!");
+		child, "Missing %s in %s!",
+		CS::Quote::Single ("alpha"), CS::Quote::Single ("aging"));
             return 0;
 	  }
 	  alpha = alphanode->GetContentsValueAsFloat ();
@@ -454,7 +464,8 @@ csPtr<iBase> csEmitLoader::Parse (iDocumentNode* node,
 	  if (!swirlnode)
 	  {
 	    synldr->ReportError ("crystalspace.emitloader.parse",
-		child, "Missing 'swirl' in 'aging'!");
+		child, "Missing %s in %s!",
+		CS::Quote::Single ("swirl"), CS::Quote::Single ("aging"));
             return 0;
 	  }
 	  swirl = swirlnode->GetContentsValueAsFloat ();
@@ -462,7 +473,8 @@ csPtr<iBase> csEmitLoader::Parse (iDocumentNode* node,
 	  if (!scalenode)
 	  {
 	    synldr->ReportError ("crystalspace.emitloader.parse",
-		child, "Missing 'scale' in 'aging'!");
+		child, "Missing %s in %s!",
+		CS::Quote::Single ("scale"), CS::Quote::Single ("aging"));
             return 0;
 	  }
 	  scale = scalenode->GetContentsValueAsFloat ();
@@ -470,7 +482,8 @@ csPtr<iBase> csEmitLoader::Parse (iDocumentNode* node,
 	  if (!rotspeednode)
 	  {
 	    synldr->ReportError ("crystalspace.emitloader.parse",
-		child, "Missing 'rotspeed' in 'aging'!");
+		child, "Missing %s in %s!",
+		CS::Quote::Single ("rotspeed"), CS::Quote::Single ("aging"));
             return 0;
 	  }
 	  rotspeed = rotspeednode->GetContentsValueAsFloat ();
@@ -478,7 +491,8 @@ csPtr<iBase> csEmitLoader::Parse (iDocumentNode* node,
 	  if (!timenode)
 	  {
 	    synldr->ReportError ("crystalspace.emitloader.parse",
-		child, "Missing 'time' in 'aging'!");
+		child, "Missing %s in %s!",
+		CS::Quote::Single ("time"),CS::Quote::Single ("aging"));
             return 0;
 	  }
 	  time = timenode->GetContentsValueAsInt ();
