@@ -36,12 +36,10 @@
 #include "csutil/refarr.h"
 #include "csutil/parray.h"
 #include "csutil/pooledscfclass.h"
+#include "csutil/scfarray.h"
 #include "csutil/weakref.h"
 #include "iengine/light.h"
-#include "iengine/lightmgr.h"
-#include "iengine/shadcast.h"
 #include "imesh/genmesh.h"
-#include "imesh/lighting.h"
 #include "imesh/object.h"
 #include "iutil/comp.h"
 #include "iutil/eventh.h"
@@ -61,7 +59,6 @@ struct iEngine;
 struct iMaterialWrapper;
 struct iMovable;
 struct iObjectRegistry;
-struct iShadowBlockList;
 
 CS_PLUGIN_NAMESPACE_BEGIN(Genmesh)
 {
@@ -103,7 +100,7 @@ public:
 
   void AddVariable (csShaderVariable *variable)
   { }
-  csShaderVariable* GetVariable (csStringID name) const
+  csShaderVariable* GetVariable (CS::ShaderVarStringID name) const
   { 
     csShaderVariable* sv = context1->GetVariable (name); 
     if (sv == 0)
@@ -114,10 +111,10 @@ public:
   { 
     return context1->GetShaderVariables();
   }
-  void PushVariables (iShaderVarStack* stacks) const
+  void PushVariables (csShaderVariableStack& stack) const
   { 
-    context2->PushVariables (stacks);
-    context1->PushVariables (stacks);
+    context2->PushVariables (stack);
+    context1->PushVariables (stack);
   }
 
   bool IsEmpty () const { return context1->IsEmpty() && context2->IsEmpty(); }
@@ -125,17 +122,14 @@ public:
   void ReplaceVariable (csShaderVariable *variable) { }
   void Clear () { }
   bool RemoveVariable  (csShaderVariable *) { return false; }
-  bool RemoveVariable  (csStringID) { return false; }
+  bool RemoveVariable  (CS::ShaderVarStringID) { return false; }
 };
 
 /**
  * Genmesh version of mesh object.
  */
-class csGenmeshMeshObject : public scfImplementation5<csGenmeshMeshObject, 
+class csGenmeshMeshObject : public scfImplementation2<csGenmeshMeshObject, 
 						      iMeshObject,
-						      iLightingInfo,
-						      iShadowCaster,
-						      iShadowReceiver,
 						      iGeneralMeshState>
 {
 private:
@@ -151,34 +145,27 @@ private:
 
   size_t factory_user_rb_state;
 
-  iMovable* lighting_movable;
-
   csDirtyAccessArray<csRenderMesh*> renderMeshes;
   mutable SubMeshProxiesContainer subMeshes;
   mutable uint factorySubMeshesChangeNum;
   void UpdateSubMeshProxies () const;
 
   csUserRenderBufferManager userBuffers;
-  csArray<csStringID> user_buffer_names;
+  csArray<CS::ShaderVarStringID> user_buffer_names;
 
   csGenmeshMeshObjectFactory* factory;
   iMeshWrapper* logparent;
   csRef<iMeshObjectDrawCallback> vis_cb;
-  bool do_lighting;
   bool do_manual_colors;
   csColor4 base_color;
   float current_lod;
   uint32 current_features;
   csFlags flags;
 
-  bool do_shadows;
-  bool do_shadow_rec;
-
   struct LegacyLightingData
   {
     csColor4* lit_mesh_colors;
     int num_lit_mesh_colors;	// Should be equal to factory number.
-    csColor4* static_mesh_colors;
     
     csRef<iRenderBuffer> color_buffer;
     
@@ -187,66 +174,26 @@ private:
     
     void SetColorNum (int num);
     void Free();
-    void Clear();
+    void Clear(const csColor4& base_color);
   };
   LegacyLightingData legacyLighting;
 
-  /**
-   * Global sector wide dynamic ambient version.
-   */
-  uint32 dynamic_ambient_version;
-
-  csHash<csShadowArray*, csPtrKey<iLight> > pseudoDynInfo;
-
-  // If we are using the iLightingInfo lighting system then this
-  // is an array of lights that affect us right now.
-  csSet<csPtrKey<iLight> > affecting_lights;
-  // In case we are not using the iLightingInfo system then we
-  // GetRenderMeshes() will updated the following array:
-  csArray<iLightSectorInfluence*> relevant_lights;
-
-  // If the following flag is dirty then some of the affecting lights
-  // has changed and we need to recalculate.
-  bool lighting_dirty;
-
-  // choose whether to draw shadow caps or not
-  bool shadow_caps;
-
   bool initialized;
 
   /// Current movable number.
   long cur_movablenr;
 
-  /**
-   * Clears out the pseudoDynInfo hash and frees the memory allocated by the
-   * shadow maps.
-   */
-  void ClearPseudoDynLights ();
-
   /**
    * Setup this object. This function will check if setup is needed.
    */
   void SetupObject ();
 
-  /**
-   * Make sure the 'lit_mesh_colors' array has the right size.
-   * Also clears the pseudo-dynamic light hash if the vertex count
-   * changed!
-   */
-  void CheckLitColors ();
-
-  /**
-   * Process one light and add the values to the genmesh light table.
-   * The given transform is the full movable transform.
-   */
-  void UpdateLightingOne (const csReversibleTransform& trans, iLight* light);
-
-  /**
-   * Update lighting using the iLightingInfo system.
-   */
-  void UpdateLighting (
-      const csArray<iLightSectorInfluence*>& lights, iMovable* movable);
-
+  /// Get positions buffer
+  iRenderBuffer* GetPositions ();
+  const csVector3* GetVertices ();
+  
+  int ComputeProgLODLevel (const SubMeshProxy& subMesh, const csVector3& camera_pos);
+  
 public:
   /// Constructor.
   csGenmeshMeshObject (csGenmeshMeshObjectFactory* factory);
@@ -269,33 +216,50 @@ public:
   
   /**\name iGeneralMeshState implementation
    * @{ */
-  void SetLighting (bool l) { do_lighting = l; }
-  bool IsLighting () const { return do_lighting; }
+  void SetLighting (bool l) { }
+  bool IsLighting () const { return false; }
   void SetManualColors (bool m) { do_manual_colors = m; }
   bool IsManualColors () const { return do_manual_colors; }
-  const csBox3& GetObjectBoundingBox ();
-  void SetObjectBoundingBox (const csBox3& bbox);
-  void GetRadius (float& rad, csVector3& cent);
-  void SetShadowCasting (bool m) { do_shadows = m; }
-  bool IsShadowCasting () const { return do_shadows; }
-  void SetShadowReceiving (bool m) { do_shadow_rec = m; }
-  bool IsShadowReceiving () const { return do_shadow_rec; }
+  void SetShadowCasting (bool m) { }
+  bool IsShadowCasting () const { return true; }
+  void SetShadowReceiving (bool m) { }
+  bool IsShadowReceiving () const { return false; }
   iGeneralMeshSubMesh* FindSubMesh (const char* name) const; 
   /** @} */
 
+  class csAnimatedModel : public scfImplementationExt0<csAnimatedModel, 
+                                                              csObjectModel>
+  {
+    csGenmeshMeshObject* object;
+
+  public:
+    csAnimatedModel (csGenmeshMeshObject* object);
+    virtual ~csAnimatedModel ();
+
+    /**\name iObjectModel implementation
+     * @{ */
+    const csBox3& GetObjectBoundingBox ();
+    void SetObjectBoundingBox (const csBox3& bbox);
+    void GetRadius (float& rad, csVector3& cent);
+  };
+  csRef<iObjectModel> objectModel;
+
   iVirtualClock* vc;
   csRef<iGenMeshAnimationControl> anim_ctrl;
-  csRef<iGenMeshAnimationControl1_4> anim_ctrl2;
   void SetAnimationControl (iGenMeshAnimationControl* anim_ctrl);
   iGenMeshAnimationControl* GetAnimationControl () const { return anim_ctrl; }
   const csVector3* AnimControlGetVertices ();
   const csVector2* AnimControlGetTexels ();
   const csVector3* AnimControlGetNormals ();
   const csColor4* AnimControlGetColors (csColor4* source);
+  const csBox3& AnimControlGetBbox ();
+  const csBox3* AnimControlGetBboxes ();
+  const float AnimControlGetRadius ();
   bool anim_ctrl_verts;
   bool anim_ctrl_texels;
   bool anim_ctrl_normals;
   bool anim_ctrl_colors;
+  bool anim_ctrl_bbox;
   struct AnimBuffers
   {
     csRef<iRenderBuffer> position;
@@ -323,22 +287,6 @@ public:
   iRenderBuffer* GetRenderBuffer (const char* name);
   iRenderBuffer* GetRenderBuffer (csRenderBufferName name);
 
-  /**\name Shadow and lighting system
-   * @{ */
-  char* GenerateCacheName ();
-  void InitializeDefault (bool clear);
-  bool ReadFromCache (iCacheManager* cache_mgr);
-  bool WriteToCache (iCacheManager* cache_mgr);
-  void PrepareLighting ();
-
-  void AppendShadows (iMovable* movable, iShadowBlockList* shadows,
-    	const csVector3& origin);
-  void CastShadows (iMovable* movable, iFrustumView* fview);
-  void LightChanged (iLight* light);
-  void LightDisconnect (iLight* light);
-  void DisconnectAllLights ();
-  /** @} */
-
   /**\name iMeshObject implementation
    * @{ */
   virtual iMeshObjectFactory* GetFactory () const;
@@ -374,14 +322,13 @@ public:
   virtual bool SetColor (const csColor& col)
   {
     base_color.Set (col);
-    lighting_dirty = true;
     return true;
   }
   virtual bool GetColor (csColor& col) const { col = base_color; return true; }
   virtual bool SetMaterialWrapper (iMaterialWrapper* mat);
   virtual iMaterialWrapper* GetMaterialWrapper () const
-  { return subMeshes.GetDefaultSubmesh()->SubMeshProxy::GetMaterial(); }
-  virtual void InvalidateMaterialHandles () { }
+  { return subMeshes.GetMaterialWrapper(); }
+
   /**
    * see imesh/object.h for specification. The default implementation
    * does nothing.
@@ -414,6 +361,8 @@ public:
   friend class RenderBufferAccessor;
 
   void PreGetBuffer (csRenderBufferHolder* holder, csRenderBufferName buffer);
+  
+  virtual void ForceProgLODLevel(int level);
 
   //------------------ iShaderVariableAccessor implementation ------------
   class ShaderVariableAccessor : 
@@ -453,7 +402,7 @@ public:
   bool do_fullbright;
 
   csWeakRef<iGraphics3D> g3d;
-  csRef<iStringSet> strings;
+  csRef<iShaderVarStringSet> svstrings;
 
   struct KnownBuffers
   {
@@ -468,7 +417,7 @@ public:
   KnownBuffers knownBuffers;
     
   csUserRenderBufferManager userBuffers;
-  csArray<csStringID> user_buffer_names;
+  csArray<CS::ShaderVarStringID> user_buffer_names;
    
   struct LegacyBuffers
   {
@@ -487,7 +436,7 @@ public:
   };
   LegacyBuffers legacyBuffers;
   void CreateLegacyBuffers();
-  void ClearLegacyBuffers (uint mask = CS_BUFFER_ALL_MASK);
+  void ClearLegacyBuffers (uint mask = (uint)CS_BUFFER_ALL_MASK);
   void UpdateFromLegacyBuffers();
   
   SubMeshesContainer subMeshes;
@@ -505,6 +454,9 @@ public:
   bool object_bbox_valid;
   bool initialized;
 
+  float prog_lod_min_dist;
+  float prog_lod_max_dist;
+
   // For animation control.
   csRef<iGenMeshAnimationControlFactory> anim_ctrl_fact;
   void SetAnimationControlFactory (iGenMeshAnimationControlFactory*
@@ -524,6 +476,9 @@ public:
 
   /// Update tangent and bitangent buffers
   void UpdateTangentsBitangents ();
+  
+  /// Get positions buffer
+  iRenderBuffer* GetPositions();
 public:
   CS_LEAKGUARD_DECLARE (csGenmeshMeshObjectFactory);
 
@@ -537,7 +492,6 @@ public:
   iObjectRegistry* object_reg;
   iMeshFactoryWrapper* logparent;
   csRef<csGenmeshMeshObjectType> genmesh_type;
-  csRef<iLightManager> light_mgr;
   csFlags flags;
 
   iEngine* engine;
@@ -559,11 +513,11 @@ public:
 
   bool SetMaterialWrapper (iMaterialWrapper* material)
   {
-    subMeshes.GetDefaultSubmesh()->SubMesh::SetMaterial (material);
+    subMeshes.SetMaterialWrapper (material);
     return true;
   }
   iMaterialWrapper* GetMaterialWrapper () const
-  { return subMeshes.GetDefaultSubmesh()->SubMesh::GetMaterial(); }
+  { return subMeshes.GetMaterialWrapper(); }
   void AddVertex (const csVector3& v,
       const csVector2& uv, const csVector3& normal,
       const csColor4& color);
@@ -587,6 +541,7 @@ public:
   { autonormals = false; }
   void Compress ();
   void GenerateBox (const csBox3& box);
+  void GenerateCylinder (float l, float r, uint sides);
   void GenerateCapsule (float l, float r, uint sides);
   void GenerateSphere (const csEllipsoid& ellips, int rim_vertices,
       	bool cyl_mapping = false, bool toponly = false,
@@ -596,7 +551,11 @@ public:
   bool IsBack2Front () const { return back2front; }
   void BuildBack2FrontTree ();
 
+  /* Internal methods to set render buffers.
+     They can also correctly replace an attached buffer. */
   bool InternalSetBuffer (csRenderBufferName name, iRenderBuffer* buffer);
+  bool InternalSetBuffer (CS::ShaderVarStringID bufID, iRenderBuffer* buffer,
+			  csRenderBufferName name);
   
   bool AddRenderBuffer (const char *name, iRenderBuffer* buffer);
   bool AddRenderBuffer (csRenderBufferName name, iRenderBuffer* buffer);
@@ -614,12 +573,12 @@ public:
   /**
    * Get the string ID's for the anonymous buffers
    */
-  const csArray<csStringID>& GetUserBufferNames ()
+  const csArray<CS::ShaderVarStringID>& GetUserBufferNames ()
   { return user_buffer_names; }
   const csUserRenderBufferManager& GetUserBuffers()
   { return userBuffers; }
-  iStringSet* GetStrings()
-  { return strings; }
+  iShaderVarStringSet* GetSVStrings()
+  { return svstrings; }
 
   void ClearSubMeshes ();
   void AddSubMesh (unsigned int *triangles,
@@ -714,7 +673,21 @@ public:
   {
     return autonormals;
   }
+  
+  virtual int GetNumProgLODLevels() const;
 
+  virtual void GetProgLODDistances(float& out_min, float& out_max) const
+  {
+    out_min = prog_lod_min_dist;
+    out_max = prog_lod_max_dist;
+  }
+
+  virtual void SetProgLODDistances(float min, float max)
+  {
+    prog_lod_min_dist = min;
+    prog_lod_max_dist = max;
+  }
+    
   //------------------------ iMeshObjectFactory implementation --------------
   virtual csFlags& GetFlags () { return flags; }
   virtual csPtr<iMeshObject> NewInstance ();
@@ -784,11 +757,13 @@ class csGenmeshMeshObjectType :
                             iMeshObjectType,
                             iComponent>
 {
-public:
+private:
   iObjectRegistry* object_reg;
-  bool do_verbose;
-  MergedSVContext::Pool mergedSVContextPool;
+  CS::Threading::Mutex m;
   csStringHash submeshNamePool;
+
+public:
+  MergedSVContext::Pool mergedSVContextPool;
   csStringID base_id;
 
   /// Constructor.
@@ -799,6 +774,8 @@ public:
   virtual csPtr<iMeshObjectFactory> NewFactory ();
   /// Initialize.
   bool Initialize (iObjectRegistry* object_reg);
+  /// Thread safe duplicate-free store of a name.
+  const char* StoreName (const char* name);
 };
 
 }
