@@ -30,8 +30,9 @@
 #include "iutil/document.h"
 #include "iutil/plugin.h"
 #include "iutil/object.h"
+#include "iutil/stringarray.h"
+
 
-CS_IMPLEMENT_PLUGIN
 
 template<>
 class csHashComputer<iParticleEmitter*> : public csHashComputerIntegral<iParticleEmitter*> {};
@@ -381,7 +382,7 @@ CS_PLUGIN_NAMESPACE_BEGIN(ParticlesLoader)
       case XMLTOKEN_EXTENT:
         if (!synldr->ParseVector (child, extent))
         {
-          return false;
+          return 0;
         }
         break;
       case XMLTOKEN_CONEANGLE:
@@ -765,7 +766,36 @@ CS_PLUGIN_NAMESPACE_BEGIN(ParticlesLoader)
         }
       }
 
-      
+    }
+    else if (!strcasecmp (effectorType, "light"))
+    {
+      csRef<iParticleBuiltinEffectorLight> lightEffector = 
+        factory->CreateLight ();
+      effector = lightEffector;
+
+      csRef<iDocumentNodeIterator> it = node->GetNodes ();
+      while (it->HasNext ())
+      {
+        csRef<iDocumentNode> child = it->Next ();
+
+        if (child->GetType () != CS_NODE_ELEMENT) 
+          continue;
+
+        const char* value = child->GetValue ();
+        csStringID id = xmltokens.Request (value);
+        switch(id)
+        {
+        case XMLTOKEN_CUTOFFDISTANCE:
+          {
+	    float distance = child->GetContentsValueAsFloat ();
+	    lightEffector->SetInitialCutoffDistance (distance);
+	  }
+          break;
+        default:
+          synldr->ReportBadToken (child);
+          return 0;
+        }
+      }
     }
     else
     {
@@ -830,7 +860,7 @@ CS_PLUGIN_NAMESPACE_BEGIN(ParticlesLoader)
           {
             synldr->ReportError (
               "crystalspace.genmeshfactoryloader.parse.unknownmaterial",
-              child, "Couldn't find material '%s'!", matname);
+              child, "Couldn't find material %s!", CS::Quote::Single (matname));
             return 0;
           }
           factoryObj->SetMaterialWrapper (mat);
@@ -879,13 +909,12 @@ CS_PLUGIN_NAMESPACE_BEGIN(ParticlesLoader)
       case XMLTOKEN_FACTORY:
         {
           const char* factname = child->GetContentsValue ();
-	  iMeshFactoryWrapper* fact = ldr_context->FindMeshFactory (factname);
+          iMeshFactoryWrapper* fact = ldr_context->FindMeshFactory (factname);
 
-          if (!fact)
+          if(!fact)
           {
             synldr->ReportError ("crystalspace.particleloader.parsesystem",
-              child, "Could not find factory '%s'!", factname);
-
+              child, "Could not find factory %s!", CS::Quote::Single (factname));
             return 0;
           }
 
@@ -895,8 +924,8 @@ CS_PLUGIN_NAMESPACE_BEGIN(ParticlesLoader)
           if (!particleSystem)
           {
             synldr->ReportError ("crystalspace.particleloader.parsesystem",
-              child, "Factory '%s' does not seem to be a particle system factory!", 
-              factname);
+              child, "Factory %s does not seem to be a particle system factory!", 
+              CS::Quote::Single (factname));
 
             return 0;
           }
@@ -921,7 +950,7 @@ CS_PLUGIN_NAMESPACE_BEGIN(ParticlesLoader)
           {
             synldr->ReportError (
               "crystalspace.genmeshfactoryloader.parse.unknownmaterial",
-              child, "Couldn't find material '%s'!", matname);
+              child, "Couldn't find material %s!", CS::Quote::Single (matname));
             return 0;
           }
           meshObj->SetMaterialWrapper (mat);
@@ -1534,6 +1563,12 @@ CS_PLUGIN_NAMESPACE_BEGIN(ParticlesLoader)
       sizeNode->SetValue ("particlesize");
       synldr->WriteVector (sizeNode, partFact->GetParticleSize ());
 
+      csRef<iDocumentNode> mixmodeNode = paramsNode->CreateNodeBefore (
+        CS_NODE_ELEMENT, 0);
+      mixmodeNode->SetValue ("mixmode");
+      synldr->WriteMixmode (mixmodeNode, meshFact->GetMixMode (),
+	  true);
+
       // Write emitters
       for (size_t i = 0; i < partFact->GetEmitterCount (); i++)
       {
