@@ -45,7 +45,7 @@
 
 #include "spr2d.h"
 
-CS_IMPLEMENT_PLUGIN
+
 
 CS_PLUGIN_NAMESPACE_BEGIN(Spr2D)
 {
@@ -64,9 +64,6 @@ csSprite2DMeshObject::csSprite2DMeshObject (csSprite2DMeshObjectFactory* factory
   material = factory->GetMaterialWrapper ();
   lighting = factory->HasLighting ();
   MixMode = factory->GetMixMode ();
-
-  scfVertices.AttachNew (new scfArrayWrap<iColoredVertices, 
-    csColoredVertices> (vertices));
 }
 
 csSprite2DMeshObject::~csSprite2DMeshObject ()
@@ -74,6 +71,19 @@ csSprite2DMeshObject::~csSprite2DMeshObject ()
   delete uvani;
 }
 
+iColoredVertices* csSprite2DMeshObject::GetVertices ()
+{
+  if (!scfVertices.IsValid())
+    return factory->GetVertices ();
+  return scfVertices;
+}
+csColoredVertices* csSprite2DMeshObject::GetCsVertices ()
+{
+  if (!scfVertices.IsValid())
+    return factory->GetCsVertices ();
+  return &vertices;
+}
+
 void csSprite2DMeshObject::SetupObject ()
 {
   if (!initialized)
@@ -81,16 +91,17 @@ void csSprite2DMeshObject::SetupObject ()
     initialized = true;
     float max_sq_dist = 0;
     size_t i;
-    bbox_2d.StartBoundingBox(vertices[0].pos);
-    for (i = 0 ; i < vertices.GetSize () ; i++)
+    csColoredVertices* vertices = GetCsVertices ();
+    bbox_2d.StartBoundingBox((*vertices)[0].pos);
+    for (i = 0 ; i < vertices->GetSize () ; i++)
     {
-      csSprite2DVertex& v = vertices[i];
+      csSprite2DVertex& v = (*vertices)[i];
       bbox_2d.AddBoundingVertexSmart(v.pos);
       if (!lighting)
       {
         // If there is no lighting then we need to copy the color_init
         // array to color.
-        v.color = vertices[i].color_init;
+        v.color = (*vertices)[i].color_init;
         v.color.Clamp (2, 2, 2);
       }
       float sqdist = v.pos.x*v.pos.x + v.pos.y*v.pos.y;
@@ -109,7 +120,7 @@ void csSprite2DMeshObject::SetupObject ()
 static csVector3 cam;
 
 void csSprite2DMeshObject::UpdateLighting (
-    const csArray<iLightSectorInfluence*>& lights,
+    const csSafeCopyArray<csLightInfluence>& lights,
     const csVector3& pos)
 {
   if (!lighting) return;
@@ -128,9 +139,12 @@ void csSprite2DMeshObject::UpdateLighting (
   int num_lights = (int)lights.GetSize ();
   for (i = 0; i < num_lights; i++)
   {
-    iLight* li = lights[i]->GetLight ();
+    iLight* li = lights[i].light;
+    if (!li)
+      continue;
+
     csColor light_color = li->GetColor ()
-    	* (256. / CS_NORMAL_LIGHT_LEVEL);
+      * (256. / CS_NORMAL_LIGHT_LEVEL);
     float sq_light_radius = csSquare (li->GetCutoffDistance ());
     // Compute light position.
     csVector3 wor_light_pos = li->GetMovable ()->GetFullPosition ();
@@ -138,22 +152,22 @@ void csSprite2DMeshObject::UpdateLighting (
       csSquaredDist::PointPoint (wor_light_pos, pos);
     if (wor_sq_dist >= sq_light_radius) continue;
     float wor_dist = csQsqrt (wor_sq_dist);
-    float cosinus = 1.;
+    float cosinus = 1.0f;
     cosinus /= wor_dist;
     light_color *= cosinus * li->GetBrightnessAtDistance (wor_dist);
     color += light_color;
   }
-  for (size_t j = 0 ; j < vertices.GetSize () ; j++)
+  csColoredVertices* vertices = GetCsVertices ();
+  for (size_t j = 0 ; j < vertices->GetSize () ; j++)
   {
-    vertices[j].color = vertices[j].color_init + color;
-    vertices[j].color.Clamp (2, 2, 2);
+    (*vertices)[j].color = (*vertices)[j].color_init + color;
+    (*vertices)[j].color.Clamp (2, 2, 2);
   }
   colors_dirty = true;
-
 }
 
 void csSprite2DMeshObject::UpdateLighting (
-    const csArray<iLightSectorInfluence*>& lights,
+    const csSafeCopyArray<csLightInfluence>& lights,
     iMovable* movable, csVector3 offset)
 {
   if (!lighting) return;
@@ -190,9 +204,13 @@ csRenderMesh** csSprite2DMeshObject::GetRenderMeshes (int &n,
 
   if (factory->light_mgr)
   {
-    const csArray<iLightSectorInfluence*>& relevant_lights = factory->light_mgr
-    	->GetRelevantLights (logparent, -1, false);
-    UpdateLighting (relevant_lights, movable, offset);
+    csSafeCopyArray<csLightInfluence> lightInfluences;
+    scfArrayWrap<iLightInfluenceArray, csSafeCopyArray<csLightInfluence> > 
+      relevantLights (lightInfluences); //Yes, know, its on the stack...
+
+    factory->light_mgr->GetRelevantLights (logparent, &relevantLights, -1);
+
+    UpdateLighting (lightInfluences, movable, offset);
   }
 
   csReversibleTransform temp = camera->GetTransform ();
@@ -216,7 +234,7 @@ csRenderMesh** csSprite2DMeshObject::GetRenderMeshes (int &n,
     rm->variablecontext = svcontext;
     rm->geometryInstance = this;
   }
-  
+
   rm->material = material;//Moved this statement out of the above 'if'
     //to make the change of the material possible at any time
     //(thru a call to either iSprite2DState::SetMaterialWrapper () or
@@ -234,8 +252,8 @@ csRenderMesh** csSprite2DMeshObject::GetRenderMeshes (int &n,
   rm->worldspace_origin = movable->GetFullPosition ();
 
   rm->object2world = tr_o2c.GetInverse () * camera->GetTransform ();
-  rm->indexend = (uint)vertices.GetSize ();
-
+  rm->bbox = GetObjectBoundingBox();
+  rm->indexend = (uint)GetCsVertices ()->GetSize ();
 
   n = 1; 
   return &rm; 
@@ -244,23 +262,24 @@ csRenderMesh** csSprite2DMeshObject::GetRenderMeshes (int &n,
 void csSprite2DMeshObject::PreGetBuffer (csRenderBufferHolder* holder, csRenderBufferName buffer)
 {
   if (!holder) return;
+  csColoredVertices* vertices = GetCsVertices ();
 
   if (buffer == CS_BUFFER_INDEX)
   {
-    size_t indexSize = vertices.GetSize ();
+    size_t indexSize = vertices->GetSize ();
     if (!index_buffer.IsValid() || 
       (indicesSize != indexSize))
     {
       index_buffer = csRenderBuffer::CreateIndexRenderBuffer (
 	indexSize, CS_BUF_DYNAMIC, 
-	CS_BUFCOMP_UNSIGNED_INT, 0, vertices.GetSize () - 1);
+	CS_BUFCOMP_UNSIGNED_INT, 0, vertices->GetSize () - 1);
       
       holder->SetRenderBuffer (CS_BUFFER_INDEX, index_buffer);
 
       csRenderBufferLock<uint> indexLock (index_buffer);
       uint* ptr = indexLock;
 
-      for (size_t i = 0; i < vertices.GetSize (); i++)
+      for (size_t i = 0; i < vertices->GetSize (); i++)
       {
 	*ptr++ = (uint)i;
       }
@@ -274,7 +293,7 @@ void csSprite2DMeshObject::PreGetBuffer (csRenderBufferHolder* holder, csRenderB
       int texels_count;
       const csVector2 *uvani_uv = 0;
       if (!uvani)
-	texels_count = (int)vertices.GetSize ();
+	texels_count = (int)vertices->GetSize ();
       else
 	uvani_uv = uvani->GetVertices (texels_count);
 	  
@@ -294,8 +313,8 @@ void csSprite2DMeshObject::PreGetBuffer (csRenderBufferHolder* holder, csRenderB
 	csVector2& v = texelLock[i];
 	if (!uvani)
 	{
-	  v.x = vertices[i].u;
-	  v.y = vertices[i].v;
+	  v.x = (*vertices)[i].u;
+	  v.y = (*vertices)[i].v;
 	}
 	else
 	{
@@ -310,7 +329,7 @@ void csSprite2DMeshObject::PreGetBuffer (csRenderBufferHolder* holder, csRenderB
   {
     if (colors_dirty)
     {
-      size_t color_size = vertices.GetSize ();
+      size_t color_size = vertices->GetSize ();
       if (!color_buffer.IsValid() || (color_buffer->GetSize() 
 	!= color_size * sizeof(float) * 2))
       {
@@ -322,9 +341,9 @@ void csSprite2DMeshObject::PreGetBuffer (csRenderBufferHolder* holder, csRenderB
 
       csRenderBufferLock<csColor> colorLock (color_buffer);
 
-      for (size_t i = 0; i < vertices.GetSize (); i++)
+      for (size_t i = 0; i < vertices->GetSize (); i++)
       {
-	colorLock[i] = vertices[i].color;
+	colorLock[i] = (*vertices)[i].color;
       }
       colors_dirty = false;
     }
@@ -333,7 +352,7 @@ void csSprite2DMeshObject::PreGetBuffer (csRenderBufferHolder* holder, csRenderB
   {
     if (vertices_dirty)
     {
-      size_t vertices_size = vertices.GetSize ();
+      size_t vertices_size = vertices->GetSize ();
       if (!vertex_buffer.IsValid() || (vertex_buffer->GetSize() 
 	!= vertices_size * sizeof(float) * 3))
       {
@@ -345,9 +364,9 @@ void csSprite2DMeshObject::PreGetBuffer (csRenderBufferHolder* holder, csRenderB
 
       csRenderBufferLock<csVector3> vertexLock (vertex_buffer);
 
-      for (size_t i = 0; i < vertices.GetSize (); i++)
+      for (size_t i = 0; i < vertices->GetSize (); i++)
       {
-	vertexLock[i].Set (vertices[i].pos.x, vertices[i].pos.y, 0.0f);
+	vertexLock[i].Set ((*vertices)[i].pos.x, (*vertices)[i].pos.y, 0.0f);
       }
       vertices_dirty = false;
     }
@@ -376,20 +395,21 @@ void csSprite2DMeshObject::CreateRegularVertices (int n, bool setuv)
 {
   double angle_inc = TWO_PI / n;
   double angle = 0.0;
-  vertices.SetSize (n);
+  csColoredVertices* vertices = GetCsVertices ();
+  vertices->SetSize (n);
   size_t i;
-  for (i = 0; i < vertices.GetSize (); i++, angle += angle_inc)
+  for (i = 0; i < vertices->GetSize (); i++, angle += angle_inc)
   {
-    vertices [i].pos.y = cos (angle);
-    vertices [i].pos.x = sin (angle);
+    (*vertices) [i].pos.y = cos (angle);
+    (*vertices) [i].pos.x = sin (angle);
     if (setuv)
     {
       // reuse sin/cos values and scale to [0..1]
-      vertices [i].u = vertices [i].pos.x / 2.0f + 0.5f;
-      vertices [i].v = vertices [i].pos.y / 2.0f + 0.5f;
+      (*vertices) [i].u = (*vertices) [i].pos.x / 2.0f + 0.5f;
+      (*vertices) [i].v = (*vertices) [i].pos.y / 2.0f + 0.5f;
     }
-    vertices [i].color.Set (1, 1, 1);
-    vertices [i].color_init.Set (1, 1, 1);
+    (*vertices) [i].color.Set (1, 1, 1);
+    (*vertices) [i].color_init.Set (1, 1, 1);
   }
 
   vertices_dirty = true;
@@ -411,7 +431,7 @@ void csSprite2DMeshObject::NextFrame (csTicks current_time,
 }
 
 void csSprite2DMeshObject::UpdateLighting (
-	const csArray<iLightSectorInfluence*>& lights,
+	const csSafeCopyArray<csLightInfluence>& lights,
 	const csReversibleTransform& transform)
 {
   csVector3 new_pos = transform.This2Other (part_pos);
@@ -563,6 +583,16 @@ iSprite2DUVAnimation *csSprite2DMeshObject::GetUVAnimation (
 }
 
 
+void csSprite2DMeshObject::EnsureVertexCopy ()
+{
+  if (!scfVertices.IsValid())
+  {
+    scfVertices.AttachNew (new scfArrayWrap<iColoredVertices, 
+      csColoredVertices> (vertices));
+    vertices = *(factory->GetCsVertices());
+  }
+}
+
 void csSprite2DMeshObject::uvAnimationControl::Advance (csTicks current_time)
 {
   int oldframeindex = frameindex;
@@ -575,58 +605,63 @@ void csSprite2DMeshObject::uvAnimationControl::Advance (csTicks current_time)
       counter = 0;
       frameindex++;
       if (frameindex == framecount)
-	if (loop)
-	  frameindex = 0;
-	else
-	{
-	  frameindex = framecount-1;
-	  halted = true;
-	}
-    }
-  }
-  else
-    if (style > 0)
-    { // skip to next frame every <style> millisecond
-      if (last_time == 0)
-	last_time = current_time;
-      counter += (current_time - last_time);
-      last_time = current_time;
-      while (counter > style)
       {
-	counter -= style;
-	frameindex++;
-	if (frameindex == framecount)
-	  if (loop)
-	    frameindex = 0;
-	  else
+	    if (loop)
+	      frameindex = 0;
+	    else
 	    {
 	      frameindex = framecount-1;
 	      halted = true;
 	    }
       }
     }
-    else
-    { // style == 0 -> use time indices attached to the frames
-      if (last_time == 0)
-	last_time = current_time;
-      while (frame->GetDuration () + last_time < current_time)
+  }
+  else if (style > 0)
+  { // skip to next frame every <style> millisecond
+    if (last_time == 0)
+	  last_time = current_time;
+    counter += (current_time - last_time);
+    last_time = current_time;
+    while (counter > style)
+    {
+	  counter -= style;
+	  frameindex++;
+	  if (frameindex == framecount)
       {
-	frameindex++;
-	if (frameindex == framecount)
-	  if (loop)
-	  {
-	    frameindex = 0;
-	  }
-	  else
-	  {
-	    frameindex = framecount-1;
-	    halted = true;
-	    break;
-	  }
-	last_time += frame->GetDuration ();
-	frame = ani->GetFrame (frameindex);
+	    if (loop)
+	      frameindex = 0;
+	    else
+	    {
+	      frameindex = framecount-1;
+	      halted = true;
+	    }
       }
     }
+  }
+  else
+  { // style == 0 -> use time indices attached to the frames
+    if (last_time == 0)
+	  last_time = current_time;
+    while (frame->GetDuration () + last_time < current_time)
+    {
+	  frameindex++;
+	  if (frameindex == framecount)
+      {
+	    if (loop)
+	    {
+	      frameindex = 0;
+	    }
+	    else
+	    {
+	      frameindex = framecount-1;
+	      halted = true;
+	      break;
+        }
+      }
+	}
+    last_time += frame->GetDuration ();
+    frame = ani->GetFrame (frameindex);
+  }
 
   if (oldframeindex != frameindex)
     frame = ani->GetFrame (frameindex);
@@ -676,20 +711,32 @@ bool csSprite2DMeshObject::HitBeamOutline(const csVector3& start,
   if (sqr < SMALL_EPSILON) return false; // Too close, Cannot intersect
   float dist;
   csIntersect3::SegmentPlane(start, end, pl, 0, isect, dist);
-  if (pr) *pr = dist;
+  if (pr) { *pr = dist; }
   csMatrix3 o2t;
   CheckBeam (start, pl, sqr, o2t);
   csVector3 r = o2t * isect;
-  int trail, len = (int)vertices.GetSize ();
+  csColoredVertices* vertices = GetCsVertices ();
+  int trail, len = (int)vertices->GetSize ();
   trail = len - 1;
   csVector2 isec(r.x, r.y);
   int i;
   for (i = 0; i < len; trail = i++)
-    if (csMath2::WhichSide2D(isec, vertices[trail].pos, vertices[i].pos) > 0)
+  {
+    if (csMath2::WhichSide2D(isec, (*vertices)[trail].pos, (*vertices)[i].pos) > 0)
       return false;
+  }
   return true;
 }
 
+bool csSprite2DMeshObject::HitBeamObject (const csVector3& start, const csVector3& end,
+					  csVector3& isect, float* pr, int* polygon_idx,
+					  iMaterialWrapper** material)
+{
+  if (material) *material = csSprite2DMeshObject::material;
+  if (polygon_idx) *polygon_idx = -1;
+  return HitBeamOutline(start, end, isect, pr);
+}
+
 //----------------------------------------------------------------------
 
 csSprite2DMeshObjectFactory::csSprite2DMeshObjectFactory (iMeshObjectType* pParent,
@@ -699,8 +746,10 @@ csSprite2DMeshObjectFactory::csSprite2DMeshObjectFactory (iMeshObjectType* pPare
 {
   light_mgr = csQueryRegistry<iLightManager> (object_reg);
   g3d = csQueryRegistry<iGraphics3D> (object_reg);
-  csRef<iStringSet> strings = csQueryRegistryTagInterface<iStringSet> (
-    object_reg, "crystalspace.shared.stringset");
+
+  scfVertices.AttachNew (new scfArrayWrap<iColoredVertices, 
+    csColoredVertices> (vertices));
+  ax = -10;
 }
 
 csSprite2DMeshObjectFactory::~csSprite2DMeshObjectFactory ()
