@@ -35,10 +35,11 @@
 #include "iutil/object.h"
 #include "iutil/objreg.h"
 #include "iutil/plugin.h"
+#include "iutil/stringarray.h"
 
 #include "loader.h"
 
-CS_IMPLEMENT_PLUGIN
+
 
 CS_PLUGIN_NAMESPACE_BEGIN(Terrain2Loader)
 {
@@ -391,15 +392,48 @@ bool csTerrain2FactoryLoader::ParseCell (iDocumentNode *node,
       {
         const char* matname = child->GetContentsValue ();
         csRef<iMaterialWrapper> baseMaterial = ldr_ctx->FindMaterial (matname);
-        
+
         if (!baseMaterial)
         {
           synldr->ReportError (
             "crystalspace.terrain.object.loader.basematerial",
-            child, "Couldn't find material '%s'!", matname);
+            child, "Couldn't find material %s!", CS::Quote::Single (matname));
           return false;
         }
-	cell->SetBaseMaterial (baseMaterial);
+
+        cell->SetBaseMaterial (baseMaterial);
+        break;
+      }
+    case XMLTOKEN_SPLATBASEMATERIAL:
+      {
+        const char* matname = child->GetContentsValue ();
+        csRef<iMaterialWrapper> baseMaterial = ldr_ctx->FindMaterial (matname);
+
+        if (!baseMaterial)
+        {
+          synldr->ReportError (
+            "crystalspace.terrain.object.loader.basematerial",
+            child, "Couldn't find material %s!", CS::Quote::Single (matname));
+          return false;
+        }
+
+        cell->SetSplatBaseMaterial (baseMaterial);
+        break;
+      }
+    case XMLTOKEN_ALPHASPLATMATERIAL:
+      {
+        const char* matname = child->GetContentsValue ();
+        csRef<iMaterialWrapper> alphaSplatMaterial = ldr_ctx->FindMaterial (matname);
+
+        if (!alphaSplatMaterial)
+        {
+          synldr->ReportError (
+            "crystalspace.terrain.object.loader.basematerial",
+            child, "Couldn't find material %s!", CS::Quote::Single (matname));
+          return false;
+        }
+
+        cell->SetAlphaSplatMaterial (alphaSplatMaterial);
         break;
       }
     case XMLTOKEN_MATERIALMAPPERSISTENT:
@@ -482,12 +516,14 @@ csPtr<iBase> csTerrain2ObjectLoader::Parse (iDocumentNode* node,
         const char* factname = child->GetContentsValue ();
         csRef<iMeshFactoryWrapper> fact = ldr_context->FindMeshFactory (
           factname);
-        if (!fact)
+
+        if(!fact)
         {
           synldr->ReportError ("crystalspace.terrain.object.loader",
-            child, "Couldn't find factory '%s'!", factname);
+            child, "Couldn't find factory %s!", CS::Quote::Single (factname));
           return 0;
         }
+
         mesh = fact->GetMeshObjectFactory ()->NewInstance ();
         terrain = scfQueryInterface<iTerrainSystem> (mesh);
             
@@ -495,8 +531,8 @@ csPtr<iBase> csTerrain2ObjectLoader::Parse (iDocumentNode* node,
         {
           synldr->ReportError (
                     "crystalspace.terrain.parse.badfactory", child,
-                    "Factory '%s' doesn't appear to be a terrain factory!",
-                    factname);
+                    "Factory %s doesn't appear to be a terrain factory!",
+                    CS::Quote::Single (factname));
           return 0;
         }
             
@@ -523,7 +559,7 @@ csPtr<iBase> csTerrain2ObjectLoader::Parse (iDocumentNode* node,
               {
                 synldr->ReportError (
                   "crystalspace.terrain.object.loader.materialpalette",
-                  child, "Couldn't find material '%s'!", matname);
+                  child, "Couldn't find material %s!", CS::Quote::Single (matname));
                 return 0;
               }
               pal.Push (mat);
@@ -605,7 +641,7 @@ bool csTerrain2ObjectLoader::ParseCell (iDocumentNode* node,
   {
     synldr->ReportError (
       "crystalspace.terrain.object.loader.cell",
-      node, "Invalid cell name '%s'", cellName);
+      node, "Invalid cell name %s", CS::Quote::Single (cellName));
     return false;
   }
 
