@@ -349,6 +349,7 @@ public:
   virtual void Step (float stepsize);
 
   virtual csPtr<iRigidBody> CreateBody ();
+  virtual void AddBody (iRigidBody* body) { /* TODO */ }
   virtual void RemoveBody (iRigidBody* body);
   virtual iRigidBody *FindBody (const char *name);
   iRigidBody *GetBody (unsigned int index);
@@ -358,6 +359,7 @@ public:
   virtual void RemoveGroup (iBodyGroup *group);
 
   virtual csPtr<iJoint> CreateJoint ();
+  virtual void AddJoint (iJoint* joint) { /* TODO */ }
   virtual csPtr<iODEBallJoint> CreateBallJoint ();
   virtual csPtr<iODEHingeJoint> CreateHingeJoint ();
   virtual csPtr<iODEHinge2Joint> CreateHinge2Joint ();
@@ -387,6 +389,9 @@ public:
   virtual bool AttachColliderCylinder (float length, float radius,
         const csOrthoTransform& trans, float friction, float elasticity,
 	float softness);
+  virtual bool AttachColliderCapsule (float length, float radius,
+        const csOrthoTransform& trans, float friction, float elasticity,
+	float softness);
   virtual bool AttachColliderBox (const csVector3 &size,
         const csOrthoTransform& trans, float friction, float elasticity,
 	float softness);
@@ -457,10 +462,8 @@ public:
   
   bool CreateSphereGeometry (const csSphere& sphere);
   bool CreatePlaneGeometry (const csPlane3& plane);
-  bool CreateConvexMeshGeometry (iMeshWrapper *mesh)
-  {
-    return CreateMeshGeometry (mesh);
-  }
+  inline bool CreateConvexMeshGeometry (iMeshWrapper *mesh)
+  { return CreateMeshGeometry (mesh); }
   bool CreateMeshGeometry (iMeshWrapper *mesh);
   bool CreateBoxGeometry (const csVector3& box_size);
   bool CreateCapsuleGeometry (float length, float radius); 
@@ -469,6 +472,11 @@ public:
   bool GetSphereGeometry (csSphere& sphere);
   bool GetPlaneGeometry (csPlane3& box); 
   bool GetCylinderGeometry (float& length, float& radius); 
+  bool GetCapsuleGeometry (float& length, float& radius);
+  bool GetMeshGeometry (csVector3*& vertices, size_t& vertexCount,
+			int*& indices, size_t& triangleCount);
+  bool GetConvexMeshGeometry (csVector3*& vertices, size_t& vertexCount,
+			      int*& indices, size_t& triangleCount);
 
   void SetCollisionCallback (iDynamicsColliderCollisionCallback* cb);
   void Collision (csODECollider* other, const csVector3& pos,
@@ -505,8 +513,6 @@ public:
 
 private:
 
-  inline void CS2ODEMatrix (const csMatrix3& csmat, dMatrix3& odemat);
-  inline void ODE2CSMatrix (const dReal* odemat, csMatrix3& csmat);
   void MassUpdate ();
   void ClearContents ();
   void KillGeoms ();
@@ -555,6 +561,7 @@ public:
   csODERigidBody (csODEDynamicSystem* sys);
   virtual ~csODERigidBody ();
 
+  using iObject::GetID;
   inline dBodyID GetID() { return bodyID; }
 
   iObject* QueryObject () { return this; }
@@ -583,6 +590,9 @@ public:
   bool AttachColliderCylinder (float length, float radius,
         const csOrthoTransform& trans, float friction, float density,
         float elasticity, float softness);
+  bool AttachColliderCapsule (float length, float radius,
+        const csOrthoTransform& trans, float friction, float density,
+        float elasticity, float softness);
   bool AttachColliderBox (const csVector3 &size,
         const csOrthoTransform& trans, float friction, float density,
         float elasticity, float softness);
